#include "constants.fxh"

texture2D BackBuffer;
texture3D ColorChart;

sampler2D samplerBackBuffer = sampler_state
{
  Texture = <BackBuffer>;
  MinFilter = Point;
  MagFilter = Point;
  MipFilter = None;
  AddressU = Clamp;
  AddressV = Clamp;
};

sampler3D samplerColorChart = sampler_state
{
  Texture = <ColorChart>;
  MinFilter = Linear;
  MagFilter = Linear;
  MipFilter = None;
  AddressU = Clamp;
  AddressV = Clamp;
  AddressW = Clamp;
};

struct VS_INPUT
{
	float4 Pos				: POSITION;
	float2 Tex0				: TEXCOORD0;
};

struct vertexOutput
{
  float4 HPosition			: POSITION;
  float2 Tex0		    		: TEXCOORD0;
};

vertexOutput std_VS(const in VS_INPUT In)
{
  vertexOutput Out;
  Out.HPosition = In.Pos;
	Out.HPosition.xy += float2(-.5f, .5f) * F_ScreenSize.xy;
  Out.Tex0 = In.Tex0;
	Out.Tex0.y = 1.f - Out.Tex0.y;

  return Out;
}

#define COLOR_CHART_SIZE 16.h
#define fScale ((COLOR_CHART_SIZE-1.h) / COLOR_CHART_SIZE)
#define fShift (.5h / COLOR_CHART_SIZE)

half4 ColorGradingPS(const in vertexOutput IN) : COLOR0
{
	// sample original buffer
	half4 vSample = tex2D(samplerBackBuffer, IN.Tex0);
	half3 vTexCoords = saturate(vSample.rgb) * fScale + fShift;
	// transform color with color chart
	vSample.rgb = tex3D(samplerColorChart, vTexCoords).rgb;
	return vSample;
}

technique ColorGrading
{
	pass p0
  {
		AlphaBlendEnable = false;
		ZEnable = false;
		CullMode = None;
		VertexShader = compile VS_SHADER_TYPE std_VS();
		PixelShader  = compile PS_SHADER_TYPE ColorGradingPS();
  }
}
