#include "constants.fxh"

texture2D baseTex;
sampler2D baseMap = sampler_state 
{
    texture = <baseTex>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = POINT;
    MAGFILTER = POINT;
};

textureCUBE cmTex;
samplerCUBE cmMap = sampler_state 
{
    texture = <cmTex>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

texture2D paletteTex;
sampler2D paletteMap = sampler_state 
{
    texture = <paletteTex>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

struct VS_INPUT 
{
   float4 Pos : POSITION;
   float2 tc0 : TEXCOORD0;
};
struct VS_CM_INPUT 
{
   float4 Pos : POSITION;
   float3 tc0 : TEXCOORD0;
};
struct VS_INPUT_3D
{
   float3 Pos : POSITION;
   float2 tc0 : TEXCOORD0;
};

struct VS_OUTPUT 
{
   float4 Pos : POSITION;
   float2 tc0 : TEXCOORD0;
};
struct VS_CM_OUTPUT 
{
   float4 Pos : POSITION;
   float3 tc0 : TEXCOORD0;
};

VS_OUTPUT VS_Main(in VS_INPUT In)
{
  VS_OUTPUT Out;
  Out.Pos = float4(In.Pos.xyz, 1);
#ifdef OPENGL
  Out.Pos.xy = ((Out.Pos.xy+float2(0.5, 0.5)) * F_ScreenSize.xy) * 2.f - 1.f;
  //Out.Pos.y = -Out.Pos.y;
#endif  
  Out.tc0 = In.tc0;
  return Out;
}
VS_OUTPUT VS_Main2(in VS_INPUT In)
{
  VS_OUTPUT Out;
  Out.Pos = float4(In.Pos.xyz, 1);
#ifdef OPENGL
  Out.Pos.xy = ((Out.Pos.xy+float2(0.5, 0.5)) * F_ScreenSize.xy) * 2.f - 1.f;
  Out.Pos.y = -Out.Pos.y;
#endif  
  Out.tc0 = In.tc0;
  return Out;
}

half4 PS_ShowInstructions(in float2 tc0: TEXCOORD0) : COLOR0
{
  float4 vVal = tex2D(baseMap, tc0);
  //return vVal;
  float fVal = vVal.x;
  return tex2D(paletteMap, float2(0.25, fVal));
}

VS_OUTPUT VS_Show(in VS_INPUT_3D In)
{
	VS_OUTPUT Out;
	Out.Pos = float4(In.Pos, 1);
  //Out.Pos.xy *= F_ScreenSize.xy;
  //Out.Pos.xy = Out.Pos.xy * 2.f - 1.f;

  Out.tc0.xy = In.tc0.xy;
	return Out;
}

half4 PS_ShowTexture(VS_OUTPUT In) : COLOR0
{
  return tex2D(paletteMap, float2(In.tc0.x, In.tc0.y));
}

half4 PS_GradInstructions(in float2 tc0: TEXCOORD0) : COLOR0
{
  return tex2D(paletteMap, float2(0.25, tc0.y));
}

VS_CM_OUTPUT VS_CMShow(in VS_CM_INPUT In)
{
  VS_CM_OUTPUT Out;
  Out.Pos = In.Pos;
  Out.Pos.xy *= F_ScreenSize.xy;
  Out.Pos.xy = Out.Pos.xy * 2.f - 1.f;

  Out.tc0.xyz = In.tc0.xyz;
  return Out;
}

half4 PS_CMShowTexture(VS_CM_OUTPUT In) : COLOR0
{
  half4 cOut = 0;
  cOut.xyz = In.tc0.xyz; 
  //return cOut;
  return texCUBE(cmMap, In.tc0.xyz);
}

technique ShowInstructions
{
  pass p0 
  {		
    VertexShader = compile VS_SHADER_TYPE VS_Main();
    PixelShader = compile PS_SHADER_TYPE PS_ShowInstructions();
  }
}

technique GradInstructions
{
  pass p0 
  {		
    VertexShader = compile VS_SHADER_TYPE VS_Main2();
    PixelShader = compile PS_SHADER_TYPE PS_GradInstructions();
  }
}

technique ShowTexture
{
  pass p0 
  {		
    VertexShader = compile VS_SHADER_TYPE VS_Show();
    PixelShader = compile PS_SHADER_TYPE PS_ShowTexture();
  }
}

technique ShowCMTexture
{
  pass p0 
  {		
    VertexShader = compile VS_SHADER_TYPE VS_CMShow();
    PixelShader = compile PS_SHADER_TYPE PS_CMShowTexture();
  }
}
