// General deferred declaration file

#ifndef _DEFERRED_DECLARED__
#define _DEFERRED_DECLARED__


// Common functions
half4 EXPAND(half4 a)
{
  return a * 2 - 1;
}
half3 EXPAND(half3 a)
{
  return a * 2 - 1;
}

//=======================================================================================================================

void ReconstructNormalFromNormalsBuffer(inout half4 vNormal, bool bNormalize)
{
  if (bNormalize)
    vNormal.xyz = normalize(vNormal.xyz);
}

half4 GetWorldSpaceNormals(half4 vNormals, bool bNormalize)
{
  // expand from unsigned
  vNormals.xyz = EXPAND(vNormals.xyz); 

  ReconstructNormalFromNormalsBuffer(vNormals, bNormalize);

  return vNormals;
}

half4 GetWorldSpaceNormals(sampler2D smpNormals, float2 tc, bool bNormalize)
{
  half4 vNormals = tex2D(smpNormals, tc.xy);
  return GetWorldSpaceNormals(vNormals, bNormalize);
}


half4 EncodeLightBuffer(in half4 color)
{
#ifdef OPENGL
  return color; //EncodeRGBK(color, SCENE_HDR_MULTIPLIER, false);
#else
  return color;
#endif
}
void EncodeLightBufferPair(inout half4 diffuse, inout half4 specular)
{
#ifdef OPENGL
  //EncodeRGBKPair(diffuse, specular, SCENE_HDR_MULTIPLIER, false);
#endif
}

//////////////////////////////////////////////////////////////////////////

float GetLinearDepth(float fDevDepth)
{
  return fDevDepth;
} 

float GetLinearDepth(sampler2D depthSampler, float2 ScreenTC)
{
  float fDepth = tex2D(depthSampler, ScreenTC.xy).x;
  return GetLinearDepth(fDepth);
}

float3 GetWorldPos(float fDepth, float3 vCamVec)
{
  float3 cameraToWorldPos = fDepth * vCamVec;
  return C_Eye.xyz + cameraToWorldPos;
}
half GetAttenuation(half3 L, half fInvRadius)
{
  half fFallOff = 1.h - saturate(dot(L, L) * fInvRadius);

  return fFallOff;
}

//=================================================================================================

#define ONE_OVER_PI 0.31831h
#define ONE_OVER_TWO_PI 0.159155h

half PhongDef(half3 R, half3 L, half Exp)
{	
  half fNormFactor = Exp * ONE_OVER_TWO_PI + ONE_OVER_TWO_PI;
  return fNormFactor *  pow(saturate(dot(L, R)), Exp);			
}

half PhongDef(half3 N, half3 V, half3 L, half Exp)
{
  half3 R = reflect(-V, N);
  return PhongDef(R, L, Exp);
}

half BlinnDef(half3 N, half3 V, half3 L, half Exp)
{
  half fNormFactor = Exp * ONE_OVER_TWO_PI + ONE_OVER_PI;
  half3 H = normalize(V + L);														
  return fNormFactor * pow(saturate(dot(N, H)), Exp);		
}

half SimpleBRDF(half3 N, half3 V, half3 L, half Exp)
{
  return BlinnDef(N, V, L, Exp);
}


//=============================================================================================================================================================
// Shadows support

uniform float4 irreg_kernel_2d[8];

void shadow_sample(sampler2D depthMap, float3 p, out half shadow)
{
  #ifdef RT_HW_PCF_COMPARE
    shadow	= tex2Dproj(depthMap, float4 (p.xyz,1)).r;
  #else
    shadow = tex2D(depthMap, p.xy).r;
  #endif
}

void DoubleSampleRotated(sampler2D depthMap, float4 p, float4 rotMatr, float4 kernel, out half2 result)
{
    // Rotate tap for this pixel location
    float4 rotatedOff;

    rotatedOff = rotMatr.xyzw * kernel.xxww + rotMatr.zwxy * kernel.yyzz;

    float4 fetchPos = p.xyxy + rotatedOff;

    #ifdef RT_HW_PCF_COMPARE
      result.x = tex2Dproj(depthMap, p + (rotatedOff.xyzw*float4(1,1,0,0))).r;
      result.y = tex2Dproj(depthMap, p + (rotatedOff.zwxy*float4(1,1,0,0))).r;
    #else
      shadow_sample(depthMap, float3(fetchPos.xy, 0), result.x);
      shadow_sample(depthMap, float3(fetchPos.zw, 0), result.y);
    #endif
}

void irregular_filter_smpl(sampler2D depthMap, sampler2D sRotations, float4 p, float2 randDirTC, float2 kernelRadius, out half shadowTest, int nChunk = 0)
{
  #define SHADOW_SAMPLES_NUM 8

  half kernelOffset = 0;
  half kernelSize = SHADOW_SAMPLES_NUM/2;

  half P_Z = p.z;

  float4 p0 = float4(p.xyz, 1.0f);

  half2 rotScale = kernelRadius.y * 1000.0f;

  shadowTest = 0;

  #define KERNEL_STEP_SIZE 2

  half2 rotSample = tex2D(sRotations, randDirTC.xy * rotScale.xy).xy;

  rotSample.xy *= (kernelRadius.xy * float2(1/512.0f, 1/512.0f));

  float4 rot = float4(rotSample.x, -rotSample.y, rotSample.y, rotSample.x);

  for (int i=kernelOffset; i<kernelSize; i+=KERNEL_STEP_SIZE)
  {
    half4 sampleDepth;

    DoubleSampleRotated(depthMap, p0, rot, irreg_kernel_2d[i+0], sampleDepth.xy);
    DoubleSampleRotated(depthMap, p0, rot, irreg_kernel_2d[i+1], sampleDepth.zw);

    // Determine whether tap is in shadow                
    half4 InShadow = (P_Z.xxxx < sampleDepth);
    half fInvSamplNum = (1.0 / SHADOW_SAMPLES_NUM);
    shadowTest += dot(InShadow, fInvSamplNum.xxxx);
  }
}

#endif // __DEFERRED_DECLARED__