/*
  Deferred shading
*/

#include "constants.fxh"
#include "Deferred.fxh"

sampler samplerN = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler samplerZ = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler samplerShadow = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler samplerMap = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler samplerLight = sampler_state 
{
  Texture = <Texture0>;
  MinFilter = Linear;
  MagFilter = Linear;
  MipFilter = Linear;
	AddressU = Border;
	AddressV = Border;
#ifdef OPENGL
	BorderColor = { 0,0,0,0 };
#else	
	BorderColor = 0;
#endif	
};
sampler samplerLightCube = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
    AddressW = Clamp;
};
/*sampler samplerShadow = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};*/
sampler samplerNoise = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler samplerShape = sampler_state 
{
    texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct vs_in
{
  float3 Pos  : POSITION;
  float2 Tex0 : TEXCOORD0;
};


struct vs_out
{
  float4 Pos  : POSITION;
  float2 Tex0 : TEXCOORD0;
};

struct vtxOutWPOSOnly
{
  float4 Pos  : POSITION;
};
struct vtxOutWPOS
{
  float4 Pos         : POSITION;
  float3 baseTC      : TEXCOORD0;
  float3 CamVec      : TEXCOORD1;
};

struct fragInWPOS
{
  float4 WPos        : VPOS;
#ifndef RT_ENVMAP       // !RT_LIGHT_VOLUMES
  float4 baseTC      : TEXCOORD0;
  float4 CamVec      : TEXCOORD1;
#endif

};

struct vtxInLightPrePass
{
  float3 Pos  : POSITION;
  float3 Normal : NORMAL;
};
struct vtxInWPOS
{
  float4 Pos    : POSITION;
  float2 Tex0   : TEXCOORD0;
  float3 CamVec : TEXCOORD1;
};

struct vs_in_def
{
  float3 Pos  : POSITION;
  float2 Tex0 : TEXCOORD0;
  float3 CamVec : TEXCOORD1;
};


struct vs_out_def
{
  float4 Pos : POSITION;
  float3 Tex0 : TEXCOORD0;
  float3 CamVec : TEXCOORD1;
};

struct pixoutMRT
{
  float4 Diffuse : COLOR0;
  float4 Specular : COLOR1;
};

struct pixout
{
  half4 Color : COLOR0;
};

uniform float4x4 mCamera; 
uniform float4x4 mViewProj; 
uniform float4x4 mTexGen; 
uniform float4x4 mLightShadowProj;
uniform float4x4 mLightProj;
uniform float4 GeneralParams; 
uniform float4 ScreenScale; 
uniform float4 LightPos; 
uniform float4 LightDir; 
uniform float4 cLightDiffuse; 
uniform float4 cAmbient; 
uniform float4 cAmbGround;
uniform float4 vAmbHeightParams;
uniform float4 ShadowMask;

uniform float4 WBasisX;
uniform float4 WBasisY;
uniform float4 WBasisZ;

float3 HPosToScreenTC(float4 HPos)
{
  float3 ScreenCoordProj = HPos.xyw;
  ScreenCoordProj.xy = ScreenCoordProj.xy * float2(.5f, -.5f) + (float2(.5f, .5f) + float2(.5f, .5f) * F_ScreenSize.xy) * HPos.w;

  return ScreenCoordProj;
}

vs_out VSShowMap(vs_in In)
{
  vs_out Out;
  Out.Pos = float4(In.Pos, 1);
#ifdef OPENGL
  Out.Pos.xy = ((Out.Pos.xy+float2(0.5, 0.5)) * F_ScreenSize.xy) * 2.f - 1.f;
#endif
  Out.Tex0 = In.Tex0;
  return Out;
}

vs_out_def VSMain(vs_in_def In)
{
  vs_out_def Out;
  Out.Pos = float4(In.Pos, 1);
#ifdef OPENGL
  Out.Pos.xy = ((Out.Pos.xy+float2(0.5, 0.5)) * F_ScreenSize.xy) * 2.f - 1.f;
#endif
  Out.Tex0 = HPosToScreenTC(Out.Pos);
  Out.CamVec = In.CamVec;
  return Out;
}

vtxOutWPOSOnly LightVolumeVS(vtxInLightPrePass IN)
{
  vtxOutWPOSOnly OUT = (vtxOutWPOSOnly)0; 

  float4 vPos = float4(IN.Pos.xyz, 1);
  OUT.Pos = mul(vPos, mViewProj);

  return OUT;
}

vtxOutWPOSOnly StencilCullVS(vtxInLightPrePass IN)
{
  vtxOutWPOSOnly OUT = (vtxOutWPOSOnly)0; 

  float4 vPos = float4(IN.Pos.xyz, 1);
  OUT.Pos = mul(vPos, mViewProj);
  //OUT.Pos.w = 1;

  return OUT;
}

vtxOutWPOSOnly StencilFrustumCullVS(vtxInLightPrePass IN)
{
  vtxOutWPOSOnly OUT = (vtxOutWPOSOnly)0; 

  float4 vPos = float4(IN.Pos.xyz, 1);
  float4 Pos = mul(vPos, mTexGen);

	Pos /= Pos.w;
	Pos.w = 1.0f;

  //float4 LightPos = float4(-5, 5, -5, 50);
	float3 vLightDir = normalize(Pos.xyz - LightPos.xyz);
	vLightDir *= LightPos.w;
	Pos.xyz = lerp(Pos.xyz, (LightPos.xyz + vLightDir), IN.Normal.z); 

	OUT.Pos = mul(Pos, mViewProj);

  return OUT;
}

vtxOutWPOS LightPassVS(vtxInWPOS IN)
{
  vtxOutWPOS OUT = (vtxOutWPOS)0; 

  float4 vPos = IN.Pos;

  OUT.Pos = vPos;
#ifdef OPENGL
  OUT.Pos.xy = ((OUT.Pos.xy+float2(0.5, 0.5)) * F_ScreenSize.xy) * 2.f - 1.f;
#endif

  OUT.baseTC = HPosToScreenTC(OUT.Pos);
  OUT.CamVec.xyz = IN.CamVec.xyz;

  return OUT;
}

//======================================================================================================

half GetDeferredLightAttenuation(float3 L, half fInvRadius)
{
  half fAttenuation = GetAttenuation(L, fInvRadius) ; 

#if RT_SHAPEDEFORM
    fAttenuation = tex2D(samplerShape, fAttenuation);
#else
  #if !VERTEX_COLOR            // !RT_SPOTLIGHT
    //fAttenuation *= fAttenuation;
  #endif
#endif

  return fAttenuation;
}

float4 CalcHomogeneousPos(float SceneDepth, float2 WPos)
{
  float4 HPos = WBasisZ.xyzw + (WBasisX.xyzw*WPos.x) + (WBasisY.xyzw*WPos.y);
  HPos.xyzw *= SceneDepth.xxxx;

  return HPos;
}

float4 PSShowMap(vs_out In) : COLOR
{
  float4 Col = tex2D(samplerMap, In.Tex0);
#ifdef TEST
  Col = Col * 2.0f - 1.0f;
#else
  //Col.xyz = float3(0.5,0.5,1);
#endif
  
  return Col;
}

pixoutMRT  PSAmbientOutdoor(vs_out_def In)
{
  pixoutMRT OUT = (pixoutMRT)0;
  
  half4 vNormal = GetWorldSpaceNormals(samplerN, In.Tex0.xy, true);
  half3 vAmbient = cAmbient.rgb;
  vAmbient *= (vNormal.y*0.25h+0.75h);
  OUT.Diffuse.rgb = vAmbient;
  
  // Height based ambient
#ifdef TEST
  float fDepth = GetLinearDepth(samplerZ, In.Tex0.xy);
  half3 vPosition = GetWorldPos(fDepth, In.CamVec);
  half3 vPositionWS = vPosition + C_Eye.xyz;

  half3 cGroundAmbient = cAmbGround.rgb;
  cGroundAmbient *= (vNormal.y*0.25h+0.75h); // 2 inst
  OUT.Diffuse.rgb = lerp(cGroundAmbient, OUT.Diffuse.rgb, saturate((vPositionWS.z - vAmbHeightParams.x) * vAmbHeightParams.z));
#endif

  OUT.Diffuse = EncodeLightBuffer(OUT.Diffuse);
  
  return OUT;
}

pixoutMRT LightPassPS(fragInWPOS IN)
{
  pixoutMRT OUT = (pixoutMRT) 0;

#if RT_DEBUG == 1
  OUT.Diffuse += 0.1;
  return OUT;
#endif

  float2 ScreenTC;
#ifdef RT_ENVMAP       // RT_LIGHT_VOLUMES
  ScreenTC = IN.WPos.xy * ScreenScale.xy + ScreenScale.zw;
#else
  ScreenTC = IN.baseTC.xy;
#endif

  float fDepth = GetLinearDepth(samplerZ, ScreenTC.xy);

#ifdef RT_ENVMAP       // RT_LIGHT_VOLUMES
  float3 vPosition = CalcHomogeneousPos(fDepth, IN.WPos.xy) + C_Eye.xyz;
#else
  float3 vPosition = GetWorldPos(fDepth, IN.CamVec);
#endif

  half fAttenuation = 1.h;

#ifndef RT_GLOW  // !RT_SUNLIGHT
  float3 vLightWS = LightPos.xyz - vPosition.xyz;  
  fAttenuation = GetDeferredLightAttenuation(vLightWS, LightPos.w);
#endif

#ifndef RT_GLOW  // !RT_SUNLIGHT
    half3 vLight = normalize(vLightWS);
#else
    float3 vLight = LightPos.xyz;
#endif

#if defined (VERTEX_COLOR) && !defined(RT_LIGHTMAP) // RT_SPOTLIGHT
		half4 vDirCosPhi = LightDir;
		fAttenuation *= saturate((-dot(vLight, vDirCosPhi.xyz) - vDirCosPhi.w) * 10.f);
#endif
  //fAttenuation = 1;
  
  half4 vNormal = GetWorldSpaceNormals(samplerN, ScreenTC.xy, true);
  {  
    half3 cFilter = 1;
#if defined(VERTEX_COLOR) && defined(RT_LIGHTMAP) // RT_TEX_PROJ
    // Projective light source
    float4 vProjTC = mul(float4(vPosition, 1.f), mLightProj);    
    cFilter = tex2Dlod(samplerLight, float4(vProjTC.xy/vProjTC.w, 0, 0));
#endif
    //cFilter = 1;

    half fInShadow = 1.0f;
#if RT_SHADOWS
    half4 vShadows = tex2D(samplerShadow, ScreenTC.xy);
    fInShadow = dot(ShadowMask, vShadows);
    /*float4 P0 = mul(mLightShadowProj, float4(vPosition, 1.f));
    P0.xy /= P0.w;
    P0.z -= 0.003f;
    irregular_filter_smpl(samplerShadow, samplerNoise, float4(P0.xyz, 1.f), P0.xy, GeneralParams.x, fInShadow); */
#endif

    half NdotL = saturate(dot(vLight.xyz, vNormal.xyz));

    // Diffuse term
    OUT.Diffuse.xyz = NdotL * fAttenuation * fInShadow * cFilter.xyz * cLightDiffuse.xyz;

#if RT_SPECULAR
    half3 vView = normalize(-vPosition);
    half fSpec = SimpleBRDF(vNormal, vView, vLight, vNormal.w * 255.0h);
    OUT.Specular.xyz = OUT.Diffuse.xyz * cLightDiffuse.w * fSpec; 
#endif

    //OUT.Diffuse.xyz = vPosition.xyz / 20; //vLight.xyz; //(vLight.xyz + 1.0) * 0.5;
    //OUT.Diffuse.xyz = fInShadow; //NdotL; //float3(IN.WPos.xy / 1000, 0); //NdotL;
  }

  EncodeLightBufferPair(OUT.Diffuse, OUT.Specular);

  return OUT;
}

pixout StencilCullPS(fragInWPOS IN)
{
  pixout OUT = (pixout) 0;

	OUT.Color = half4(1,1,1,1);
	return OUT;
}

technique AmbientOutdoor
{
  pass p0 
  {		
    VertexShader = compile VS_SHADER_TYPE VSMain();
    PixelShader  = compile PS_SHADER_TYPE PSAmbientOutdoor();
  }
}

technique LightVolume
{
  pass p0
  {
    VertexShader = compile vs_3_0 LightVolumeVS(); 
    PixelShader = compile ps_3_0 LightPassPS();

    AlphaBlendEnable = true;
#ifdef OGL
		CullMode = Front;
    BlendFunc = { One, One };
#else
    CullMode = CCW;
    SrcBlend = One;
    DestBlend = One;
#endif    
  }
}

technique LightPass
{
  pass p0
  {
    VertexShader = compile vs_3_0 LightPassVS();
    PixelShader = compile ps_3_0 LightPassPS();

    ZEnable = false;
    AlphaBlendEnable = true;
#ifdef OGL
		CullMode = FrontAndBack;
    BlendFunc = { One, One };
#else
    CullMode = None;
    SrcBlend = One;
    DestBlend = One;
#endif    
  }
}

technique StencilCull
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE StencilCullVS(); 
    PixelShader = compile PS_SHADER_TYPE StencilCullPS();

    ZEnable = false;
    ZWriteEnable = false;
#ifdef OGL
		CullMode = Back;
#else
    CullMode = CW;
#endif    
  }
}
technique StencilFrustumCull
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE StencilFrustumCullVS(); 
    PixelShader = compile PS_SHADER_TYPE StencilCullPS();

    ZEnable = false;
    ZWriteEnable = false;
#ifdef OGL
		CullMode = Back;
#else
    CullMode = CW;
#endif    
  }
}

technique TShow
{
  pass p0 
  {		
    VertexShader = compile VS_SHADER_TYPE VSShowMap();
    PixelShader  = compile PS_SHADER_TYPE PSShowMap();
  }
}
