#define FRAG_CUSTOM_END

#include "transform.fxh"

#define fragInput vertexOutput

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  pLight.fNdotL = saturate( pLight.fNdotL );
  half cK = pLight.fFallOff * pLight.cFilter;

  half3 cDiffuse = min(pLight.fNdotL, pLight.fOcclShadow) * pLight.cDiffuse.xyz;
  pPass.cDiffuseAcc += cDiffuse * cK;

  // Specular term
#ifdef RT_SPECULAR
	half fSpec = Phong(pPass.vNormal, pPass.vView, pLight.vLight, pPass.fSpecPow);
  half3 cSpecular = fSpec * pLight.fOcclShadow;
	if( pPass.nQuality >= QUALITY_HIGH )
	{
  	cSpecular *= saturate( pLight.fNdotL * 4 );  // apply self-shadowing
		cSpecular *= pLight.cSpecular.xyz;
	}
  pPass.cSpecularAcc += cSpecular * cK;
#endif
}

void frag_custom_end(inout fragPass pPass, inout FIXED3 cFinal)
{
	// do a secondary diffuse texture lookup
	half4 vSecondColor = tex2D(SecondColorSampler, pPass.IN.Tex0.xy * fDetailDiffuseScale);
	pPass.cDiffuseAcc *= vSecondColor.rgb;

#ifdef RT_SPECULAR

	#ifdef RT_ENVMAP
		// do a reflection cubemap lookup
		if( pPass.nQuality >= QUALITY_HIGH )
			pPass.cSpecularAcc += texCUBE(EnvMapSampler, pPass.vReflVec).rgb;
	#endif // RT_ENVMAP

		//if(pPass.nQuality >= QUALITY_MEDIUM)
			pPass.cSpecularAcc *= pPass.fSpecularMap;
		//if( pPass.nQuality >= QUALITY_HIGH )
		//	pPass.cSpecularAcc *= materialColor.rgb;

	#ifdef RT_FRESNEL
		// affect specular by a Fresnel term
		half fFresnelTerm = GetFresnel(pPass.fNdotE, pPass.nQuality, false);
		if(pPass.nQuality > QUALITY_MEDIUM)
			pPass.cSpecularAcc += fFresnelTerm * saturate(pPass.vNormal.y * 2.h) * Ks * FresnelColor;
		pPass.fAlpha *= fFresnelTerm;
	#endif // RT_FRESNEL
#endif

  half3 cDiffuse = (pPass.cAmbientAcc.xyz + pPass.cDiffuseAcc.xyz) * pPass.cDiffuseMap.xyz * pPass.cVertexColor.rgb;
	//if( pPass.nQuality > QUALITY_LOW )
	//	cDiffuse *= materialColor.rgb;

  cFinal += cDiffuse;
#ifdef RT_SPECULAR
  cFinal += pPass.cSpecularAcc;
#endif
}

#include "fragLib.fxh"

half4 Detail_PS(const in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass = (fragPass) 0;
  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

technique Detail < string name = "Material with detailed texture."; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE Detail_PS();
	DEFAULT_STATES
  }
}
