#include "constants.fxh"

struct vs_in {
	float3 Pos   : POSITION;
	float2 Tex0  : TEXCOORD0;
};
struct vs_out {
	float4 Pos  : POSITION;
	float2 Tex0 : TEXCOORD0;
};

struct vsVC_in {
	float3 Pos   : POSITION;
	float4 Color : COLOR0;
	float2 Tex0  : TEXCOORD0;
};
struct vsVC_out {
	float4 Pos  : POSITION;
	float4 Color : COLOR0;
	float2 Tex0 : TEXCOORD0;
};

uniform float4 uniColor = {1.0f, 1.0f, 1.0f, 1.0f};

vs_out VS_main(vs_in In)
{
	vs_out Out;
	Out.Pos = float4(In.Pos.xy, 1, 1);
	Out.Tex0 = In.Tex0;
#ifdef OPENGL
  Out.Pos.xy = (Out.Pos.xy * F_ScreenSize.xy) * 2.f - 1.f;
  Out.Pos.y = -Out.Pos.y;
#endif
	return Out;
}

vsVC_out VS_mainVC(vsVC_in In)
{
	vsVC_out Out;
	Out.Pos = float4(In.Pos.xy, 1, 1);
	Out.Tex0 = In.Tex0;
	Out.Color = In.Color;
//#ifdef OPENGL
  Out.Pos.xy = (Out.Pos.xy * F_ScreenSize.xy) * 2.f - 1.f;
//#ifdef OPENGL
  Out.Pos.y = -Out.Pos.y;
//#endif
	return Out;
}

half4 PS_main(vs_out In) : COLOR0
{
	float4 Col = tex2D(ColorSampler, In.Tex0) * uniColor;
 	
 	return Col;
}
half4 PS_mainNC(vs_out In) : COLOR0
{
	float4 Col = tex2D(ColorSampler, In.Tex0);
 	
 	return Col;
}
half4 PS_mainVC(vsVC_out In) : COLOR0
{
	float4 Col = tex2D(ColorSampler, In.Tex0) * In.Color;
 	
 	return Col;
}

technique Default // uniform color technique
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE VS_main();
		PixelShader = compile PS_SHADER_TYPE PS_main();
		DEFAULT_STATES
	}
}

technique FP_NoColor // no color technique
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE VS_main();
		PixelShader = compile PS_SHADER_TYPE PS_mainNC();
		DEFAULT_STATES
	}
}

technique FP_VertColor // vertex color technique
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE VS_mainVC();
		PixelShader = compile PS_SHADER_TYPE PS_mainVC();
		DEFAULT_STATES
	}
}

