/*
	GESilhouette
*/

#include "fragDef.fxh"
#include "Transform.fxh"

struct VS_OUT
{
	float4 Pos	: POSITION;
	float4 baseTC	: TEXCOORD0;
	float3 ScreenCoordProj	: TEXCOORD1;
	float4 Color	: TEXCOORD2;
};

VS_OUT VSPass(in VS_IN In)
{
	VS_OUT Out = (VS_OUT)0;
  float3 WorldPos = WorldPosition(In);
  Out.Pos = mul(float4(WorldPos,1),C_VP);
  Out.ScreenCoordProj = HPosToScreenTC(Out.Pos);
  Out.baseTC.z = Out.Pos.z;
  float3 instPos = float3(PI_mW[3].x, PI_mW[3].y, PI_mW[3].z);
  Out.baseTC.w = frac(length(instPos.xyz)) + 0.01;
  float3 vColor = float3(PI_mW[0].w, PI_mW[1].w, PI_mW[2].w);
  Out.Color = float4(vColor, PI_mW[3].w);
	return Out;
}

float4 PSPass(in VS_OUT In): COLOR0
{
	// depth-based effects
	half2 vTexCoord = In.ScreenCoordProj.xy / In.ScreenCoordProj.z;
	float fDepth = tex2D(DepthBufferSampler, vTexCoord).r * C_FarClip.x;

  half fMask = 1; //saturate(1 - abs(In.baseTC.z - fDepth)*0.333);

  half3 cColor = In.Color.xyz * In.Color.w;
  return float4(fMask.xxx * cColor, In.baseTC.w) ;
}



technique TPass
{
  pass p0 
  {
		VertexShader = compile VS_SHADER_TYPE VSPass();
		PixelShader  = compile PS_SHADER_TYPE PSPass();
  }
}

//=========================================================================================================

const float4 vpsParams = float4(1.f, 0.f, 0.f, 0.f);

struct vs_in {
	float3 Pos  : POSITION;
	float2 Tex0 : TEXCOORD0;
};

struct vs_out {
	float4 Pos   : POSITION;
	float2  Tex0 : TEXCOORD0;
};
struct vs_out_det {
	float4 Pos   : POSITION;
	float4  Tex0 : TEXCOORD0;
	float4  Tex1 : TEXCOORD1;
	float2  Tex2 : TEXCOORD2;
};


sampler2D sampler0 = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D samplerZ = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler2D samplerOrg = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D samplerScreen = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D samplerMask = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D samplerMaskBlured = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

vs_out_det VSPostDetect(in vs_in In)
{
	vs_out_det Out;
	Out.Pos = float4(In.Pos, 1);
  Out.Pos.xy *= F_ScreenSize.xy;
  Out.Pos.xy = Out.Pos.xy * 2.f - 1.f;

  Out.Tex0.xy = In.Tex0.xy + float2(-1, -1) * F_ScreenSize.xy * 1.5f;
  Out.Tex0.zw = In.Tex0.xy + float2(1, -1) * F_ScreenSize.xy * 1.5f;
  Out.Tex1.xy = In.Tex0.xy + float2(-1, 1) * F_ScreenSize.xy * 1.5f;
  Out.Tex1.zw = In.Tex0.xy + float2(1, 1) * F_ScreenSize.xy * 1.5f;
  Out.Tex2.xy = In.Tex0.xy;
	return Out;
}

float4 PSPostDetect(in vs_out_det In): COLOR0
{
  half4 cc  = tex2D(sampler0, In.Tex2.xy);
  half4 cc0 = tex2D(sampler0, In.Tex0.xy);
  half4 cc1 = tex2D(sampler0, In.Tex0.zw);
  half4 cc2 = tex2D(sampler0, In.Tex1.xy);
  half4 cc3 = tex2D(sampler0, In.Tex1.zw);

  half4 edges = 0;  
  edges.x = cc.w - cc0.w;
  edges.y = cc.w - cc1.w;
  edges.z = cc.w - cc2.w;
  edges.w = cc.w - cc3.w;
  edges = (abs(edges)) > 0;

  half fSilhouete = dot(edges.xyzw, 1.333); 

  half4 cSilhouetteColor = cc*0.5;
  cSilhouetteColor +=  cc0*0.125;
  cSilhouetteColor +=  cc1*0.125;
  cSilhouetteColor +=  cc2*0.125;
  cSilhouetteColor +=  cc3*0.125;

  half3 Color = (cSilhouetteColor * fSilhouete);
  return float4(Color, 1);
}

vs_out VSPostBase(vs_in In)
{
	vs_out Out;
	Out.Pos = float4(In.Pos, 1);
  Out.Pos.xy *= F_ScreenSize.xy;
  Out.Pos.xy = Out.Pos.xy * 2.f - 1.f;

	Out.Tex0 = In.Tex0;
#ifdef OPENGL
  Out.Tex0.y = 1 - Out.Tex0.y;
#endif

	return Out;
}

float4 PSPostGlow(in vs_out_det In): COLOR0
{
  half4 cc  = tex2D(samplerOrg, In.Tex2.xy)*0.5;
  half4 cc0 = tex2D(samplerOrg, In.Tex0.xy)*0.125;
  half4 cc1 = tex2D(samplerOrg, In.Tex0.zw)*0.125;
  half4 cc2 = tex2D(samplerOrg, In.Tex1.xy)*0.125;
  half4 cc3 = tex2D(samplerOrg, In.Tex1.zw)*0.125;

  // remove body tinting from glow, only leave silhouettes
  float4 Color = cc + cc0 + cc1 + cc2 + cc3;
  
  return Color;
}


float4 PSPostFinal(in vs_out IN): COLOR0
{
  half4 cScreen = tex2D(samplerScreen, IN.Tex0);
  half4 cSilhouettes = tex2D(samplerMask, IN.Tex0);
  half4 cSilhouettesBlurred = tex2D(samplerMaskBlured, IN.Tex0);

  half fInterlation = abs(frac((IN.Tex0.y)*F_ScreenSize.w*0.25)*2-1)*0.5+0.5;
  half fVigneti = saturate( 1 - length(IN.Tex0.xy - 0.5)*0.6 );

  half4 Color =  fInterlation * 1.5 * (cScreen * half4( 0.7, 0.8, 1.0, 1.0) )+ (cSilhouettesBlurred + cSilhouettes*saturate(fInterlation*2-1)*1.5)*0.666;
  Color *= fVigneti;

  Color = lerp(cScreen, Color, vpsParams.x);
  
  return Color;
}

float4 PSPostFinalNoTinting(in vs_out IN): COLOR0
{
  half4 cSilhouettesBlurred = tex2D(samplerMaskBlured, IN.Tex0);

  half4 Color = cSilhouettesBlurred * 1.5f;
  
  return Color;
}

technique TPostDetect
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSPostDetect();
		PixelShader  = compile PS_SHADER_TYPE PSPostDetect();
#ifdef OGL
		CullMode = FrontAndBack;
#else
		CullMode = none;
#endif
		ZEnable = false;
	}
}
technique TPostGlow
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSPostDetect();
		PixelShader  = compile PS_SHADER_TYPE PSPostGlow();
#ifdef OGL
		CullMode = FrontAndBack;
#else
		CullMode = none;
#endif
		ZEnable = false;
	}
}
technique TPostFinal
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSPostBase();
		PixelShader  = compile PS_SHADER_TYPE PSPostFinal();
#ifdef OGL
		CullMode = FrontAndBack;
#else
		CullMode = none;
#endif
		ZEnable = false;
	}
}
technique TPostFinalNoTinting
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSPostBase();
		PixelShader  = compile PS_SHADER_TYPE PSPostFinalNoTinting();
#ifdef OGL
		CullMode = FrontAndBack;
    BlendEnable = true;
    BlendFunc = { One, One };
#else
		CullMode = none;
    AlphaBlendEnable = true;
    SrcBlend = ONE;
    DestBlend = ONE;
#endif
		ZEnable = false;
	}
}

//===========================================================================================

sampler2D samplerSil = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

vs_out VSShow(vs_in In)
{
	vs_out Out;
	Out.Pos = float4(In.Pos.xyz, 1);
	Out.Tex0 = In.Tex0;
	return Out;
}

float4 PSShow(vs_out In): COLOR
{
	float4 Col = tex2D(samplerSil, In.Tex0);
	
	return Col;
}

technique TShow
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSShow();
		PixelShader  = compile PS_SHADER_TYPE PSShow();
	}
}
