/*
	GEToon
*/

#include "fragDef.fxh"
#include "Transform.fxh"

struct VS_OUT
{
	float4 Pos	: POSITION;
	float4 baseTC	: TEXCOORD0;
	float3 ScreenCoordProj	: TEXCOORD1;
	float4 Color	: TEXCOORD2;
};

VS_OUT VSPass(in VS_IN In)
{
	VS_OUT Out = (VS_OUT)0;
  float3 WorldPos = WorldPosition(In);
  Out.Pos = mul(float4(WorldPos,1),C_VP);
  Out.ScreenCoordProj = HPosToScreenTC(Out.Pos);
  Out.baseTC.z = Out.Pos.z;
  float3 instPos = float3(PI_mW[3].x, PI_mW[3].y, PI_mW[3].z);
  Out.baseTC.w = frac(length(instPos.xyz)) + 0.01;
  float3 vColor = float3(PI_mW[0].w, PI_mW[1].w, PI_mW[2].w);
  Out.Color = float4(vColor, PI_mW[3].w);
	return Out;
}

float4 PSPass(in VS_OUT In): COLOR0
{
	// depth-based effects
	half2 vTexCoord = In.ScreenCoordProj.xy / In.ScreenCoordProj.z;
	float fDepth = tex2D(DepthBufferSampler, vTexCoord).r * C_FarClip.x;

  half fMask = 1; //saturate(1 - abs(In.baseTC.z - fDepth)*0.333);

  half3 cColor = In.Color.xyz * In.Color.w;
  return float4(fMask.xxx * cColor, In.baseTC.w) ;
}



technique TPass
{
  pass p0 
  {
		VertexShader = compile VS_SHADER_TYPE VSPass();
		PixelShader  = compile PS_SHADER_TYPE PSPass();
  }
}

//=========================================================================================================

uniform float4 DepthParams; // [0] = Threshold; [1] = Sensitivity; [2] = Width; [3] = EdgeIntensity
uniform float4 EdgeColor;

struct vs_in {
	float3 Pos  : POSITION;
	float2 Tex0 : TEXCOORD0;
};

struct vs_out {
	float4 Pos   : POSITION;
	float2  Tex0 : TEXCOORD0;
};
struct vs_out_det {
	float4 Pos   : POSITION;
	float2  Tex0 : TEXCOORD0;
};


sampler2D samplerZ = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D samplerDepth = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

vs_out_det VSPostDetect(in vs_in In)
{
	vs_out_det Out;
	Out.Pos = float4(In.Pos, 1);
  Out.Pos.xy *= F_ScreenSize.xy;
  Out.Pos.xy = Out.Pos.xy * 2.f - 1.f;
  Out.Tex0.xy = In.Tex0.xy;
	return Out;
}

float4 PSPostDetect(in vs_out_det In): COLOR0
{
  float2 edgeOffset = DepthParams.z * F_ScreenSize.xy;
  float4 vMainZ = tex2D(samplerDepth, In.Tex0.xy);
  float4 vMainToon = tex2D(samplerZ, In.Tex0.xy);
  float3 EdgeColor = Unpack(vMainToon.g);
  //return float4(EdgeWidthAndColor.xyz, 1); 
  clip(vMainZ.r-vMainToon.r); // Depth test
  edgeOffset *= vMainToon.w;

  half4 cc0 = tex2D(samplerZ, In.Tex0.xy + float2(-1, -1) * edgeOffset);
  half4 cc1 = tex2D(samplerZ, In.Tex0.xy + float2(1, 1) * edgeOffset);
  half4 cc2 = tex2D(samplerZ, In.Tex0.xy + float2(-1, 1) * edgeOffset);
  half4 cc3 = tex2D(samplerZ, In.Tex0.xy + float2(1, -1) * edgeOffset);

  half fDiagonalDelta = abs(cc0.b - cc1.b) + abs(cc2.b - cc3.b);

  //float normalDelta = dot(diagonalDelta.xyz, 1);
  float depthDelta = fDiagonalDelta;

  // Filter out very small changes, in order to produce nice clean results.
  //normalDelta = saturate((normalDelta - NormalThreshold) * NormalSensitivity);
  depthDelta = saturate((depthDelta - DepthParams.x) * DepthParams.y);

  // Does this pixel lie on an edge?
  //float edgeAmount = saturate(normalDelta + depthDelta) * DepthParams.w;
  float edgeAmount = depthDelta * DepthParams.w;

  //return 0;
  return float4(EdgeColor.xyz, edgeAmount);
}


technique TEdgeDetect
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSPostDetect();
		PixelShader  = compile PS_SHADER_TYPE PSPostDetect();

    ZEnable = false;
    AlphaBlendEnable = true;
#ifdef OGL
		CullMode = FrontAndBack;
    BlendFunc = { SrcAlpha, InvSrcAlpha };
#else
    CullMode = None;
    SrcBlend = SrcAlpha;
    DestBlend = InvSrcAlpha;
#endif    
	}
}

vs_out VSShow(vs_in In)
{
  vs_out Out;
  Out.Pos = float4(In.Pos, 1);
#ifndef OPENGL
  Out.Pos.xy = Out.Pos.xy * 2.f - float2(1.f, 1.f);
#endif
  Out.Tex0 = In.Tex0;
  return Out;
}

float4 PSShow(vs_out In): COLOR
{
	float4 Col = tex2D(samplerZ, In.Tex0).b;
	
	return Col;
}

technique TShow
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSShow();
		PixelShader  = compile PS_SHADER_TYPE PSShow();
	}
}
