#include "constants.fxh"
/////////////////////////////////////// Tweakables ////////

float fIntensity = 3.0f;

float Glowness = 3.0f;

// file texture (surface)

texture ModelTex;

sampler ModelTexSamp = sampler_state 
{
    texture = <ModelTex>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

///////////////////////////// Render-to-Texture Data //////

#define RTT_SIZE 512

float TexelIncrement = 1.0f / RTT_SIZE;

texture GlowMap1;
sampler GlowSamp1 = sampler_state 
{
    texture = <GlowMap1>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

texture GlowMap2;
sampler GlowSamp2 = sampler_state 
{
    texture = <GlowMap2>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

/////////////////////////////////// data structures ///////

struct VS_OUTPUT_BLUR
{
  float4 Position   : POSITION;
  float4 TexCoord0   : TEXCOORD0;
  float4 TexCoord1   : TEXCOORD1;
  float4 TexCoord2   : TEXCOORD2;
  float4 TexCoord3   : TEXCOORD3;
  float4 TexCoord4   : TEXCOORD4;
  float4 TexCoord5   : TEXCOORD5;
  float4 TexCoord6   : TEXCOORD6;
  float4 TexCoord7   : TEXCOORD7;
  float4 TexCoord8   : COLOR1;   
};

struct VS_OUTPUT
{
 	float4 Position   : POSITION;
  float4 TexCoord0   : TEXCOORD0;
};

VS_OUTPUT VS_Quad(float4 Position : POSITION, float2 TexCoord : TEXCOORD0)
{
    VS_OUTPUT OUT = (VS_OUTPUT)0;
    OUT.Position = Position;
    OUT.TexCoord0 = float4(TexCoord,1,1); 
    return OUT;
}

VS_OUTPUT_BLUR VS_Quad_Vertical_9tap(float4 Position : POSITION, 
			float2 TexCoord : TEXCOORD0)
{
    VS_OUTPUT_BLUR OUT;
    OUT.Position = Position;
    
    float3 Coord = float3(TexCoord.x + TexelIncrement, TexCoord.y + TexelIncrement, 1);
    OUT.TexCoord0 = float4(Coord.x, Coord.y + TexelIncrement, 1, 1);
    OUT.TexCoord1 = float4(Coord.x, Coord.y + TexelIncrement * 2, 1, 1);
    OUT.TexCoord2 = float4(Coord.x, Coord.y + TexelIncrement * 3, 1, 1);
    OUT.TexCoord3 = float4(Coord.x, Coord.y + TexelIncrement * 4, 1, 1);
    OUT.TexCoord4 = float4(Coord.x, Coord.y, 1, 1);
    OUT.TexCoord5 = float4(Coord.x, Coord.y - TexelIncrement, 1, 1);
    OUT.TexCoord6 = float4(Coord.x, Coord.y - TexelIncrement * 2, 1, 1);
    OUT.TexCoord7 = float4(Coord.x, Coord.y - TexelIncrement * 3, 1, 1);
    OUT.TexCoord8 = float4(Coord.x, Coord.y - TexelIncrement * 4, 1, 1);
    return OUT;
}

VS_OUTPUT_BLUR VS_Quad_Horizontal_9tap(float4 Position : POSITION, 
			float2 TexCoord : TEXCOORD0)
{
    VS_OUTPUT_BLUR OUT = (VS_OUTPUT_BLUR)0;
    OUT.Position = Position;
    
    float3 Coord = float3(TexCoord.x + TexelIncrement, TexCoord.y + TexelIncrement, 1);
    OUT.TexCoord0 = float4(Coord.x + TexelIncrement, Coord.y, 1, 1);
    OUT.TexCoord1 = float4(Coord.x + TexelIncrement * 2, Coord.y, 1, 1);
    OUT.TexCoord2 = float4(Coord.x + TexelIncrement * 3, Coord.y, 1, 1);
    OUT.TexCoord3 = float4(Coord.x + TexelIncrement * 4, Coord.y, 1, 1);
    OUT.TexCoord4 = float4(Coord.x, Coord.y, 1, 1);
    OUT.TexCoord5 = float4(Coord.x - TexelIncrement, Coord.y, 1, 1);
    OUT.TexCoord6 = float4(Coord.x - TexelIncrement * 2, Coord.y, 1, 1);
    OUT.TexCoord7 = float4(Coord.x - TexelIncrement * 3, Coord.y, 1, 1);
    OUT.TexCoord8 = float4(Coord.x - TexelIncrement * 4, Coord.y, 1, 1);
    return OUT;
}

////////////////////////////////// pixel shaders /////

// For two-pass blur, we have chosen to do  the horizontal blur FIRST. The
//	vertical pass includes a post-blur scale factor.

// Relative filter weights indexed by distance from "home" texel
//    This set for 9-texel sampling
#define WT9_0 1.0
#define WT9_1 0.8
#define WT9_2 0.6
#define WT9_3 0.4
#define WT9_4 0.2

// Alt pattern -- try your own!
// #define WT9_0 0.1
// #define WT9_1 0.2
// #define WT9_2 3.0
// #define WT9_3 1.0
// #define WT9_4 0.4

#define WT9_NORMALIZE (WT9_0+2.0*(WT9_1+WT9_2+WT9_3+WT9_4))

float4 PS_Blur_Horizontal_9tap(VS_OUTPUT_BLUR IN) : COLOR
{   
    float4 OutCol = tex2D(GlowSamp1, IN.TexCoord0) * (WT9_1/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord1) * (WT9_2/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord2) * (WT9_3/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord3) * (WT9_4/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord4) * (WT9_0/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord5) * (WT9_1/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord6) * (WT9_2/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord7) * (WT9_3/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp1, IN.TexCoord8) * (WT9_3/WT9_NORMALIZE);
    return OutCol;
} 

float4 PS_Blur_Vertical_9tap(VS_OUTPUT_BLUR IN) : COLOR
{   
    float4 OutCol = tex2D(GlowSamp2, IN.TexCoord0) * (WT9_1/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord1) * (WT9_2/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord2) * (WT9_3/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord3) * (WT9_4/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord4) * (WT9_0/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord5) * (WT9_1/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord6) * (WT9_2/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord7) * (WT9_3/WT9_NORMALIZE);
    OutCol += tex2D(GlowSamp2, IN.TexCoord8) * (WT9_3/WT9_NORMALIZE);
    return Glowness*OutCol;
} 

// add glow on top of model

float4 PS_GlowPass(VS_OUTPUT IN) : COLOR
{   
	float4 tex = tex2D(GlowSamp1, float2(IN.TexCoord0.x, IN.TexCoord0.y));
	return tex;
}  

/////////////////////////////////////// techniques /////////
technique Default
{
  pass BlurGlowBuffer_Horz
	{
		cullmode = CW;
		ZEnable = false;
		ZWriteEnable = false;
		VertexShader = compile VS_SHADER_TYPE VS_Quad_Horizontal_9tap();
		PixelShader  = compile PS_SHADER_TYPE PS_Blur_Horizontal_9tap();
  }
    pass BlurGlowBuffer_Vert
		{
		VertexShader = compile VS_SHADER_TYPE VS_Quad_Vertical_9tap();
		PixelShader  = compile PS_SHADER_TYPE PS_Blur_Vertical_9tap();
    }
    pass GlowPass
		{
		AlphaBlendEnable = true;
		SrcBlend = one;
		DestBlend = one;
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_GlowPass();	
    }
}

