#include "VertexDef.fxh"

#if RT_PROFILE <= QUALITY_LOW
#define LOW_SPEC_CUSTOM_LLIGHTING
void LowSpecLighting(inout vertexOutput Out, const in VS_IN In, const float3 vWorldPos, const float3 vWorldNormal, int nType)
{
#if RT_PROFILE == QUALITY_LOW
    Out.Diffuse = float4(0,0,0,1);
 #if defined (RT_NUM_LIGHTS)
    float3 cLightDiffuse = LGetDiffuse(0);
    float4 WorldLightPos = LGetPosition(0);    
    float fNdotL = abs(dot(LGetDir(0).xyz, vWorldNormal.xyz)) * .5f + .5f;
    Out.Diffuse.rgb = lerp(fNdotL, saturate(LGetDir(0).y), .7h) * cLightDiffuse;
    Out.Diffuse.rgb += F_WorldAmbient.rgb;
    //Out.Diffuse.rgb *= materialColor.rgb;
 #endif
#elif RT_PROFILE == QUALITY_VERYLOW
  Out.Diffuse = float4(0.85,0.85,0.85,0) + PI_WColor;
#endif
}
#endif 

#include "transform.fxh"

#define fragInput vertexOutput

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
	half fDiffuse = abs( pLight.fNdotL );
	pLight.fNdotL = saturate(pLight.fNdotL);
#if RT_PROFILE > QUALITY_HIGH	
	fDiffuse = pLight.fNdotL + LeafShadingBack(pPass.vView, pLight.vLight, pPass.vNormal);
#endif
	
	fDiffuse = lerp(fDiffuse, saturate(pLight.vLight.y), .7h);

	half3 cDiffuse = min(fDiffuse, pLight.fOcclShadow) * pLight.cDiffuse.xyz;

  half cK = pLight.fFallOff * pLight.cFilter;
  pPass.cDiffuseAcc += cDiffuse * cK;
}

#include "fragLib.fxh"

half4 GrassPS(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

technique Grass < string name = "Material for grass. Features: double-sided lighting, adopts to terrain color"; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE GrassPS();
		DEFAULT_STATES
  }
}
