#include "VertexDef.fxh"

#if RT_PROFILE <= QUALITY_LOW
#define LOW_SPEC_CUSTOM_LLIGHTING
void LowSpecLighting(inout vertexOutput Out, const in VS_IN In, const float3 vWorldPos, const float3 vWorldNormal, int nType)
{
#if RT_PROFILE == QUALITY_LOW
    Out.Diffuse.rgb = F_WorldAmbient.rgb;
    Out.Diffuse.a = 1.f;
 #ifdef RT_NUM_LIGHTS
    half3 Diffuse = LGetDiffuse(0);
    half3 vDirToLight;
    FIXED fFallOff;
    GetLighting(vDirToLight, fFallOff, vWorldPos.xyz, RT_LIGHTTYPE0, 0, 0);
    // Compute diffuse
    half fNdotL = abs(dot(vDirToLight, vWorldNormal.xyz)) * .5f + .5f;
    Out.Diffuse.rgb += fNdotL * Diffuse.xyz * fFallOff;
 #endif
#elif RT_PROFILE == QUALITY_VERYLOW
  Out.Diffuse = float4(0.85,0.85,0.85,0) + PI_WColor;
#endif
	//Out.Diffuse.rgb *= materialColor.rgb;
}
#endif 

#include "transform.fxh"

#define fragInput vertexOutput

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
	half fDiffuse = abs( pLight.fNdotL ) * .5h + .5h;
	pLight.fNdotL = saturate(pLight.fNdotL);
#if RT_PROFILE > QUALITY_VERYLOW
	if( pPass.nQuality > QUALITY_HIGH )
	{
		fDiffuse = pLight.fNdotL + LeafShadingBack(pPass.vView, pLight.vLight, pPass.vNormal);
	}
#endif	

	fDiffuse = lerp(fDiffuse, saturate(pLight.vLight.y), .7h);

	half3 cDiffuse = fDiffuse * pLight.cDiffuse.xyz;

  half cK = pLight.fOcclShadow * pLight.fFallOff * pLight.cFilter;
  pPass.cDiffuseAcc += cDiffuse * cK;
}

#include "fragLib.fxh"

half4 Leafs_PS(in fragInput IN, bool bReceiveShadows) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, IN);
  pPass.bIgnoreShadows = !bReceiveShadows;
  half4 cFinal = frag_shared_output(pPass);
  return cFinal;
}

half4 Leafs_PS_t(in fragInput IN) : COLOR
{
  return Leafs_PS(IN, true);
}
half4 Leafs_PS_f(in fragInput IN) : COLOR
{
  return Leafs_PS(IN, false);
}

technique Leafs < string name = "Diffuse material for vegetation. Used for leafs. Features: subscattering and double-sided lighting"; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
#ifndef OPENGL
    PixelShader = compile PS_SHADER_TYPE Leafs_PS_t();
#else
    PixelShader = compile PS_SHADER_TYPE Leafs_PS_f();
#endif
    DEFAULT_STATES
  }
}

technique LeafsNoShadows < string name = "Diffuse material for vegetation. Used for leafs. Doesn't receive shadows! Features: subscattering and double-sided lighting"; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE Leafs_PS_f();
    DEFAULT_STATES
  }
}

