#include "transform.fxh"

#define fragInput vertexOutput

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  pLight.fNdotL = saturate( pLight.fNdotL );
  half cK = pLight.fFallOff * pLight.cFilter;

  half3 cDiffuse = min(pLight.fNdotL, pLight.fOcclShadow) * pLight.cDiffuse.xyz;
  pPass.cDiffuseAcc += cDiffuse * cK;

  // Specular term
#if RT_PROFILE > QUALITY_VERYLOW
 #if defined(RT_SPECULAR)
  half fSpec = Phong(pPass.vNormal, pPass.vView, pLight.vLight, pPass.fSpecPow);
  half3 cSpecular = fSpec * pLight.fOcclShadow * pLight.cSpecular.xyz * cK;
  if( pPass.nQuality >= QUALITY_HIGH )
  {
    cSpecular *= saturate(pLight.fNdotL * 4);  // apply self-shadowing
  }
  pPass.cSpecularAcc += cSpecular;
 #endif
#endif
}

#include "fragLib.fxh"

half4 Main_PS(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif

  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
  return cFinal;
}

half4 MainBuilding_PS(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif

  frag_unify(pPass, IN);  
  pPass.bBuilding = 1;
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

half4 Main_PS2(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif

  frag_unify(pPass, IN);  
  pPass.nPass = 1;
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

half4 Main_PSC1(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif

  frag_unify(pPass, IN);  
  pPass.nType = 1;
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}
half4 Main_PSC2(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  
  frag_unify(pPass, IN);  
  pPass.nType = 2;
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

technique Main < string name = "Main uber-material. Used for most objects in the game."; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE Main_PS();
    DEFAULT_STATES
#if defined(GLASS) 
 #ifdef OGL
    BlendEnable = true;
    BlendFunc = { SrcAlpha, InvSrcAlpha };
 #else
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = InvSrcAlpha;
 #endif
#endif
  }
#ifndef OPENGL
 #if defined(GLASS) 
  pass p1
  {
    VertexShader = compile VS_SHADER_TYPE UberVS2();
    PixelShader = compile PS_SHADER_TYPE Main_PS2();
    DEFAULT_STATES
  #ifdef OGL
    BlendEnable = true;
    BlendFunc = { One, One };
  #else
    AlphaBlendEnable = true;
    SrcBlend = ONE;
    DestBlend = ONE;
  #endif
  }
 #endif
#endif
}

#ifndef OPENGL
technique MainBuilding
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberBuilding_VS();
    PixelShader = compile PS_SHADER_TYPE MainBuilding_PS();
    DEFAULT_STATES
  }
}

technique MainCustom1
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVSC1();
    PixelShader = compile PS_SHADER_TYPE Main_PSC1();
    DEFAULT_STATES
  }
}

technique MainCustom2
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVSC2();
    PixelShader = compile PS_SHADER_TYPE Main_PSC2();
    DEFAULT_STATES
  }
}
#endif
