#include "transform.fxh"

#define fragInput vertexOutput

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  pLight.fNdotL = saturate( pLight.fNdotL );
  half cK = pLight.fFallOff * pLight.cFilter;

  half3 cDiffuse = min(pLight.fNdotL, pLight.fOcclShadow) * pLight.cDiffuse.xyz;
  pPass.cDiffuseAcc += cDiffuse * cK;

  // Specular term
#ifdef RT_SPECULAR
	half fSpec = 0;
 #if RT_PROFILE >= QUALITY_HIGH
	fSpec = WardAnisotropic_opt(pPass.mTangentToWS0, pPass.mTangentToWS1, pPass.vNormal, pPass.vView, 
																pLight.vLight, pLight.fNdotL, pPass.fNdotE, 1.f, pPass.fSpecPow);
 	fSpec *= saturate( pLight.fNdotL * 4 );  // apply self-shadowing
 #elif RT_PROFILE > QUALITY_VERYLOW
	fSpec = Phong(pPass.vNormal, pPass.vView, pLight.vLight, pPass.fSpecPow);
 #endif
 #if RT_PROFILE > QUALITY_VERYLOW
  half3 cSpecular = pLight.cSpecular.xyz * fSpec * pLight.fOcclShadow;
  pPass.cSpecularAcc += cSpecular * cK;
 #endif 
#endif
}

#include "fragLib.fxh"

float4 aniso_PS(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
  return cFinal;
}

technique Main_AnisoNM < string name = "Material for objects with anisotropic specular."; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE aniso_PS();
    DEFAULT_STATES
  }
}
