#include "constants.fxh"

uniform half4 Intensity;

texture2D SceneMap;
sampler2D SceneSampler = sampler_state 
{
  Texture = <SceneMap>;
  AddressU  = CLAMP;        
  AddressV  = CLAMP;
  AddressW  = CLAMP;
  MIPFILTER = NONE;
  MINFILTER = LINEAR;
  MAGFILTER = LINEAR;
};

struct vs_out
{
  float4 Position   : POSITION;
  half2 Tex0  : TEXCOORD0;
};

struct vs_in {
  float3 Pos: POSITION;
  half2 Tex0: TEXCOORD0;
};

vs_out ScreenQuadVS(vs_in In)
{
  vs_out Out;
  Out.Position = float4(In.Pos.xyz, 1);
  Out.Position.xy *= F_ScreenSize.xy;
  Out.Position.xy = Out.Position.xy * 2.f - 1.f;
  Out.Tex0 = In.Tex0;

  return Out;
}

half4 main0(vs_in IN) : COLOR
{   
  half4 scene = tex2D(SceneSampler, IN.Tex0);
  scene.a = Intensity.x;
  return scene;
}  

technique Default
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE ScreenQuadVS();
    PixelShader  = compile PS_SHADER_TYPE main0();

    ZEnable = false;
    AlphaBlendEnable = true;
#ifdef OGL
    CullMode = FrontAndBack;
    BlendFunc = { SrcAlpha, InvSrcAlpha };
#else
    CullMode = None;
    SrcBlend = SrcAlpha;
    DestBlend = InvSrcAlpha;
#endif    
  }
  pass p1
  {
    VertexShader = compile VS_SHADER_TYPE ScreenQuadVS();
    PixelShader  = compile PS_SHADER_TYPE main0();

    ZEnable = false;
    CullMode = None;
  }
}

