#include "Constants.fxh"
#include "Transform.fxh"


uniform float4 GlowParams;
uniform float4 GlowOffsets[5];
uniform float4 GlowWeights[3];

texture GlowMap;
sampler GlowSamp : register(s0) = sampler_state 
{
  Texture = <GlowMap>;
  AddressU  = CLAMP;        
  AddressV  = CLAMP;
  AddressW  = CLAMP;
  MIPFILTER = NONE;
  MINFILTER = LINEAR;
  MAGFILTER = LINEAR;
};
sampler GlowSamp1 : register(s1) = sampler_state 
{
  Texture = <GlowMap>;
  AddressU  = CLAMP;        
  AddressV  = CLAMP;
  AddressW  = CLAMP;
  MIPFILTER = NONE;
  MINFILTER = LINEAR;
  MAGFILTER = LINEAR;
};
sampler GlowSamp2 : register(s2) = sampler_state 
{
  Texture = <GlowMap>;
  AddressU  = CLAMP;
  AddressV  = CLAMP;
  AddressW  = CLAMP;
  MIPFILTER = NONE;
  MINFILTER = LINEAR;
  MAGFILTER = LINEAR;
};

struct VS_OUT
{
	float4 Pos	: POSITION;
	float2 Tex0	: TEXCOORD0;
#if defined (VERTEX_COLOR)
  float4 VtxColor : COLOR0; 
#endif
};

struct vtxIn
{
  float3 Pos    : POSITION;
  float2 Tex0   : TEXCOORD0;
};
struct vtxOut
{
  float4 Pos    : POSITION;
  float2 Tex0   : TEXCOORD0;
};

vtxOut VS_Quad(vtxIn In)
{
  vtxOut Out;
  Out.Pos = float4(In.Pos, 1);
#ifndef OPENGL
  Out.Pos.xy = Out.Pos.xy * 2.f - float2(1.f, 1.f);
#endif
  Out.Tex0 = In.Tex0;
  return Out;
}

VS_OUT VS_GlowPass(in VS_IN In)
{
  VS_OUT OUT = (VS_OUT)0;
    
  float3 vWorldPos = WorldPosition(In);

#ifdef RT_SKINNING
  int4 IndexVector = D3DCOLORtoUBYTE4(In.BoneID);
  float4 BlendWeightsArray = D3DCOLORtoUBYTE4(In.Weight);
  vWorldPos = In.Position.xyz;
  float3 vWorldNormal = (float3)0;
   __SkinningPositionNormal(IndexVector, BlendWeightsArray, vWorldPos, vWorldNormal);
#endif
  
	OUT.Pos = mul(float4(vWorldPos, 1), C_VP);
    
  OUT.Tex0.xy = GetTexCoords(In.Tex0.xy);
    
#if defined (VERTEX_COLOR)
  OUT.VtxColor = In.VtxColor; 
#endif
    
  return OUT;
}


float4 PS_GlowPass(VS_OUT IN) : COLOR
{   
	float4 cGlow = tex2D(ColorSampler, IN.Tex0.xy);
	cGlow.xyz *= materialColor.xyz;
	cGlow.xyz *= cGlow.a * fGlowIntensity;
#if defined (VERTEX_COLOR)
	cGlow.xyz *= IN.VtxColor.xyz;
#endif
	cGlow.a = 1;
	
	return cGlow;
}  


half4 PS_PostABlur(in vtxOut IN) : COLOR0
{
  half4 cFlare = 0;
  half nSamples = 9;
  half fRecipSamples = (1.0h / (nSamples - 1.0h));

  half4 cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[0].xy).xyz, 1);
  cFlare += cCurr * GlowWeights[0].x;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[0].zw).xyz, 1);
  cFlare += cCurr * GlowWeights[0].y;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[1].xy).xyz, 1);
  cFlare += cCurr * GlowWeights[0].z;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[1].zw).xyz, 1);
  cFlare += cCurr * GlowWeights[0].w;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[2].xy).xyz, 1);
  cFlare += cCurr * GlowWeights[1].x;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[2].zw).xyz, 1);
  cFlare += cCurr * GlowWeights[1].y;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[3].xy).xyz, 1);
  cFlare += cCurr * GlowWeights[1].z;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[3].zw).xyz, 1);
  cFlare += cCurr * GlowWeights[1].w;

  cCurr = half4(tex2D(GlowSamp, IN.Tex0.xy + GlowOffsets[4].xy).xyz, 1);
  cFlare += cCurr * GlowWeights[2].x;

  cFlare /= cFlare.w;  
#ifndef TEST
  cFlare *= 0.5;
#endif

  half4 Color = cFlare;

  return Color;
}

half4 PS_PostMergeBlurWithGlow(in vtxOut IN) : COLOR0
{
  half4 cFlares = tex2D(GlowSamp, IN.Tex0.xy);
  cFlares += tex2D(GlowSamp1, IN.Tex0.xy);
  cFlares *= GlowParams;

  half4 Color = cFlares;

  return Color;
}

half4 PS_PostGlowMerge(in vtxOut IN) : COLOR0
{
  half3 tGlow1 = tex2D(GlowSamp, IN.Tex0.xy);
  half3 tGlow2 = tex2D(GlowSamp1, IN.Tex0.xy);
  half3 tGlow3 = tex2D(GlowSamp2, IN.Tex0.xy);

  half3 finalGlow = (tGlow1 + tGlow2 + tGlow3) * GlowParams.w;

  half3 cGhost = 0;
#ifdef TEST
  half2 Tex0 = IN.Tex0.xy - 0.5h;
  half2 uv = Tex0.xy * -5.h + 0.5h;
  cGhost = tex2D(GlowSamp1, uv) * saturate(1.h-length(uv*2.h-1.h));

  uv = Tex0.xy * -2.h + 0.5h;
  cGhost += tex2D(GlowSamp1, uv) * saturate(1.h-length(uv*2.h-1.h));

  uv = Tex0.xy * -1.h + 0.5h;
  cGhost += tex2D(GlowSamp1, uv) * saturate(1.h-length(uv*2.h-1.h));

  uv = Tex0.xy * 3.h + 0.5h;
  cGhost += tex2D(GlowSamp1, uv) * saturate(1.h-length(uv*2.h-1.h));

  uv = Tex0.xy * 2.h + 0.5h;
  cGhost += tex2D(GlowSamp1, uv) * saturate(1.h-length(uv*2.h-1.h));
#endif

  half4 Color = (half4(finalGlow.xyz + cGhost.xyz * GlowParams.z, 1));
  Color.xyz = sqrt(Color.xyz * 0.5f);

  return Color;
}

half4 PS_PostDisplay(in vtxOut IN) : COLOR0
{
  half3 tScreen = tex2D(GlowSamp, IN.Tex0.xy);
  half3 tGlow = tex2D(GlowSamp1, IN.Tex0.xy);
  tGlow.xyz *= tGlow.xyz * 2.0;

  half4 Color;
  Color.xyz = tScreen + tGlow;
  Color.w = 1;

  return Color;
}

/////////////////////////////////////// techniques /////////
technique PassGlow
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_GlowPass();
		PixelShader = compile PS_SHADER_TYPE PS_GlowPass();	
  }
}


technique PostABlur
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_PostABlur();	
  }
}

technique PostMergeBlurWithGlow
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_PostMergeBlurWithGlow();
  }
}

technique PostGlowMerge
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_PostGlowMerge();
  }
}

technique PostDisplay
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_PostDisplay();
  }
}

//============================================================================================================
// Utils

texture Map;
sampler Samp : register(s0) = sampler_state 
{
  Texture = <Map>;
  AddressU  = CLAMP;        
  AddressV  = CLAMP;
  AddressW  = CLAMP;
  MIPFILTER = NONE;
  MINFILTER = LINEAR;
  MAGFILTER = LINEAR;
};
uniform float4 TexToTexParams0;
uniform float4 TexToTexParams1;


half4 PS_TextureToTexture(in vtxOut IN) : COLOR0
{
  half4 Color = tex2D(Samp, IN.Tex0.xy);

  return Color;
}

half4 PS_TextureToTextureResampled(in vtxOut IN) : COLOR0
{
  float2 tc = IN.Tex0.xy;

  half4 c1 = tex2D(Samp, tc + TexToTexParams0.xy);
  half4 c2 = tex2D(Samp, tc + TexToTexParams0.zw);
  half4 c3 = tex2D(Samp, tc + TexToTexParams1.xy);
  half4 c4 = tex2D(Samp, tc + TexToTexParams1.zw);
				
	half4 Color = (c1 + c2 + c3 + c4) * 0.25h;

  return Color;
}

technique TextureToTexture
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_TextureToTexture();	
  }
}

technique TextureToTextureResampled
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_TextureToTextureResampled();	
  }
}

//=========================================================================================

struct vtxOutGauss
{
  float4 Pos    : POSITION;
  float4 Tex0   : TEXCOORD0;
  float4 Tex1   : TEXCOORD1;
  float4 Tex2   : TEXCOORD2;
  float4 Tex3   : TEXCOORD3;
  float2 Tex4   : TEXCOORD4;
};

uniform float4 GaussOffsets[8];
uniform float4 GaussWeights[8];

vtxOutGauss VS_GaussianBlur(in vtxIn IN)
{
  vtxOutGauss OUT = (vtxOutGauss)0;

  OUT.Pos = float4(IN.Pos, 1);
#ifndef OPENGL
  OUT.Pos.xy = OUT.Pos.xy * 2.f - float2(1.f, 1.f);
#endif
  
  OUT.Tex0.xy = IN.Tex0.xy + GaussOffsets[0].xy;
  OUT.Tex0.zw = IN.Tex0.xy + GaussOffsets[1].xy;
  OUT.Tex1.xy = IN.Tex0.xy + GaussOffsets[2].xy;
  OUT.Tex1.zw = IN.Tex0.xy + GaussOffsets[3].xy;
  OUT.Tex2.xy = IN.Tex0.xy + GaussOffsets[4].xy;
  OUT.Tex2.zw = IN.Tex0.xy + GaussOffsets[5].xy;
  OUT.Tex3.xy = IN.Tex0.xy + GaussOffsets[6].xy;
  OUT.Tex3.zw = IN.Tex0.xy + GaussOffsets[7].xy;

	OUT.Tex4.xy = IN.Tex0.xy;

  return OUT;
}

half4 PS_GaussianBlur(in vtxOutGauss IN) : COLOR0
{
  half4 sum = 0;

	half4 col = tex2D(Samp, IN.Tex0.xy);
	sum += col * GaussWeights[0].x;  

	col = tex2D(Samp, IN.Tex0.zw);
	sum += col * GaussWeights[1].x;  
	
  col = tex2D(Samp, IN.Tex1.xy);
	sum += col * GaussWeights[2].x;  

	col = tex2D(Samp, IN.Tex1.zw);
	sum += col * GaussWeights[3].x;

	col = tex2D(Samp, IN.Tex2.xy);
	sum += col * GaussWeights[4].x;  
	
	col = tex2D(Samp, IN.Tex2.zw);
	sum += col * GaussWeights[5].x;  
	
	col = tex2D(Samp, IN.Tex3.xy);
	sum += col * GaussWeights[6].x;  
	
	col = tex2D(Samp, IN.Tex3.zw);
	sum += col * GaussWeights[7].x;

  return sum;
}


technique GaussianBlur
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_GaussianBlur();
		PixelShader = compile PS_SHADER_TYPE PS_GaussianBlur();	
  }
}


half4 PS_Show(in vtxOut IN) : COLOR0
{
  half4 Color = tex2D(Samp, IN.Tex0.xy);

  return Color;
}

technique TShow
{
  pass p0
  {
		VertexShader = compile VS_SHADER_TYPE VS_Quad();
		PixelShader = compile PS_SHADER_TYPE PS_Show();	
  }
}
