
#include "constants.fxh"

float4 vStartShadows = {0.0f, 3.0f, 9.0f, 27.0f};
float  fShadowsFadeDist;

float4x4 mShadowWLP0: ViewLightProjection;
float4x4 mShadowWLP1: ViewLightProjection;
float4x4 mShadowWLP2: ViewLightProjection;
float4x4 mShadowWLP3: ViewLightProjection;

texture Texture3;
texture Texture4;


sampler2D samplerSceneZ = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct VS_INPUT
{
	float4 Pos				: POSITION;
	float2 Tex0				: TEXCOORD0;
	float3 CameraPos	: TEXCOORD1;
};

struct vertexOutput
{
    float4 HPosition	: POSITION;
    float2 Tex0		    : TEXCOORD0;
    float3 CameraPos	: TEXCOORD1;
};

vertexOutput std_VS( const in VS_INPUT In )
{
	vertexOutput Out;
	Out.HPosition = In.Pos;
	Out.Tex0 = In.Tex0;
	Out.CameraPos = In.CameraPos;
	return Out;
}

float4 std_PS(const in vertexOutput IN) : COLOR
{
	float4 light;

	float  cameraZ = tex2D(samplerSceneZ,IN.Tex0);

	float4 worldPos = float4((C_Eye.xyz + IN.CameraPos*cameraZ),1);

	float4 shadow0TC = mul( worldPos, mShadowWLP0 );
	float  light0Z = shadow0TC.z/shadow0TC.w; 
	light.x = (tex2Dproj(samplerSM0, shadow0TC) > light0Z + SHADOW_EPSILON )? 0.0f: 1.0f;  

	float4   shadow1TC = mul( worldPos, mShadowWLP1 );
	float  light1Z = shadow1TC.z/shadow1TC.w; 
	light.y = (tex2Dproj( samplerSM1, shadow1TC) > light1Z + SHADOW_EPSILON  )? 0.0f: 1.0f;  

	float4   shadow2TC = mul( worldPos, mShadowWLP2 );
	float  light2Z = shadow2TC.z/shadow2TC.w; 
	light.z = (tex2Dproj( samplerSM2, shadow2TC) > light2Z + SHADOW_EPSILON  )? 0.0f: 1.0f;  

	float4   shadow3TC = mul( worldPos, mShadowWLP3 );
	float  light3Z = shadow3TC.z/shadow3TC.w; 
	light.w = (tex2Dproj( samplerSM3, shadow3TC) < light3Z + SHADOW_EPSILON  )? 0.0f: 1.0f;  
	return light;    
}

technique DefaultMerge
{
	pass p0 
	{		
		VertexShader = compile vs_2_0 std_VS();
		PixelShader  = compile PS_SHADER_TYPE std_PS();
	}
}
technique DefaultApply
{
	pass p0 
	{		
		VertexShader = compile vs_2_0 std_VS();
		PixelShader  = compile PS_SHADER_TYPE std_PS();
	}
}
