///////////////////////////////////////////////////////////////
//	global

#include "constants.fxh"
#include "lighting.fxh"

float3 vReflect ;
float3 vRefract ;
float4 vNoise0  ;
float4 vNoise1  ; 
float3 WaterColor = {0.02, 0.1, 0.15};;
static const float fScale = 10.0f;

///////////////////////////////////////////////////////////////
//	data
//
struct VpIn20
{
    float4 Pos				: POSITION;
    float3 TexCoord0 		: TEXCOORD0;
    float4 Normal			: NORMAL;
    float4 Binormal			: BINORMAL;
    float4 Tangent			: TANGENT;
};

struct VpOut20
{
    float4 HPos       : POSITION; 
    float4 Eye        : TEXCOORD0;
    float4 Wave0      : TEXCOORD1;
    float4 Wave12     : TEXCOORD2;
    float2 Wave3      : TEXCOORD3;
    float3 ScreenPos  : TEXCOORD4; 
	float4 Inscatter : COLOR0;  
};
struct VpIn14
{
    float4 Pos				: POSITION;
    float4 Normal			: NORMAL;
    float3 TexCoord0 		: TEXCOORD0;
};

struct VpOut14
{
    float4 HPos       : POSITION; 
    float2 Wave0      : TEXCOORD0;
    float2 Wave1      : TEXCOORD1;
    float2 Wave2      : TEXCOORD2;
    float3 Cube      : TEXCOORD3;
    float2 ScreenPos  : TEXCOORD4; 
};

struct VpIn11
{
    float4 Pos				: POSITION;
    float3 TexCoord0 		: TEXCOORD0;
};

struct VpOut11
{
    float4 HPos       : POSITION; 
    float2 Wave0      : TEXCOORD0;
    float2 Wave1      : TEXCOORD1;
    float2 TexCoord0  : TEXCOORD2;
    float2 TexCoord1  : TEXCOORD3; 

};

float scaleTime()
{
	return F_Time.x.x * TimeScale;
}
float3x3 GetTangentSpaceBasis(float3 T, float3 N,float3 B)
{
   float3x3 objToTangentSpace;

   objToTangentSpace[0]=T;
   objToTangentSpace[1]=B;
   objToTangentSpace[2]=N;

   return objToTangentSpace;
}
float Fresnel(float NdotL, float fresnelBias, float fresnelPow)
{
  float facing = (1.0 - NdotL);
  return  max(fresnelBias + (1.0-fresnelBias)*pow(facing, fresnelPow), 0.0);
}

struct Wave {
  float freq;  // 2*PI / wavelength
  float amp;   // amplitude
  float phase; // speed * 2*PI / wavelength
  float2 dir;
};

#define NWAVES 2
Wave wave[NWAVES] = {
	{ 1.0, 1.0, 0.5, float2(-1, 0) },
	{ 2.0, 0.5, 1.3, float2(-0.7, 0.7) }	
};

float evaluateWave(Wave w, float2 pos, float t)
{
  return w.amp * sin( dot(w.dir, pos)*w.freq + t*w.phase);
}

///////////////////////////////////////////////////////////////
//	ps20
//
VpOut20 VS_main20Ocean(VpIn20 IN)
{
    VpOut20 OUT;

    float4 vPos=IN.Pos;

		const float fScaledTime = scaleTime();
     
	vPos.y = 0.0;
	float ddx = 0.0, ddy = 0.0;
	for(int i=0; i<NWAVES; i++) {
    	vPos.y += evaluateWave(wave[i], vPos.xz, fScaledTime);    
    	}
    
	float4 view = mul(float4(vPos.xyz,1), PI_mW);
	view = normalize(C_Eye - view);

	float sceneZ = mul(mul(float4(vPos.xyz,1), PI_mW), C_V).z;
	half3 vDummyExtinction;
	Scattering(view, sceneZ, OUT.Inscatter, vDummyExtinction);

  OUT.HPos = mul(mul(vPos, PI_mW), C_VP);

  float2 fTranslation=float2(fmod(fScaledTime, 100)*0.01, 0);
  float2 vTexCoords=IN.TexCoord0.xy*fScale;

  float  fSinTranslation=sin(fTranslation*100)*0.005;
  float2 vTranslation0=fTranslation+fSinTranslation;
  float2 vTranslation1=fTranslation-fSinTranslation;
  float2 vTranslation2=fTranslation;

  OUT.Wave0.xy = vTexCoords.xy+fTranslation*vNoise0.x;
  OUT.Wave12.xy = vTexCoords.xy*vNoise0.y+fTranslation*vNoise0.z;
  OUT.Wave12.zw = vTexCoords.xy*vNoise0.w+fTranslation*vNoise1.x;
  OUT.Wave3.xy = vTexCoords.xy*vNoise1.y+fTranslation*vNoise1.z;

  float4 vHPos = mul(mul(vPos, PI_mW), C_VP); 
  OUT.Wave0.zw=vHPos.w;

  vHPos.y = -vHPos.y;
  OUT.ScreenPos.xy = (vHPos.xy + vHPos.w)*0.5;    
  OUT.ScreenPos.z =  vHPos.w;      

	//HACK:)
  float3x3 objToTangentSpace=GetTangentSpaceBasis(float3(0,0,1), float3(0,1,0),float3(1,0,0));

  float3 EyeVec=C_Eye.xyz - vPos;        
  OUT.Eye.xyz = mul(objToTangentSpace,EyeVec);
  OUT.Eye.w = 1.0f;
  return OUT;	
}



///////////////////////////////////////////////////////////////
//	ps20
//
VpOut20 VS_main20(VpIn20 IN)
{
  VpOut20 OUT;

  float4 vPos=IN.Pos;
  
	float4 view = mul(float4(vPos.xyz,1), PI_mW);
	view = normalize(C_Eye - view);
	float sceneZ = mul(mul(float4(vPos.xyz,1), PI_mW), C_V).z;
	half3 vDummyExtinction;
	Scattering(view, sceneZ, OUT.Inscatter, vDummyExtinction);

  const float fScaledTime = scaleTime();

	float dist = length(mul(mul(vPos, PI_mW), C_V));
	OUT.Inscatter.xyz = (300.0f - dist) *0.0033f;

  OUT.HPos = mul(mul(vPos, PI_mW), C_VP);

  float2 fTranslation=float2(fmod(fScaledTime, 100)*0.01, 0);
  float2 vTexCoords=IN.TexCoord0.xy*fScale;

  float  fSinTranslation=sin(fTranslation*100)*0.005;
  float2 vTranslation0=fTranslation+fSinTranslation;
  float2 vTranslation1=fTranslation-fSinTranslation;
  float2 vTranslation2=fTranslation;

  OUT.Wave0.xy = vTexCoords.xy+fTranslation*vNoise0.x;
  OUT.Wave12.xy = vTexCoords.xy*vNoise0.y+fTranslation*vNoise0.z;
  OUT.Wave12.zw = vTexCoords.xy*vNoise0.w+fTranslation*vNoise1.x;
  OUT.Wave3.xy = vTexCoords.xy*vNoise1.y+fTranslation*vNoise1.z;

  float4 vHPos = mul(mul(vPos, PI_mW), C_VP); 
  OUT.Wave0.zw=vHPos.w;

  vHPos.y = -vHPos.y;
  OUT.ScreenPos.xy = (vHPos.xy + vHPos.w)*0.5;    
  OUT.ScreenPos.z =  vHPos.w;      

	//HACK:)
  //float3x3 objToTangentSpace=GetTangentSpaceBasis(IN.Tangent.xyz, IN.Normal.xyz,IN.Binormal.xyz);
  float3x3 objToTangentSpace=GetTangentSpaceBasis(float3(0,0,1), float3(0,1,0),float3(1,0,0));

  float3 EyeVec=C_Eye.xyz - vPos;        
  OUT.Eye.xyz = mul(objToTangentSpace,EyeVec);
  OUT.Eye.w = 1.0f;
  return OUT;	
}

float4 PS_main20(VpOut20 IN):COLOR
{
  half3 vEye = normalize(IN.Eye);


  float3 vBumpTexA = tex2D(ColorSampler, IN.Wave0.xy).xyz;
  float3 vBumpTexB = tex2D(ColorSampler, IN.Wave12.xy).xyz;
  float3 vBumpTexC = tex2D(ColorSampler, IN.Wave12.zw).xyz;
  float3 vBumpTexD = tex2D(ColorSampler, IN.Wave3.xy).xyz;
  
  float3 vBumpTex=		normalize(2* (vBumpTexA.xyz + vBumpTexB.xyz + vBumpTexC.xyz + vBumpTexD.xyz)-4.0);
  float3 vRefrBump = 	vBumpTex.xyz * vRefract;
  float3 vReflBump = 	vBumpTex.xyz * vReflect; 
 
  float2 vProj = 		(IN.ScreenPos.xy/IN.ScreenPos.z);
  float4 vReflection = 	tex2D(SecondColorSampler, vProj.xy+ vReflBump.xy);
  float4 vRefrA = 		tex2D(ThirdColorSampler, vProj.xy + vRefrBump.xy);
  float4 vRefrB = 		tex2D(ThirdColorSampler, vProj.xy - vRefrBump.xy);
  float4 vRefraction = 	vRefrB * vRefrA.w + vRefrA * (1-  vRefrA.w);

  float NdotL = 		max(dot(vEye, vReflBump), 0); 
  float facing = 		(1 - NdotL);
  float fresnel = 		Fresnel(NdotL, 0.2, 5.0);
  float fDistScale = 	saturate(vNoise1.w/IN.Wave0.w);

  float3 WaterDeepColor = (vRefraction.xyz * fDistScale + (1 - fDistScale) * half3(0, 0.1, 0.125));  
 
  float3 waterColor = (WaterColor * facing + WaterDeepColor * (1.0 - facing)); // WaterColor;//
  float3 cReflect =  fresnel * vReflection; // vReflection;//

//  return float4(vReflBump,1) ;//return float4(cReflect + waterColor, 1);   
  float4 Color = float4(cReflect + waterColor, 1);  
//  Color.rgb *= IN.Extinction.rgb;//Applyextinction
  Color.rgb = lerp(float3(0.4, 0.58, 0.87),Color.rgb,IN.Inscatter.rgb);//Addinscatter
  return Color;
}

float4 PS_main20Ocean(VpOut20 IN):COLOR
{

  half3 vEye = normalize(IN.Eye);


  float3 vBumpTexA = tex2D(ColorSampler, IN.Wave0.xy).xyz;
  float3 vBumpTexB = tex2D(ColorSampler, IN.Wave12.xy).xyz;
  float3 vBumpTexC = tex2D(ColorSampler, IN.Wave12.zw).xyz;
  float3 vBumpTexD = tex2D(ColorSampler, IN.Wave3.xy).xyz;
  
  float3 vBumpTex=		normalize(2* (vBumpTexA.xyz + vBumpTexB.xyz + vBumpTexC.xyz + vBumpTexD.xyz)-4.0);
  float3 vRefrBump = 	vBumpTex.xyz * vRefract;
  float3 vReflBump = 	vBumpTex.xyz * vReflect; 
 
  float2 vProj = 		(IN.ScreenPos.xy/IN.ScreenPos.z);
  float4 vReflection = 	tex2D(SecondColorSampler, vProj.xy+ vReflBump.xy);
  float4 vRefrA = 		tex2D(ThirdColorSampler, vProj.xy + vRefrBump.xy);
  float4 vRefrB = 		tex2D(ThirdColorSampler, vProj.xy - vRefrBump.xy);
  float4 vRefraction = 	vRefrB * vRefrA.w + vRefrA * (1-  vRefrA.w);

  float NdotL = 		max(dot(vEye, vReflBump), 0); 
  float facing = 		(1 - NdotL);
  float fresnel = 		Fresnel(NdotL, 0.2, 5.0);
  float fDistScale = 	saturate(vNoise1.w/IN.Wave0.w);

  float3 WaterDeepColor = (vRefraction.xyz * fDistScale + (1 - fDistScale) * half3(0, 0.1, 0.125));  
 
  float3 waterColor = (WaterColor * facing + WaterDeepColor * (1.0 - facing)); // WaterColor;//
  float3 cReflect =  fresnel * vReflection; // vReflection;//

//  return float4(vReflBump,1) ;//return float4(cReflect + waterColor, 1);   
  float4 Color = float4(cReflect + waterColor, 1);  
//  Color.rgb *= IN.Extinction.rgb;//Applyextinction
  Color.rgb += IN.Inscatter.rgb;//Addinscatter
  return Color;
}

///////////////////////////////////////////////////////////////
//	ps14
//
VpOut14 VS_main14(VpIn14 IN)
{
	VpOut14 OUT;

	OUT.HPos = mul(mul(IN.Pos, PI_mW), C_VP);

	const float fScaledTime = scaleTime();
	float2 fTranslation={fScaledTime, 0};
	float2 vTexCoords=IN.TexCoord0.xy*fScale;

	float  fSinTranslation=sin(fTranslation*100)*0.005;
	float2 vTranslation0=fTranslation+fSinTranslation;
	float2 vTranslation1=fTranslation-fSinTranslation;
	float2 vTranslation2=fTranslation;


	OUT.Wave0.xy = vTexCoords.xy+fTranslation*vNoise0.x;
	OUT.Wave1.xy = vTexCoords.xy*vNoise0.y+fTranslation*vNoise0.z;
	OUT.Wave2.xy = vTexCoords.xy*vNoise0.w+fTranslation*vNoise1.x;

	OUT.Cube = (IN.TexCoord0);

	OUT.HPos.y = -OUT.HPos.y;
	OUT.ScreenPos.xy = (OUT.HPos.xy + OUT.HPos.w)*0.5/OUT.HPos.w;      
	OUT.HPos.y = -OUT.HPos.y;

	return OUT;
}



float4 PS_main14(VpOut14 IN):COLOR
{
  	float3 vBumpTexA = tex2D(ColorSampler, IN.Wave0.xy).xyz;
  	float3 vBumpTexB = tex2D(ColorSampler, IN.Wave1.xy).xyz;
  	float3 vBumpTexC = tex2D(ColorSampler, IN.Wave2.xy).xyz;


   	float3 vBumpTex =(3*(vBumpTexA+vBumpTexB+ vBumpTexC)-4).xyz;


  	float3 vRefrBump = 	vBumpTex.xyz * vRefract;
  	float3 vReflBump = 	vBumpTex.xyz * vReflect; 


  	float4 vRefraction = 	tex2D(ThirdColorSampler,IN.ScreenPos.xy+vRefrBump.xy);
  	float4 vReflection = 	tex2D(SecondColorSampler,IN.ScreenPos.xy+vReflBump.xy);
 
 	return vRefraction*0.5+vReflection*0.5+WaterColor.xyzz;
}

///////////////////////////////////////////////////////////////
//	ps11
//
VpOut11 VS_main11(VpIn11 IN)
{
	VpOut11 OUT;
	OUT.HPos = mul(mul(IN.Pos, PI_mW), C_VP);

	const float fScaledTime = scaleTime();
	float2 fTranslation={fScaledTime, 0};
	float2 vTexCoords=IN.TexCoord0.xy;

	OUT.Wave0.xy = vTexCoords.xy+fTranslation;
	OUT.Wave1.xy = vTexCoords.xy*.5+fTranslation*3;


	OUT.HPos.y = -OUT.HPos.y;
	OUT.TexCoord0 = (OUT.HPos.xy + OUT.HPos.w)*0.5/OUT.HPos.w;
	OUT.TexCoord1 = (OUT.HPos.xy + OUT.HPos.w)*0.5/OUT.HPos.w;
	OUT.HPos.y = -OUT.HPos.y;

	return OUT;
}

float4 PS_main11(VpOut11 IN):COLOR
{
  	float3 vBumpTexA = tex2D(ColorSampler, IN.Wave0.xy).xyz;
  	float3 vBumpTexB = tex2D(ColorSampler, IN.Wave1.xy).xyz;

  	float4 vRefraction = 	tex2D(ThirdColorSampler,IN.TexCoord0.xy);
  	float4 vReflection = 	tex2D(SecondColorSampler,IN.TexCoord1.xy);
 
 	float4 fragment = vRefraction*1.2+vReflection*0.5;
	
 	return fragment + ((vBumpTexA.x )* 0.2)* vBumpTexB.y;

}
///////////////////////////////////////////////////////////////
//	FX
//
technique ps20
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VS_main20();
		PixelShader = compile PS_SHADER_TYPE PS_main20();
		DEFAULT_STATES
    }
}

technique ps20Ocean
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VS_main20Ocean();
		PixelShader = compile PS_SHADER_TYPE PS_main20Ocean();
		DEFAULT_STATES
    }
}

/*
technique ps14
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VS_main14();
		PixelShader = compile PS_SHADER_TYPE PS_main14();
		DEFAULT_STATES
    }
}
technique ps11
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VS_main11();
		PixelShader = compile PS_SHADER_TYPE PS_main11();
		DEFAULT_STATES
    }
}
*/