#include "constants.fxh"
/////////////////////////////////////// Tweakables ////////

texture2D Tex;
sampler2D blurMap = sampler_state 
{
    texture = <Tex>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

texture2D Tex1;
sampler2D jitterMap = sampler_state 
{
    texture = <Tex>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

uniform float4 blurParams0;
uniform float4 blurParams1;
uniform float4 blurParams2;
uniform float4 blurParams3;

////////////////////////////////////////////////////////////////////////////////////////////////////
/// Anisotropic Blur technique (used for water volumes anisotropic reflections aproximation) ///////

struct vtxIn
{
	float3 Pos    : POSITION;
	float2 baseTC : TEXCOORD0;
};

struct vtxOut
{
  float4 HPosition : POSITION;
  float2 baseTC : TEXCOORD0;    
};

struct vtxOutAnisotropicVertical
{
  float4 HPosition : POSITION;
  float2 baseTC0 : TEXCOORD0;    
  float2 baseTC1 : TEXCOORD1;    
  float2 baseTC2 : TEXCOORD2;    
  float2 baseTC3 : TEXCOORD3;    
  float2 baseTC4 : TEXCOORD4;    
  float2 baseTC5 : TEXCOORD5;    
  float2 baseTC6 : TEXCOORD6;    
  float2 baseTC7 : TEXCOORD7;    
};

vtxOut BaseVS(vtxIn IN)
{
  vtxOut OUT = (vtxOut)0; 

  OUT.HPosition = float4(IN.Pos, 1);
  OUT.HPosition.xy *= F_ScreenSize.xy;
  OUT.HPosition.xy = OUT.HPosition.xy * 2.f - 1.f;

  OUT.baseTC.xy = IN.baseTC.xy;

  return OUT;
}

vtxOutAnisotropicVertical AnisotropicVerticalVS(vtxIn IN)
{
  vtxOutAnisotropicVertical OUT = (vtxOutAnisotropicVertical)0; 

  OUT.HPosition = float4(IN.Pos, 1);
  OUT.HPosition.xy *= F_ScreenSize.xy;
  OUT.HPosition.xy = OUT.HPosition.xy * 2.f - 1.f;

  
  OUT.baseTC0.xy = IN.baseTC.xy + blurParams0.xy*0.125*0.75f;
  OUT.baseTC1.xy = IN.baseTC.xy + blurParams0.zw*0.125*0.75f;
  OUT.baseTC2.xy = IN.baseTC.xy + blurParams1.xy*0.125*0.75f;
  OUT.baseTC3.xy = IN.baseTC.xy + blurParams1.zw*0.125*0.75f;

  OUT.baseTC4.xy = IN.baseTC.xy - blurParams0.xy*0.75f;
  OUT.baseTC5.xy = IN.baseTC.xy - blurParams0.zw*0.75f;
  OUT.baseTC6.xy = IN.baseTC.xy - blurParams1.xy*0.75f;
  OUT.baseTC7.xy = IN.baseTC.xy - blurParams1.zw*0.75f;

  return OUT;
}

///////////////// pixel shader //////////////////
float4 AnisotropicVerticalBlurPS(vtxOutAnisotropicVertical IN) : COLOR
{
  float4 canis = tex2D(blurMap, IN.baseTC0.xy);
  canis += tex2D(blurMap, IN.baseTC1.xy);
  canis += tex2D(blurMap, IN.baseTC2.xy);
  canis += tex2D(blurMap, IN.baseTC3.xy);
  canis += tex2D(blurMap, IN.baseTC4.xy);
  canis += tex2D(blurMap, IN.baseTC5.xy);
  canis += tex2D(blurMap, IN.baseTC6.xy);
  canis += tex2D(blurMap, IN.baseTC7.xy);

  float4 Color = canis / 8.0;
   
  return Color;
}

float4 BlurDirectionalPS(vtxOut IN) : COLOR
{
	float2 tcJitter = (tex2D(jitterMap, IN.baseTC.xy * (F_ScreenSize.xy/64.0))*2-1)*0.03 + 0.985;

	float4 cAcc = half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams0.xy).rgb,1);// * 0.1h;
  cAcc += half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams0.zw).rgb,1);// * 0.2h;

  cAcc += half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams1.xy).rgb,1);// * 0.3h;
  cAcc += half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams1.zw).rgb,1);// * 0.5h;

	cAcc += half4(tex2D(blurMap, IN.baseTC.xy).rgb,1);// * 0.5h;

  cAcc += half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams2.xy).rgb,1);// * 0.5h;
  cAcc += half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams2.zw).rgb,1);// * 0.3h;

  cAcc += half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams3.xy).rgb,1);// * 0.2h;
  cAcc += half4(tex2D(blurMap, IN.baseTC.xy + tcJitter * blurParams3.zw).rgb,1);// * 0.1h;

	// normalize
	cAcc /= cAcc.w;

  float4 Color = cAcc;
   
  return Color;
}

////////////////// technique /////////////////////
technique BlurAniso
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE AnisotropicVerticalVS();
    PixelShader  = compile PS_SHADER_TYPE AnisotropicVerticalBlurPS();

		CullMode = None;
  }
}


technique BlurDirect
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE BaseVS();
    PixelShader  = compile PS_SHADER_TYPE BlurDirectionalPS();    

		CullMode = None;
  }
}
