#include "constants.fxh"
#include "lighting.fxh"

struct vsIn
{
  float4  Position   	: POSITION;
  half2  Tex0        	: TEXCOORD0;
};

struct vsOut
{
  float4 HPosition			: POSITION;

#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW
  FIXED4 FogColor    		: COLOR1;
 #else	
  FIXED FogColor   		  : FOG;
 #endif	
#endif

  float4 Tex0				: TEXCOORD0;
};


vsOut RoadVS(in vsIn In)
{
  vsOut Out = (vsOut)0;

  float4 Po = float4(In.Position.xyz, 1);
  float3 vWorldPos = MUL_VEC_MAT(Po, PI_mW).xyz;

  Out.Tex0.xy = In.Tex0.xy * 8;
  Out.Tex0.zw = In.Tex0.xy;

  float4 vWPos = float4(vWorldPos, 1);
  Out.HPosition = MUL_VEC_MAT(vWPos, C_VP);

#if defined (RT_FOG)
  Out.FogColor = Fog(Out.HPosition.w);
#endif

  return Out;
}

FIXED4 RoadPS(in vsOut IN) : COLOR
{
  half2 baseTC = IN.Tex0.xy;
  half2 detailTC = IN.Tex0.zw;

  FIXED4 cDiffuseMap = tex2D(ColorSampler, baseTC);
  FIXED fOpacity = tex2D(ColorSampler, detailTC).a;

  FIXED4 cOut;
  cOut.xyz = cDiffuseMap.xyz;
  cOut.w = fOpacity;

#if defined(RT_FOG) && RT_PROFILE > QUALITY_VERYLOW
  frag_fog_setup(IN.FogColor, cOut);
#endif

  return cOut;
}

technique Default
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE RoadVS();
    PixelShader = compile PS_SHADER_TYPE RoadPS();
    DEFAULT_STATES
  }
}
