/*

	Screen space Ambient Occlusion

*/

#include "constants.fxh"

uniform float4 SSAO_params = float4(1.f, 0.075f, 2.f, 2.f);

uniform float4 vSSAOAmount = float4(0.f, 1.f, 0.f, 0.f);

sampler2D sRotSampler4x4_16  = sampler_state
{
    Texture = <Texture2>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Wrap;
    AddressV = Wrap;
};

sampler2D sceneDepthSamplerAO = sampler_state
{
    Texture = <Texture1>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};


sampler2D samplerSceneZ = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler2D zMap = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D BlurSrcSampler = sampler_state
{
    Texture = <Texture1>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Mirror;
    AddressV = Mirror;
};

struct VS_INPUT 
{
   float4 Pos : POSITION;
   float2 tc0 : TEXCOORD0;
};

struct VS_INPUT4 
{
   float4 Pos : POSITION;
   float4 tc0 : TEXCOORD0;
};

struct VS_OUTPUT 
{
   float4 Pos : POSITION;
   float2 tc0 : TEXCOORD0;
};

// downsample camera depth

VS_OUTPUT downsample_VS(const in VS_INPUT In)
{
  VS_OUTPUT Out;
  Out.Pos = In.Pos;
  Out.tc0 = In.tc0;
  return Out;
}

float4 downsample_PS(in float2 tc0: TEXCOORD0) : COLOR0
{
   tc0 += F_ScreenSize.xy;
   float sum = tex2D(samplerSceneZ, tc0).r;
   sum = max(sum, tex2D(samplerSceneZ, tc0 - float2(F_ScreenSize.x, 0.f             )).r);
   sum = max(sum, tex2D(samplerSceneZ, tc0 - float2(0.f,            F_ScreenSize.y  )).r);
   sum = max(sum, tex2D(samplerSceneZ, tc0 - float2(F_ScreenSize.x, F_ScreenSize.y  )).r);
   return sum;
}

// SSAO itself

struct vert2fragSSAO
{
   float4 HPosition   :   POSITION;
   float4 ScreenTC    :   TEXCOORD0;
};

vert2fragSSAO Deferred_SSAO_Pass_VS(const in VS_INPUT4 In)
{
   vert2fragSSAO Out;

    Out.HPosition = In.Pos;
    Out.HPosition.xy *= F_ScreenSize.xy;
    Out.HPosition.xy = Out.HPosition.xy * 2.f - 1.f;
    Out.ScreenTC.xyzw = In.tc0.xyzw;

    return Out;
}

half4 Deferred_SSAO_Pass_PS(const in vert2fragSSAO IN) : COLOR0
{
   // define kernel
   const half step = 1.h - 1.h/8.h;
   const half fScale = 0.025h; 
   half n = 0;
   const half3 arrKernel[8] =
   {
      normalize(half3( 1, 1, 1))*fScale*(n+=step),
      normalize(half3(-1,-1,-1))*fScale*(n+=step),
      normalize(half3(-1,-1, 1))*fScale*(n+=step),
      normalize(half3(-1, 1,-1))*fScale*(n+=step),
      normalize(half3(-1, 1 ,1))*fScale*(n+=step),
      normalize(half3( 1,-1,-1))*fScale*(n+=step),
      normalize(half3( 1,-1, 1))*fScale*(n+=step),
      normalize(half3( 1, 1,-1))*fScale*(n+=step),
   };
  
	// create random rot matrix
	const half3 rotSample = tex2D(sRotSampler4x4_16, IN.ScreenTC.zw).rgb * 2.h - 1.h;

  const half fSceneDepth = tex2D( samplerSceneZ, IN.ScreenTC.xy ).r;       

  // range conversions
  const  half fSceneDepthM = fSceneDepth * C_FarClip.x;  

	half3 vSampleScale = SSAO_params.zzw
														* saturate(fSceneDepthM / 5.3h) // make area smaller if distance less than 5 meters
													* (1.h + fSceneDepthM / 9.h ); // make area bigger if distance more than 32 meters

  const float fDepthRangeScale = C_FarClip.x / vSampleScale.z * 0.85f;
   
	// convert from meters into SS units
	vSampleScale.xy *= 1.h / fSceneDepthM;
	vSampleScale.z  *= 2.h * C_FarClip.y;

  const float fDepthTestSoftness = 40.f/vSampleScale.z;

#define fHQScale 0.5h

   // sample
  half4 vSkyAccess = 0.f;
  [unroll]for(int i=0; i<2; i++)
  {    
		half4 arrSceneDepth2[2];
		half3 vIrrSample;

    vIrrSample = reflect(arrKernel[i*4+0], rotSample) * vSampleScale;      
    arrSceneDepth2[0].x = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  
    vIrrSample.xyz *= fHQScale;
    arrSceneDepth2[1].x = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  

    vIrrSample = reflect(arrKernel[i*4+1], rotSample) * vSampleScale;      
    arrSceneDepth2[0].y = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  
    vIrrSample.xyz *= fHQScale;
    arrSceneDepth2[1].y = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  

    vIrrSample = reflect(arrKernel[i*4+2], rotSample) * vSampleScale;      
    arrSceneDepth2[0].z = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  
    vIrrSample.xyz *= fHQScale;
    arrSceneDepth2[1].z = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  

    vIrrSample = reflect(arrKernel[i*4+3], rotSample) * vSampleScale;      
    arrSceneDepth2[0].w = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  
    vIrrSample.xyz *= fHQScale;
    arrSceneDepth2[1].w = tex2D( sceneDepthSamplerAO, IN.ScreenTC.xy + vIrrSample.xy ).r + vIrrSample.z;  

    static const half fDefVal = 0.55h;

    [unroll]for(int s=0; s<2; s++)
    {
      const half4 vDistance = fSceneDepth - arrSceneDepth2[s]; 
      const half4 vDistanceScaled = vDistance * fDepthRangeScale;
      const half4 fRangeIsInvalid = (saturate( abs(vDistanceScaled) ) + saturate( vDistanceScaled )) * .5h;  
      vSkyAccess += lerp(saturate((-vDistance)*fDepthTestSoftness), fDefVal, fRangeIsInvalid);
    }
  }

  half Color = dot( vSkyAccess, .125h ) - SSAO_params.y; // 0.075f

	// amount
	Color = saturate(vSSAOAmount.x + Color * vSSAOAmount.y);

	return Color;
}


struct vert2fragShadow
{
  float4 HPosition  : POSITION;
  float2 baseTC     : TEXCOORD0;
};

vert2fragShadow SSAO_Blur_VS(const in VS_INPUT In)
{
  vert2fragShadow Out;
  Out.HPosition = In.Pos;
  Out.baseTC = In.tc0;
  return Out;
}

half4  SSAO_Blur_PS(in vert2fragShadow IN) : COLOR0
{
  const half2 BlurOffset = F_ScreenSize.xy;
  const half2 centerTC = IN.baseTC.xy + BlurOffset * .5h;

  // low weight center sample - will be used on edges
	half fSumWeight = 0.0125f;
	half cColor = tex2D(BlurSrcSampler, centerTC).r * fSumWeight;

	const half normalizedZ = tex2D(zMap, centerTC).r;
	const float repCenterDepth = 1.f / normalizedZ;

	static const float2 arrOffsets[] =
	{
		float2( 1,-1),
		float2(-1,-1),
		float2( 1, 1),
		float2(-1, 1),
	};

	for(int i=0; i<4; i++)
	{
		const half2 sampleTC = centerTC + BlurOffset * arrOffsets[i];
		const half fDepth = tex2D(zMap, sampleTC).r;
		const half fDiff = (1.f - fDepth * repCenterDepth) * 8.f;
		const half fWeight = saturate(0.5f - 0.75f*abs(fDiff) - 0.25f*(fDiff));
		cColor += tex2D(BlurSrcSampler, sampleTC).r * fWeight;
		fSumWeight += fWeight;
	}

	cColor /= fSumWeight;

	cColor = lerp(cColor, 1.h, normalizedZ*normalizedZ);

	return half4(0, 0, 0, cColor);
}

technique Downsample
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE downsample_VS();
		PixelShader  = compile PS_SHADER_TYPE downsample_PS();
		CullMode = none;
		ZEnable = false;
	}
}

technique Main
{
	pass p0 
	{		
		VertexShader = compile vs_3_0 Deferred_SSAO_Pass_VS();
		PixelShader  = compile ps_3_0 Deferred_SSAO_Pass_PS();
		CullMode = none;
		ZEnable = false;
	}
}

technique Blur
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE SSAO_Blur_VS();
		PixelShader  = compile PS_SHADER_TYPE SSAO_Blur_PS();
		CullMode = none;
		ZEnable = false;
	}
}

//================================================================================================

struct vs_in {
	float3 Pos  : POSITION;
	float2 Tex0 : TEXCOORD0;
};
struct vs_out {
	float4 Pos: POSITION;
	float2 Tex0: TEXCOORD0;
};

sampler sampler0 = sampler_state 
{
    Texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

vs_out VSShow(vs_in In)
{
	vs_out Out;
	Out.Pos = float4(In.Pos.xyz, 1);
#ifdef OPENGL
  Out.Pos.xy = ((Out.Pos.xy+float2(0.5, 0.5)) * F_ScreenSize.xy) * 2.f - 1.f;
#endif  
	Out.Tex0 = In.Tex0;
	return Out;
}

float4 PSShow(vs_out In): COLOR
{
	float4 Col = tex2D(sampler0, In.Tex0).a;
	
	return Col;
}

technique TShow
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSShow();
		PixelShader  = compile PS_SHADER_TYPE PSShow();
	}
}
