/* Shadow map techniques */
#include "fragDef.fxh"
#include "transform.fxh"

const float fAlphaTest = 0.49995f;

struct ShadowMapPSIn
{
	float4 Pos: POSITION;
	float2 Tex0: TEXCOORD0;
};

struct  ShadowMapPSInAlpha
{
	float4 Pos: POSITION;
	float4 Tex0: TEXCOORD0;
};

ShadowMapPSIn VSGenerateShadowMap(in VS_IN In)
{
	ShadowMapPSIn Out;
  float4 Pw = float4(WorldPosition(In), 1);
	float4 HPos = MUL_VEC_MAT(Pw, C_VP);
	float fLinDepth = HPos.z;
/*
#ifdef SHADOW_VARIANCE
  float fDepthShift = 0.5;
	HPos.z *= C_FarClip.y;
	HPos.z -= fDepthShift;
	fLinDepth = HPos.z;
	HPos.z *= HPos.w;
#endif
*/
	Out.Pos = HPos;
	Out.Tex0 = HPos.zw;
	return Out;
}

ShadowMapPSInAlpha VSGenerateShadowMapAlpha(in VS_IN In)
{
	ShadowMapPSInAlpha Out;
  float4 Pw = float4(WorldPosition(In),1);
	Out.Pos = MUL_VEC_MAT(Pw, C_VP);
	Out.Tex0 = float4(Out.Pos.zw, In.Tex0.xy);
	return Out;
}

float4 PSGenerateShadowMap(in ShadowMapPSIn In): COLOR
{
#ifndef SHADOW_VARIANCE
	float depth = (In.Tex0.x / In.Tex0.y + 1.0f) * 0.5f;
	float4 vDepth = float4(depth.xxx, 1.h);
#else
	float depth = In.Tex0.x / In.Tex0.y;
	float4 vDepth = float4(depth.xxx, 1.h);
  vDepth.g = depth * depth;
#endif 
	
	return vDepth;
}

float4 PSGenerateShadowMapAlpha(in ShadowMapPSInAlpha In): COLOR
{
	half alpha = tex2D(ColorSampler, In.Tex0.zw).a;
	clip(alpha - fAlphaTest);
	float depth = (In.Tex0.x / In.Tex0.y + 1.0f) * 0.5f;
	return float4(depth.xxx, alpha);
}

technique Generate
{
	pass p0 
	{		
		VertexShader = compile vs_3_0 VSGenerateShadowMap();
		PixelShader  = compile ps_3_0 PSGenerateShadowMap();
	}
}

technique GenerateAlpha
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE VSGenerateShadowMapAlpha();
		PixelShader  = compile PS_SHADER_TYPE PSGenerateShadowMapAlpha();
	}
}
