/*

	Shadow map techniques

*/

#include "constants.fxh"

float  fScale = 5.0f;

//------------------------------------

sampler2D ShadowMapSampler = sampler_state 
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};


struct vs_in {
	float3 Pos: POSITION;
};

struct vs_inf {
	float3 Pos: POSITION;
	float4 Color: COLOR0;
};

struct vs_in_show {
	float3 Pos: POSITION;
	float2 Tex0: TEXCOORD0;
};


struct vs_out {
	float4 Pos: POSITION;
	float2  Tex0: TEXCOORD0;
};

struct vs_outf {
	float4 Pos: POSITION;
	float4 Color: COLOR0;
};

/*
struct vs_out_show {
	float4 Pos: POSITION;
	float  Tex0: TEXCOORD0;
};
*/

vs_out VSShowShadowMap(vs_in_show In)
{
  vs_out Out;
  Out.Pos = float4(In.Pos.xyz, 1);
#ifdef OPENGL
  Out.Pos.xy = ((Out.Pos.xy+float2(0.5, 0.5)) * F_ScreenSize.xy) * 2.f - 1.f;
  //Out.Pos.y = -Out.Pos.y;
#endif
  Out.Tex0 = In.Tex0;
  return Out;
}

vs_outf VSFrustrum(vs_inf In)
{
	vs_outf Out;
	Out.Pos = mul(float4(In.Pos, 1), C_VP);
	Out.Color = In.Color;
	return Out;
}

/*
float4 PSGenerateShadowMap(vs_out In): COLOR
{
	return float4(In.Tex0.x/In.Tex0.y, In.Tex0.x/In.Tex0.y, In.Tex0.x/In.Tex0.y, In.Tex0.x/In.Tex0.y);
}
*/

float4 PSShowShadowMap(vs_out In): COLOR
{
  float4 Col = tex2D(ShadowMapSampler, In.Tex0); // * 5.0f;
 
  Col.x -= 0.745;
  Col.x *= 100;
  //Col.x = pow(Col.x, 4);
  //Col.x *= 20;
  return float4(Col.x, Col.x, Col.x, 1);
  //return float4(1,1,1,1);

}

float4 PSFrustrum(vs_outf In): COLOR
{
	float4 Col = In.Color;
	return Col;
}

/*
technique Generate
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VSGenerateShadowMap();
		PixelShader  = compile PS_SHADER_TYPE PSGenerateShadowMap();
    }
}
*/

technique Default
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VSShowShadowMap();
		PixelShader  = compile PS_SHADER_TYPE PSShowShadowMap();

	}
}
technique Frustrum
{
  pass p0 
  {		
		VertexShader = compile VS_SHADER_TYPE VSFrustrum();
		PixelShader  = compile PS_SHADER_TYPE PSFrustrum();

  	ZEnable = true;
#ifdef OGL
		CullMode = FrontAndBack;
#else
		CullMode = none;
#endif
	}
}
