#include "transform.fxh"

#define fragInput vertexOutput

extern const float Ksss = 0.5;

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  half cK = pLight.fFallOff * pLight.cFilter;

	// Subsurface scattering aproximation (just simple wrapped diffuse shading)
  half fWrappedNdotL;
	if(pPass.nQuality >= QUALITY_MEDIUM)
		fWrappedNdotL = max(0, (min(pLight.fOcclShadow, pLight.fNdotL) + saturate(Ksss)) / (1.f + saturate(Ksss)) );
	else
		fWrappedNdotL = saturate(pLight.fNdotL);
  const half3 cDiffuse = fWrappedNdotL * pLight.cDiffuse.xyz;
  pPass.cDiffuseAcc += cDiffuse * cK;

	// Specular term  
#ifdef RT_SPECULAR
	half fSpec = Phong(pPass.vNormal, pPass.vView, pLight.vLight, pPass.fSpecPow);
  half3 cSpecular = fSpec * pLight.fOcclShadow;
  if( pPass.nQuality >= QUALITY_HIGH )
	{
		cSpecular *= pLight.cSpecular.xyz;
  	cSpecular *= saturate( pLight.fNdotL * 4.h );  // apply self-shadowing
	}
  pPass.cSpecularAcc += cSpecular * cK;
#endif
}

#include "fragLib.fxh"

half4 PS_Skin(const in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

technique Skin < string name = "Material for human skin. Features: subscattering"; >
{
    pass p0
    {
      VertexShader = compile VS_SHADER_TYPE UberVS();
	    PixelShader = compile PS_SHADER_TYPE PS_Skin();
			DEFAULT_STATES
    }
}
