#include "constants.fxh"

uniform float4 Color1 = float4(0, 0, 0, 0);
uniform float4 Color2 = float4(0, 0, 0, 0);
uniform float4 Color3 = float4(0, 0, 0, 0);
uniform float  Scale = 10.0;
uniform float  Lerp = 0.0;

struct VSOUTPUT
{
  float4 Pos	 : POSITION;
  float2 Tex0  : TEXCOORD0;
};

VSOUTPUT VS(float3 vPos:POSITION, float2 Tex0:TEXCOORD0)
{
	VSOUTPUT Out;
	Out.Pos = mul(mul(float4(vPos.xyz,1), PI_mW), C_VP);
	Out.Tex0 = Tex0;
	
	return Out;
}

half4 PSTexturedWithColor(VSOUTPUT In):COLOR0
{
	return Color1 * tex2D(ColorSampler, In.Tex0);
}

half4 PSTexturedWithColor2(VSOUTPUT In):COLOR0
{
	half4 C1 = Color1 * tex2D(ColorSampler, In.Tex0);
	half4 C2 = Color2 * tex2D(SecondColorSampler, In.Tex0);

  return lerp(C1, C2, Lerp);
}

half4 PSLinearGradient(VSOUTPUT In):COLOR0
{
  half dist = In.Tex0.y;
  if (dist < 0.5h)
		return lerp(Color1, Color2, dist*2.0h);
  else
		return lerp(Color2, Color3, dist*2.0h - 1.0h);
}

half4 PSRadialGradient(VSOUTPUT In):COLOR
{
  half dist = length(In.Tex0 - half2(0.5, 0.5)) *2.0h;
  return lerp(Color1, Color2, dist);
}

technique Textured
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSTexturedWithColor();
		VertexShader = compile VS_SHADER_TYPE VS();
	}
}

technique Textured2
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSTexturedWithColor2();
		VertexShader = compile VS_SHADER_TYPE VS();
	}
}

technique MoonSun
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSTexturedWithColor();
		VertexShader = compile VS_SHADER_TYPE VS();
	}
}

technique RadialGradient
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSRadialGradient();
		VertexShader = compile VS_SHADER_TYPE VS();
	}
}

technique LinearGradient
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSLinearGradient();
		VertexShader = compile VS_SHADER_TYPE VS();
	}
}
