
#include "constants.fxh"
#include "lighting.fxh"

#define NUM_WIND_MATRICES 10
float4x3	WindMatrices[NUM_WIND_MATRICES];

float3 WindEffect(float3 vPosition, float2 vWindInfo)
{
    float2 vWeights = frac(vWindInfo.xy);
    float2 vIndices = (vWindInfo - vWeights) * 0.05f * NUM_WIND_MATRICES;
    float3 vWindEffect = lerp(vPosition.xyz, mul(vPosition, WindMatrices[int(vIndices.x)]), vWeights.x);
	return lerp(vWindEffect, mul(vWindEffect, WindMatrices[int(vIndices.y)]), vWeights.y);
}




void vs_Main(	in float4 inPos : POSITION,
				in float3 inNormal : NORMAL,
				in float2 inUv0 : TEXCOORD,
				in float3 inT : TANGENT,
				in float3 inB : BINORMAL,
				out float4 vPos : POSITION,
				out float2 vUv0 : TEXCOORD0,
				out float4 vDiffuse : COLOR )
{
    inPos.xyz = mul( inPos.xyz, (float3x3)PI_mW);
    inPos.xyz = WindEffect(inPos.xyz, inB.xy);
    inPos.xyz += PI_mW._41_42_43;
	vPos = mul(inPos, C_VP);
	
	vUv0 = inUv0;
	vDiffuse = 1;
	vDiffuse.xyz = F_WorldAmbient.xyz;
#if defined(RT_NUM_LIGHTS)
	vDiffuse.xyz += saturate(dot(inNormal,LGetDir(0).xyz))*LGetDiffuse(0).xyz;
#endif
}

sampler2D smpDiffuse : register(s0);

float4 ps_Main( in float4 vColor : COLOR0,
				in float2 vUv0 : TEXCOORD0 ) : COLOR
{
    float4 vOutColor = vColor * tex2D(smpDiffuse,vUv0);
	return vOutColor;
}


//-----------------------------------
technique Default
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE vs_Main();
		PixelShader = compile PS_SHADER_TYPE ps_Main();		
    }
}