///////////////////////////////////////////////////////////////
//	global

#include "lighting.fxh"

///////////////////////////////////////////////////////////////
//	resource
//
//
float4 Offset;
float3 vReflect = {0.01,0.01,1.0};
float3 vRefract = {0.03,0.03,1.0};
float4 vNoise0 = {1.04,0.389,1.23,0.99 };//= 
float4 vNoise1 = {0.677,0.41,0.90,100.0 };//= 
float3 WaterColor = {0.02, 0.1, 0.15};
static float fScale = 10.0f;

///////////////////////////////////////////////////////////////
//	samplers
samplerCUBE tex2Cube = sampler_state 
{
    texture = <Texture2>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
///////////////////////////////////////////////////////////////
//	data
//
struct VpIn20
{
    float4 Pos				: POSITION;
    float2 TexCoord0 		: TEXCOORD0;
};

struct VpOut20
{
  float4 HPos       : POSITION; 
  float4 Eye        : TEXCOORD0;
  float4 Wave0      : TEXCOORD1;
  float2 Wave1     : TEXCOORD2;
  float2 Wave2      : TEXCOORD3;
  float3 ScreenPos  : TEXCOORD4; 
	float4 Inscatter : COLOR0;
#if defined (RT_FOG) && RT_PROFILE > QUALITY_VERYLOW
	half4 FogColor    			: COLOR1;
#endif
};

float scaleTime(void)
{
	return F_Time.x * TimeScale;
}
float3x3 GetTangentSpaceBasis(float3 T, float3 N,float3 B)
{
   float3x3 objToTangentSpace;

   objToTangentSpace[0]=T;
   objToTangentSpace[1]=B;
   objToTangentSpace[2]=N;

   return objToTangentSpace;
}
float Fresnel(float NdotL, float fresnelBias, float fresnelPow)
{
  float facing = (1.0 - NdotL);
  return  max(fresnelBias + (1.0-fresnelBias)*pow(facing, fresnelPow), 0.0);
}

struct Wave {
  float freq;  // 2*PI / wavelength
  float amp;   // amplitude
  float phase; // speed * 2*PI / wavelength
  float2 dir;
};

#define NWAVES 2
Wave wave[NWAVES] = {
	{ 1.0, 1.0, 0.5, float2(-1, 0) },
	{ 2.0, 0.5, 1.3, float2(-0.7, 0.7) }	
};

float evaluateWave(Wave w, float2 pos, float t)
{
  return w.amp * sin( dot(w.dir, pos)*w.freq + t*w.phase);
}




///////////////////////////////////////////////////////////////
//	ps20
//

VpOut20 VS_main20(VpIn20 IN)
{
  VpOut20 OUT;

	float2 Texcoord0=(IN.Pos.xz) / 32.0f;

  float4 vPos=float4(IN.Pos.xyz,1);
  
	float4 view = vPos;
	const float dist = length(C_Eye.xyz - view.xyz);
	view = normalize(C_Eye - view);

	half4 vDummyExtinction;
	float sceneZ = mul(mul(float4(vPos.xyz,1), PI_mW), C_V).z;
	Scattering(view, sceneZ, OUT.Inscatter, vDummyExtinction);

  float fTime = scaleTime();

	OUT.Inscatter.xyz = (300.0f - dist) *0.0033f;

  OUT.HPos = mul(mul(vPos, PI_mW ), C_VP);

  float fTranslation = fTime*0.017415;
  float2 vTexCoords=Texcoord0;//*fScale;

  float2 vTranslation0=vNoise0.xy*fTranslation;
  float2 vTranslation1=vNoise1.xy*fTranslation;
  float2 vTranslation2=float2(vNoise1.y, vNoise0.x)*fTranslation;

  OUT.Wave0.xy = (vTexCoords.xy + vTranslation0)*vNoise0.z;
  OUT.Wave1.xy = (vTexCoords.xy + vTranslation1)*vNoise1.z;
  OUT.Wave2.xy = (vTexCoords.xy + vTranslation2)*vNoise0.w;

  float4 vHPos = mul(mul(vPos, PI_mW ), C_VP); 
  OUT.Wave0.zw=vHPos.w;

  vHPos.y = -vHPos.y;
  OUT.ScreenPos.xy = (vHPos.xy + vHPos.w)*0.5;    
  OUT.ScreenPos.z =  vHPos.w;      

//HACK:)
  //float3x3 objToTangentSpace=GetTangentSpaceBasis(IN.Tangent.xyz, IN.Normal.xyz,IN.Binormal.xyz);
  float3x3 objToTangentSpace=GetTangentSpaceBasis(float3(0,0,1), float3(0,1,0),float3(1,0,0));

  float3 EyeVec=C_Eye.xyz - vPos;        
  OUT.Eye.xyz = mul(objToTangentSpace,EyeVec);
  OUT.Eye.w = 1.0f;
#if defined (RT_FOG) && RT_PROFILE > QUALITY_VERYLOW
  OUT.FogColor = Fog(dist);
#endif
  return OUT;	
}



float4 PS_CubeMap(VpOut20 IN):COLOR
{

  half3 vWorldView = normalize(IN.Eye);


  float3 vBumpTexA = 2.0f*tex2D(ColorSampler, IN.Wave0.xy).xyz - 1.0f;
  float3 vBumpTexB = 2.0f*tex2D(ColorSampler, IN.Wave1.xy).xyz - 1.0f;
  float3 vBumpTexC = 2.0f*tex2D(ColorSampler, IN.Wave2.xy).xyz - 1.0f;
  
  float3 vBumpTex=		normalize((vBumpTexA.xyz + vBumpTexB.xyz + vBumpTexC.xyz)*2 - 3.0);
  float3 vRefrBump = 	vBumpTex.xyz * vRefract;//
  float3 vReflBump = 	vBumpTex.xyz * vReflect;// 
 
  float3 vReflected = reflect(-vWorldView, vReflBump);
  vReflected.y = - vReflected.y;

 
  float2 vProj = 		(IN.ScreenPos.xy/IN.ScreenPos.z);
  float4 vReflection = 	texCUBE(tex2Cube, vReflected.yzx);
  float4 vRefrA = 		tex2D(SecondColorSampler, vProj.xy + vRefrBump.xy);
 // float4 vRefrB = 		tex2D(SecondColorSampler, vProj.xy - vRefrBump.xy);
 // float4 vRefraction = 	vRefrB * vRefrA.w + vRefrA * (1-  vRefrA.w);
  float4 vRefraction = 	vRefrA;
  
  float NdotL = 		max(dot(vWorldView, vReflBump), 0); 
  float facing = 		(1 - NdotL);
  float fresnel = 		Fresnel(NdotL, 0.2, 5.0);
  float fDistScale = 	saturate(vNoise1.w/IN.Wave0.w);

  float3 WaterDeepColor = (vRefraction.xyz * fDistScale + (1 - fDistScale) * half3(0, 0.1, 0.125));  
 
  float3 waterColor = (WaterColor * facing + WaterDeepColor * (1.0 - facing)); // WaterColor;//
  float3 cReflect =  fresnel * vReflection; // vReflection;//

//  return float4(vReflBump,1) ;//return float4(cReflect + waterColor, 1);   
  float4 Color = float4(cReflect + waterColor, 1);  
//  Color.rgb *= IN.Extinction.rgb;//Applyextinction
  Color.rgb = lerp(float3(0.4, 0.58, 0.87),Color.rgb,IN.Inscatter.rgb);//Addinscatter
	frag_fog_setup(IN.FogColor, Color);
  return Color;
}



float4 PS_main20(VpOut20 IN):COLOR
{

  half3 vWorldView = normalize(IN.Eye);

  float3 vBumpTexA = 2.0f*tex2D(ColorSampler, IN.Wave0.xy).xyz - 1.0f;
  float3 vBumpTexB = 2.0f*tex2D(ColorSampler, IN.Wave1.xy).xyz - 1.0f;
  float3 vBumpTexC = 2.0f*tex2D(ColorSampler, IN.Wave2.xy).xyz - 1.0f;
  
  float3 vBumpTex=		normalize(vBumpTexA.xyz + vBumpTexB.xyz + vBumpTexC.xyz);

  float3 vRefrBump = 	vBumpTex.xyz * vRefract;//
  float3 vReflBump = 	vBumpTex.xyz * vReflect;// 
 
  float2 vProj = 		(IN.ScreenPos.xy/IN.ScreenPos.z);
  float4 vReflection = 	tex2D(SecondColorSampler, vProj.xy+ vReflBump.xy);
  float4 vRefrA = 		tex2D(ThirdColorSampler, vProj.xy + vRefrBump.xy);
  float4 vRefrB = 		tex2D(ThirdColorSampler, vProj.xy - vRefrBump.xy);
  float4 vRefraction = 	vRefrB * vRefrA.w + vRefrA * (1-  vRefrA.w);

  float NdotL = 		max(dot(vWorldView, vReflBump), 0); 
  float facing = 		(1 - NdotL);
  float fresnel = 		Fresnel(NdotL, 0.2, 5.0);
  float fDistScale = 	saturate(vNoise1.w/IN.Wave0.w);

  float3 WaterDeepColor = (vRefraction.xyz * fDistScale + (1 - fDistScale) * half3(0, 0.1, 0.125));  
 
  float3 waterColor = (WaterColor * facing + WaterDeepColor * (1.0 - facing)); // WaterColor;//
  float3 cReflect =  fresnel * vReflection; // vReflection;//

//  return float4(vReflBump,1) ;//return float4(cReflect + waterColor, 1);   
  float4 Color = float4(cReflect + waterColor, 1);  
//  Color.rgb *= IN.Extinction.rgb;//Applyextinction
  Color.rgb = lerp(float3(0.4, 0.58, 0.87),Color.rgb,IN.Inscatter.rgb);//Addinscatter
	frag_fog_setup(IN.FogColor, Color);
  return Color;
}


///////////////////////////////////////////////////////////////
//	FX
//
technique ps20
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VS_main20();
		PixelShader = compile PS_SHADER_TYPE PS_main20();
    }
}


technique ps20cube
{
    pass p0 
    {		
		VertexShader = compile VS_SHADER_TYPE VS_main20();
		PixelShader = compile PS_SHADER_TYPE PS_CubeMap();
    }
}

