/*********************************************************************NVMH3****

******************************************************************************/

#include "VertexDef.fxh"
#include "Lighting.fxh"

#ifdef RT_INSTANCING
  #define  GEN_WORLD_PW3 float4x4 world = float4x4(In.mat0, In.mat1, In.mat2, In.mat3); float3 Pw = MUL_VEC_MAT(Po, world).xyz;
  #define  GEN_WORLD_PW2_HACK float2(In.mat3.x + In.Position.x, In.mat3.z + In.Position.z)
  #define  GEN_WORLD_N   return _Quat_x_Vec3(In.quatWIT, Normal);
#else
  #define  GEN_WORLD_PW3 float3 Pw = MUL_VEC_MAT(Po, PI_mW).xyz;
  #define  GEN_WORLD_PW2_HACK float2(PI_mW[3].x + In.Position.x, PI_mW[3].z + In.Position.z)
  #define  GEN_WORLD_N   return normalize(MUL_VEC_MAT(Normal.xyz, (float3x3)PI_mW).xyz);
#endif


float3 HPosToScreenTC(float4 HPos)
{
  float3 ScreenCoordProj = HPos.xyw;
  ScreenCoordProj.xy = ScreenCoordProj.xy * float2(.5f, -.5f) + (float2(.5f, .5f) + float2(.5f, .5f) * F_ScreenSize.xy) * HPos.w;

  return ScreenCoordProj;
}

///////// VERTEX SHADING /////////////////////

float3 WorldPosition(inout VS_IN In)
{
#ifdef RT_SKINNING
  int4 IndexVector = D3DCOLORtoUBYTE4(In.BoneID);
  // cast the vectors to arrays for use in the for loop below
  float4 BlendWeightsArray = D3DCOLORtoUBYTE4(In.Weight);///255.0f;
  float3 Pw = _SkinningPosition(IndexVector,BlendWeightsArray, In.Position);
#else
  float4 Po = float4(0,0,0,1);
  // bending is temporary disabled due to issues with z-prepass
  Po.xyz = In.Position.xyz;
 #if RT_PROFILE > QUALITY_LOW
  #if defined(BEND_TREE)
      Po.xyz = BendVeg(In);
  #elif defined(BEND_LOCAL)
      Po.xyz = BendVegLocal(In);
  #elif defined(BEND_GRASS)
      Po.xyz = BendGrass(In, GEN_WORLD_PW2_HACK);
  #endif
 #endif	
  GEN_WORLD_PW3
#endif
  return Pw;
}

float3 WorldNormal(const in VS_IN In, const in float3 Normal) 
{
#ifdef RT_SKINNING
  // cast the vectors to arrays for use in the for loop below
  int4 IndexVector = D3DCOLORtoUBYTE4(In.BoneID);
  float4 BlendWeightsArray = D3DCOLORtoUBYTE4(In.Weight);
  return _SkinningNormal(IndexVector, BlendWeightsArray, Normal);
#else
  GEN_WORLD_N
#endif
}

float2 GetTexCoords(const in float2 vSrcTexCoords)
{
  float2 vOutTexCoords = vSrcTexCoords;
  // tex coord animation
#if defined(RT_TEX_SCROLL)
  vOutTexCoords += F_Time.x*vScrollSpeed;
#elif defined(RT_TEX_ANIMATION)
 #ifndef OPENGL
  vOutTexCoords.x += round(F_Time.x * vAnimParams.y);
 #else
  vOutTexCoords.x += (F_Time.x * vAnimParams.y);
 #endif 
  vOutTexCoords.x /= vAnimParams.x;
#elif defined(RT_TEX_TRANSFORM)
  float4 vcHTexCoords = float4(vSrcTexCoords, 1.f, F_Time.x);
  vOutTexCoords.x = dot(vcHTexCoords, mTexTransform[0].xyzw);
  vOutTexCoords.y = dot(vcHTexCoords, mTexTransform[1].xyzw);
#endif
  return vOutTexCoords;
}

#if RT_PROFILE <= QUALITY_LOW
#ifndef LOW_SPEC_CUSTOM_LLIGHTING
 #ifdef RT_NUM_LIGHTS
  half3 GetLowDiffuse(const float3 vWorldPos, const float3 vWorldNormal, const int nTerrain, int nType)
  {
    half3 vDiffuseAcc = half3(0,0,0);

    // Light types
    const int aLType[4] = { RT_LIGHTTYPE0, RT_LIGHTTYPE1, RT_LIGHTTYPE2, RT_LIGHTTYPE3 };
    if (nType >= 2)
    {
      for (int iLight=0; iLight<RT_NUM_LIGHTS; iLight++)
      {
        int nType = aLType[iLight];
        if(nType != LIGHT_DIRECTIONAL)
          continue;
        half3 Diffuse = LGetDiffuse(iLight);
        vDiffuseAcc += Diffuse;
      }
      return vDiffuseAcc;
    }
    
#if RT_PROFILE > QUALITY_VERYLOW    
    for (int iLight=0; iLight<RT_NUM_LIGHTS; iLight++)
    {
      int nType = aLType[iLight];
      if(nType == LIGHT_UNKNOWN)
        continue;
      half3 Diffuse = LGetDiffuse(iLight);
      half3 vDirToLight;
      FIXED fFallOff;
      GetLighting(vDirToLight, fFallOff, vWorldPos.xyz, nType, iLight, nTerrain);
      // Compute diffuse
      half fNdotL = saturate(dot(vDirToLight, vWorldNormal));
      half3 cDiffuse = fNdotL * Diffuse.xyz * fFallOff;
      vDiffuseAcc += cDiffuse;
    }
#else    
    float4 WorldLightPos = LGetPosition(0);
    half3 vVertexSpaceLightPos = WorldLightPos.xyz - vWorldPos;
    half3 vDirToLight = normalize(vVertexSpaceLightPos);
    float fNdotL = dot(vDirToLight.xyz, vWorldNormal);
    vDiffuseAcc.rgb = 1; //saturate(fNdotL) * LGetDiffuse(0);
#endif		
    return vDiffuseAcc;
  }
 #endif
  void LowSpecLighting(inout vertexOutput Out, const in VS_IN In, const float3 vWorldPos, const float3 vWorldNormal, int nType)
  {
    Out.Diffuse = float4(0,0,0,1);
#if RT_PROFILE > QUALITY_VERYLOW    
  #ifdef RT_NUM_LIGHTS
    Out.Diffuse.rgb += GetLowDiffuse(vWorldPos, vWorldNormal.xyz, 0, nType);
   #if defined(RT_STATIC_LIGHTING) && RT_PROFILE > QUALITY_LOW
    Out.Diffuse.rgb *= In.VtxColor.b;
   #endif
  #endif
  #if defined(VERTEX_COLOR)
   #if !defined(RT_STATIC_LIGHTING)
    Out.Diffuse.rgb *= In.VtxColor.rgb;
   #endif	
    Out.Diffuse.a = In.VtxColor.a;
  #endif
  
  #ifndef RT_LIGHTMAP
    if (nType == 2)
      Out.Diffuse.rgb += F_WorldAmbient.rgb;
    else
    if (nType == 3)
    {
      Out.Diffuse.rgb += F_WorldAmbient.rgb;
      Out.Diffuse.rgb *= 0.5f;
    }
    else
    {
   #if defined(DISABLE_RTLIGHTING)
      Out.Diffuse.rgb = F_WorldAmbient.rgb * (normalize(Out.WorldNormal.xyz).y * .25h + .75h);
   #else
      Out.Diffuse.rgb += F_WorldAmbient.rgb * (normalize(Out.WorldNormal.xyz).y * .25h + .75h);
   #endif      
    }
    //Out.Diffuse.rgb *= materialColor.rgb;
  #endif
#else
    if (nType >= 2)
    {
  #ifdef RT_NUM_LIGHTS
    Out.Diffuse.rgb = GetLowDiffuse(vWorldPos, vWorldNormal.xyz, 0, nType);
  #endif      
      if (nType == 2)
        Out.Diffuse.rgb += F_WorldAmbient.rgb;
      else
      if (nType == 3)
      {
        Out.Diffuse.rgb += F_WorldAmbient.rgb;
        Out.Diffuse.rgb *= 0.5f;
      }
    }
    else
      Out.Diffuse.rgb = F_WorldAmbient.rgb /*float4(0.85,0.85,0.85,0)*/ + PI_WColor.rgb;
#endif
  }
#endif
#endif

vertexOutput _UberVS(in VS_IN In, int nType) // 4-building
{
  vertexOutput Out = (vertexOutput)0;

  float3 vWorldPos = WorldPosition(In);
  float3 vWorldNormal = WorldNormal(In, In.Normal);
  
#ifdef RT_SKINNING
 #if RT_PROFILE > QUALITY_VERYLOW && defined(TANGENT_BASIS)
  vWorldPos = In.Position.xyz;
  vWorldNormal = In.Normal;
  int4 IndexVector = D3DCOLORtoUBYTE4(In.BoneID);
  // cast the vectors to arrays for use in the for loop below
  float4 BlendWeightsArray = D3DCOLORtoUBYTE4(In.Weight);///255.0f;
#  if defined(TANGENT_BASIS)	
  float3 vWorldTangent = In.Tangent;
  float3 vWorldBinormal = In.Binormal;
  __SkinningPositionTangent(IndexVector, BlendWeightsArray, vWorldPos, vWorldTangent, vWorldBinormal, vWorldNormal);
#  else
  __SkinningPositionNormal(IndexVector, BlendWeightsArray, vWorldPos, vWorldNormal);
#  endif
 #endif
#elif defined(TANGENT_BASIS)
  float3 vWorldTangent = WorldNormal(In, In.Tangent);
  float3 vWorldBinormal = WorldNormal(In, In.Binormal);
#endif
   
  Out.Tex0.xy = GetTexCoords(In.Tex0.xy);

#if defined(VERTEX_COLOR) && RT_PROFILE > QUALITY_LOW
  Out.VtxColor = In.VtxColor;
#endif

#if defined(RT_LIGHTMAP) || defined(SECONDARY_DIFFUSE)
 #if defined(RT_PLANAR_LIGHTMAP)
  #if RT_PROFILE >= QUALITY_HIGH
  Out.Tex1.xy = (vWorldPos.xz - TerrainPatchOffset.xy) / 32.0f * 64.0f/65.0f;
  Out.Tex1.xy += 0.5f/64.0f;
  Out.Tex1.y = 1 - Out.Tex1.y;
  #else
  Out.Tex1.xy = (vWorldPos.xz - TerrainPatchOffset.xy) / 32.0f * 33.0f/32.0f;
  Out.Tex1.xy = FC_MaskOffset.x + Out.Tex1.xy * FC_MaskOffset.y - 0.5f/32.0f;
  #endif
 #else 
  #if defined(RT_CUSTOM_LIGHTMAP) 	 
    Out.Tex1.xy = (vWorldPos.xz - LightmapCoords.xy) * LightmapCoords.z;
    Out.Tex1.xy += LightmapCoords.w;
    Out.Tex1.y = 1 - Out.Tex1.y;
  #else
    Out.Tex1.xy = In.Tex1.xy;
  #endif	
 #endif
#endif

#if defined(SECONDARY_DIFFUSE) && RT_PROFILE > QUALITY_VERYLOW
  Out.Tex1.zw = In.Tex1.xy;
#endif

#ifdef RT_POINT_SHADOWMAP
  Out.Tex1.xy = In.Tex1.xy;
#endif

#if RT_PROFILE > QUALITY_VERYLOW && !defined(OPENGL)
  Out.WorldNormal.xyz = vWorldNormal;  
 #if defined(TANGENT_BASIS)
  Out.WorldTangent = vWorldTangent;
  Out.WorldBinormal = vWorldBinormal;
 #endif
  Out.WorldView.xyz = MAT_POS(C_VI) - vWorldPos;
 #ifndef OPENGL
 #ifdef RT_IGNORE_BRIGHTNESS
  Out.WorldView.w = 1.0f;
  Out.WorldNormal.w = 1.0f;
  Out.Tex0.z = 1.0f;
 #else
  Out.WorldView.w = PI_mW[0].w + 1.0f;
  Out.WorldNormal.w = PI_mW[1].w + 1.0f;
  Out.Tex0.z = PI_mW[2].w + 1.0f;
 #endif
 #endif
#endif

#if RT_PROFILE > QUALITY_VERYLOW && defined(OPENGL)
  Out.WorldNormal.xyz = vWorldNormal;  
#endif

#if defined (OPENGL)
  #if defined (RT_FRESNEL)
    Out.WorldView.xyz = MAT_POS(C_VI) - vWorldPos;
  #endif
#endif

#if (RT_PROFILE > QUALITY_VERYLOW && !defined (OPENGL)) || defined (RT_SKINNING)
  float4 vWPos = float4(vWorldPos, 1);
  Out.HPosition = MUL_VEC_MAT(vWPos, C_VP);
 #if RT_PROFILE == QUALITY_LOW && defined (RT_UNDERWATER)
  Out.texFog.x = dot(C_FogMatrix[0], vWPos);
  Out.texFog.y = dot(C_FogMatrix[1], vWPos);
  Out.texFog.z = dot(C_FogEnterMatrix[0], vWPos);
  Out.texFog.w = dot(C_FogEnterMatrix[1], vWPos);
 #endif
#else
 #if defined (OPENGL)
  #if defined(RT_LIGHTMAP) && (RT_PROFILE <= QUALITY_LOW)
  Out.HPosition = MUL_VEC_MAT(In.Position, PI_mWVP);
  #else
  float4 vWPos = float4(vWorldPos, 1);
  Out.HPosition = MUL_VEC_MAT(vWPos, C_VP);
  #endif
 #else
  Out.HPosition = MUL_VEC_MAT(In.Position, PI_mWVP);
 #endif
#endif  

#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW && !defined(OPENGL)
  Out.FogColor = Fog(length(Out.WorldView.xyz));
 #else
  Out.FogColor = Fog(Out.HPosition.w);
 #endif	
#endif

#if defined(BUILDING)
  Out.GlobalNontranslatedPos = vWorldPos - PI_mW[3].xyz;
#endif

#if defined (RT_ALPHA_GRADIENT) || defined(RT_DEFERRED) || ((defined(RT_SHADOWS) || defined(RT_SSAO) || defined(USE_SCREENCOORDS)) && RT_PROFILE > QUALITY_LOW)
  // comments in the end it would be: (Out.Pos.xy / Out.Pos.w) * float2(.5f, -.5f) + float2(.5f, .5f) + float2(.5f, .5f) / F_ScreenSize
 #ifdef OPENGL
  Out.ScreenCoordProj = Out.HPosition.xyw;
  Out.ScreenCoordProj.xy = Out.ScreenCoordProj.xy * float2(.5f, .5f) + (float2(.5f, .5f) + float2(.5f, .5f) * F_ScreenSize.xy) * Out.HPosition.w; 
 #else
   Out.ScreenCoordProj = Out.HPosition.xyw;
   Out.ScreenCoordProj.xy = Out.ScreenCoordProj.xy * float2(.5f, -.5f) + (float2(.5f, .5f) + float2(.5f, .5f) * F_ScreenSize.xy) * Out.HPosition.w;
 #endif
#endif

#if RT_PROFILE > QUALITY_LOW
  Out.WorldPos.xyz = vWorldPos;
#endif

#if RT_PROFILE > QUALITY_LOW
  float4 viewPos = MUL_VEC_MAT(float4(vWorldPos, 1), C_V);
  Out.WorldPos.w = viewPos.z * C_FarClip.y;
#else
  LowSpecLighting(Out, In, vWorldPos, vWorldNormal, nType);
#endif

  return Out;
}

vertexOutput UberVS(in VS_IN In)
{
  return _UberVS(In, 0);
}
vertexOutput UberVS2(in VS_IN In)
{
  return _UberVS(In, 1);
}

vertexOutput UberVSC1(in VS_IN In)
{
  return _UberVS(In, 2);
}
vertexOutput UberVSC2(in VS_IN In)
{
  return _UberVS(In, 3);
}

vertexOutput UberBuilding_VS(in VS_IN In)
{
  return _UberVS(In, 4);
}
