#include "transform.fxh"

#define fragInput vertexOutput

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  pLight.fNdotL = saturate( pLight.fNdotL );
  half cK = pLight.fFallOff * pLight.cFilter;

  half3 cDiffuse = min(pLight.fNdotL, pLight.fOcclShadow) * pLight.cDiffuse.xyz;
  pPass.cDiffuseAcc += cDiffuse * cK;

  // Specular term
#ifdef RT_SPECULAR
  half fSpec = Phong(pPass.vNormal, pPass.vView, pLight.vLight, pPass.fSpecPow);
  if( pPass.nQuality >= QUALITY_HIGH )
  fSpec *= saturate( pLight.fNdotL * 4 );  // apply self-shadowing
  half3 cSpecular = pLight.cSpecular.xyz * fSpec * pLight.fOcclShadow;
  pPass.cSpecularAcc += cSpecular * cK;
#endif
}

#include "fragLib.fxh"

float4 PS_Vegetation(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
  return cFinal;
}

technique Vegetation
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE PS_Vegetation();
    DEFAULT_STATES
  }
}
