#define FRAG_CUSTOM_END	// move to RT flag
#define fragInput vertexOutput

#include "transform.fxh"
#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  pLight.fNdotL = saturate( pLight.fNdotL );
  half cK = pLight.fFallOff * pLight.cFilter;

  half3 cDiffuse = min(pLight.fNdotL, pLight.fOcclShadow) * pLight.cDiffuse.xyz;
  pPass.cDiffuseAcc += cDiffuse * cK;

  // Specular term
#ifdef RT_SPECULAR
  half fSpec = Phong(pPass.vNormal, pPass.vView, pLight.vLight, pPass.fSpecPow);
  if( pPass.nQuality >= QUALITY_HIGH )
  fSpec *= saturate( pLight.fNdotL * 4 );  // apply self-shadowing
  half3 cSpecular = fSpec * pLight.fOcclShadow;
  if( pPass.nQuality >= QUALITY_HIGH )
    cSpecular * pLight.cSpecular.xyz;
  pPass.cSpecularAcc += cSpecular * cK;
#endif
}

void frag_custom_end(inout fragPass pPass, inout FIXED3 cFinal)
{
#if defined(VERTEX_COLOR) && RT_PROFILE > QUALITY_LOW
  half4 diffuseColor = lerp(tex2D(SecondColorSampler,pPass.IN.Tex0).rgba,pPass.cDiffuseMap.rgba,pPass.IN.VtxColor.r );
#else
  half4 diffuseColor = pPass.cDiffuseMap.rgba;
#endif
  half3 cDiffuse = ( pPass.cAmbientAcc.xyz + pPass.cDiffuseAcc.xyz ) * diffuseColor.rgb;
  //cDiffuse *= materialColor.rgb;
  cFinal.xyz += cDiffuse;
  
#ifdef RT_SPECULAR
  half3 cSpecular = pPass.cSpecularAcc.xyz * pPass.fSpecularMap;
  //if( pPass.nQuality >= QUALITY_HIGH )
  //	cSpecular *= materialColor.rgb;
  cFinal.xyz += cSpecular;
#endif

  pPass.fAlpha = diffuseColor.a;
}

#include "fragLib.fxh"

float4 PS_Vegetation2(in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
  return cFinal;
}

technique Vegetation2
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE PS_Vegetation2();
    DEFAULT_STATES
  }
}
