#include "transform.fxh"

#define fragInput vertexOutput

#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  pLight.fNdotL = saturate( pLight.fNdotL );

  half3 cDiffuse = min(pLight.fNdotL, pLight.fOcclShadow) * pLight.cDiffuse.xyz;

  // Specular term
	half fSpec = Phong(pPass.vNormal, pPass.vView, pLight.vLight, pPass.fSpecPow);
		
	if( pPass.nQuality >= QUALITY_HIGH )
  	fSpec *= saturate( pLight.fNdotL * 4 );  // apply self-shadowing

  half3 cSpecular = pLight.cSpecular.xyz * fSpec * pLight.fOcclShadow;

  half cK = pLight.fFallOff * pLight.cFilter;
  pPass.cDiffuseAcc += cDiffuse * cK;
  pPass.cSpecularAcc += cSpecular * cK;
}

#include "fragLib.fxh"

float4 PS_VegetationC(const in fragInput IN) : COLOR
{
  // Initialize fragPass structure
  fragPass pPass = (fragPass) 0;
  frag_unify(pPass, IN);  
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

technique VegetationC
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE UberVS();
    PixelShader = compile PS_SHADER_TYPE PS_VegetationC();
  }
}
