/*********************************************************************NVMH3****

******************************************************************************/

#ifndef _VERTEXDEF_DECLARED__
#define _VERTEXDEF_DECLARED__

#include "constants.fxh"
#include "skinning.fxh"
#include "lighting.fxh"

// OpenGL Binding:
// Tangent:  T2
// Binormal: T3
// BLENDINDICES: T4
// BLENDWEIGHTS: T5


//////// CONNECTOR DATA STRUCTURES ///////////
/* data from application vertex buffer */
struct VS_IN
{
  float4  Position    	: POSITION;
  half3  Normal      	: NORMAL;
#ifdef VERTEX_COLOR
  float4 VtxColor    	  : COLOR0;
#endif
  half2  Tex0        	: TEXCOORD0;
#ifdef TANGENT_BASIS
  half3  Tangent     	: TANGENT;
  half3  Binormal    	: BINORMAL;
#endif
#ifdef RT_SKINNING
	#if defined(RT_LIGHTMAP) || defined(SECONDARY_DIFFUSE)
   #if !defined(RT_PLANAR_LIGHTMAP)
    half2 Tex1					: TEXCOORD1;
   #endif 
	#endif
  float4  BoneID				: BLENDINDICES;
  float4  Weight				: BLENDWEIGHT;
#else
	#ifdef RT_INSTANCING
    float4 Tex1					: TEXCOORD1;
	  float4 mat0					: TEXCOORD2;
    float4 mat1					: TEXCOORD3;
    float4 mat2					: TEXCOORD4;
    float4 mat3					: TEXCOORD5;
    float4 quatWIT  		: TEXCOORD6;
    float4 Tex2					: TEXCOORD7;
	#else
    half4 Tex1					: TEXCOORD1;
    half4 Tex2					: TEXCOORD7;
	#endif
#endif
};

#include "bend.fxh"

/* data passed from vertex shader to pixel shader */
struct vertexOutput
{
  float4 HPosition				: POSITION;
#if defined(VERTEX_COLOR) && RT_PROFILE > QUALITY_LOW
	FIXED4 VtxColor					: COLOR0;
#endif

#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW
	FIXED4 FogColor    		: COLOR1;
 #else	
	FIXED FogColor    		  : FOG;
 #endif	
#endif

#if RT_PROFILE > QUALITY_VERYLOW && !defined(OPENGL) && !defined(RT_IGNORE_BRIGHTNESS)
  half3 Tex0				: TEXCOORD0;
#else
  half2 Tex0				: TEXCOORD0;
#endif

#if defined(RT_LIGHTMAP) || defined(SECONDARY_DIFFUSE) || defined(RT_POINT_SHADOWMAP)
 #if defined(SECONDARY_DIFFUSE) && RT_PROFILE > QUALITY_VERYLOW
  half4 Tex1				: TEXCOORD1;
 #else
  half2 Tex1				: TEXCOORD1;
 #endif
#endif
#if RT_PROFILE > QUALITY_VERYLOW
  half4 WorldNormal			: TEXCOORD2;
  float4 WorldView				: TEXCOORD3;
#endif  
  float4 WorldPos					: TEXCOORD4;		// camera depth in w (for fog)
#if defined(TANGENT_BASIS) && RT_PROFILE > QUALITY_VERYLOW
  half3 WorldTangent			: TEXCOORD5;
  half3 WorldBinormal		  : TEXCOORD6;
#endif  
#if defined (RT_ALPHA_GRADIENT) || defined(RT_DEFERRED) || ((defined(RT_SHADOWS) || defined(RT_SSAO) || defined(USE_SCREENCOORDS)) && RT_PROFILE > QUALITY_LOW)
	half3 ScreenCoordProj	: TEXCOORD7;
#endif

#if defined(BUILDING)
	half3 GlobalNontranslatedPos : TEXCOORD8;
#endif

// low spec
#if RT_PROFILE == QUALITY_LOW
	FIXED4 Diffuse						: COLOR0;
 #if defined (RT_UNDERWATER)
	half4 texFog						: TEXCOORD7;
 #endif
#endif
#if RT_PROFILE == QUALITY_VERYLOW
	FIXED4 Diffuse						: TEXCOORD2;
#endif
};

#endif // __VERTEXDEF_DECLARED__
