#include "constants.fxh"
#include "lighting.fxh"

struct vsIn
{
  float4 Position    	: POSITION;
  FIXED4 VtxColor    	  : COLOR0;
  half2  Tex0        	: TEXCOORD0;
};

struct vsOut
{
  float4 HPosition			: POSITION;
  FIXED4 VtxColor			: COLOR0;

#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW
  FIXED4 FogColor    		: COLOR1;
 #else	
  FIXED FogColor    		  : FOG;
 #endif	
#endif

  half2 Tex0				: TEXCOORD0;
};


vsOut VertLightVS(in vsIn In)
{
  vsOut Out = (vsOut)0;

  float4 Po = float4(In.Position.xyz, 1);
  float3 vWorldPos = MUL_VEC_MAT(Po, PI_mW).xyz;

  Out.Tex0.xy = In.Tex0.xy;

  Out.VtxColor = In.VtxColor;

  float4 vWPos = float4(vWorldPos, 1);
  Out.HPosition = MUL_VEC_MAT(vWPos, C_VP);

  float3 WorldView = MAT_POS(C_VI) - vWorldPos;

#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW && !defined(OPENGL)
  Out.FogColor = Fog(length(WorldView.xyz));
 #else
  Out.FogColor = Fog(Out.HPosition.w);
 #endif	
#endif

  return Out;
}

FIXED4 VertLightPS(in vsOut IN) : COLOR
{
  half2 baseTC = IN.Tex0.xy;

  FIXED4 cDiffuseMap = tex2D(ColorSampler, baseTC);

  FIXED fAlpha = cDiffuseMap.a;
  FIXED4 cVertexColor = IN.VtxColor;
  fAlpha *= cVertexColor.a;

  FIXED4 cOut;
  cOut.xyz = cDiffuseMap.xyz * cVertexColor.rgb;
  cOut.w = fAlpha;

#if defined(RT_FOG) && RT_PROFILE > QUALITY_VERYLOW
  frag_fog_setup(IN.FogColor, cOut);
#endif

  return cOut;
}

technique Default
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE VertLightVS();
    PixelShader = compile PS_SHADER_TYPE VertLightPS();
    DEFAULT_STATES
  }
}
