/*
	Water drop effects on the screen
*/

#include "constants.fxh"

sampler2D bumpMap = sampler_state
{
    Texture = <Texture0>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};
sampler2D screenMap = sampler_state
{
    Texture = <Texture0>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = NONE;
    AddressU = Clamp;
    AddressV = Clamp;
};
extern const float4 ParamWaterDrops;

struct VS_INPUT 
{
   float3 Pos : POSITION;
   float2 tc0 : TEXCOORD0;
};

struct vtxOutWaterDrops
{
  float4 HPosition  : POSITION;
  float2 baseTC    : TEXCOORD0;
  float4 noiseTC   : TEXCOORD1;  
};

vtxOutWaterDrops WDP_Pass_VS(VS_INPUT IN)
{
  vtxOutWaterDrops OUT = (vtxOutWaterDrops)0; 

	OUT.HPosition = float4(IN.Pos, 1);
  OUT.HPosition.xy *= F_ScreenSize.xy;
  OUT.HPosition.xy = OUT.HPosition.xy * 2.f - 1.f;

  OUT.baseTC.xy  = IN.tc0.xy;  
  OUT.noiseTC.xy = IN.tc0.xy*0.5;  
  
  // animate height noise
  OUT.noiseTC.zw = IN.tc0.xy*float2(1.0, 0.25)*0.5 + float2(0, -F_Time.w) ;
  OUT.noiseTC.z += (sin(F_Time.w) + sin(F_Time.w*0.5))*0.5 + 0.5;
    
  return OUT;
}

half4 WDP_Pass_PS(vtxOutWaterDrops IN) : COLOR0
{
  half4 OUTColor;
  
  half4 noiseColorA = tex2D(bumpMap, IN.noiseTC.xy);        
  half4 noiseColorB = tex2D(bumpMap, IN.noiseTC.zw);        
  
  half4 screenNormalVec;
  screenNormalVec.z = (noiseColorA.z*2-1)*0.5+ (2*noiseColorB.w-1); // animate z component to simulate water movement + coesion
  screenNormalVec.xy = (noiseColorA.xy*2-1);
  
  // get refraction vector  
  half3 refrUV = normalize(screenNormalVec.xyz*float3(0.01, 0.01, 1));
  
  // fetch backbuffer and displace it
  half4 screenColor = tex2D(screenMap, IN.baseTC.xy - refrUV.xy*0.25 * ParamWaterDrops.w);
  
  // get fake spec highlights
  refrUV = normalize(screenNormalVec.xyz*float3(0.025, 0.025, 1));
  
  half fSpec = saturate(pow(refrUV.x, 9) * 0.5) * ParamWaterDrops.w;
  
  OUTColor.xyz = screenColor.xyz + fSpec;
  OUTColor.w = screenColor.w;

  return OUTColor;
}

technique Main
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE WDP_Pass_VS();
		PixelShader  = compile PS_SHADER_TYPE WDP_Pass_PS();
		CullMode = none;
		ZEnable = false;
	}
}

