/*

	Screen space Ambient Occlusion

*/

#include "constants.fxh"

uniform float4 waterVolumesParams = float4(1, 0.075, 2, 2);

sampler2D samplerNRM = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = None;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VS_INPUT 
{
   float4 Pos : POSITION;
   float2 tc0 : TEXCOORD0;
};

struct vert2fragWNM
{
   float4 Pos         :   POSITION;
   float2 baseTC      :   TEXCOORD0;
};

vert2fragWNM WNM_Pass_VS(in VS_INPUT In)
{
  vert2fragWNM Out;
  Out.Pos = In.Pos;
  Out.baseTC = In.tc0;
  return Out;
}


half4  WNM_Pass_PS(in vert2fragWNM IN) : COLOR0
{
  half4 Color;

  float p0 = tex2D( samplerNRM, IN.baseTC.xy ).z;
  float p1 = tex2D( samplerNRM, IN.baseTC.xy + float2(1,0)/waterVolumesParams.w).z;
  float p2 = tex2D( samplerNRM, IN.baseTC.xy + float2(0,1)/waterVolumesParams.w).z;

  float v0 = p1-p0;
  float v1 = p2-p0;

  float3 vNormal = float3(float2(v0, v1), 4);
  vNormal = normalize(vNormal.xyz);
 
  Color.xyz = vNormal*0.5+0.5;
  Color.w = clamp(p0/20.0, -1.0, 1.0)*0.5+0.5; //vWeights.x*0.5+0.5;
  
  return Color;
}

half4  WNM_Show_NRM_PS(in vert2fragWNM IN) : COLOR0
{
  half4 Color;

  Color = tex2D(samplerNRM, IN.baseTC.xy);
  
  return Color;
}

technique Main
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE WNM_Pass_VS();
		PixelShader  = compile PS_SHADER_TYPE WNM_Pass_PS();
#ifdef OGL
		CullMode = FrontAndBack;
#else
		CullMode = none;
#endif
		ZEnable = false;
	}
}

technique ShowNormals
{
	pass p0 
	{		
		VertexShader = compile VS_SHADER_TYPE WNM_Pass_VS();
		PixelShader  = compile PS_SHADER_TYPE WNM_Show_NRM_PS();
#ifdef OGL
		CullMode = FrontAndBack;
#else
		CullMode = none;
#endif
		ZEnable = false;
	}
}
