#define fragInput vertexOutput
#define RT_TEX_SCROLL
#define FRAG_CUSTOM_END
#define USE_SCREENCOORDS

#include "transform.fxh"
#include "fragDef.fxh"


//========================================================================================================================================================================

struct v2fOcean
{
  float4 Position  : POSITION;  // in projection space
  float4 baseTC    : TEXCOORD0;
  float4 Normal    : TEXCOORD1;
  float4 Wave0     : TEXCOORD2;  

  float4 projTC    : TEXCOORD3;
  float4 viewVec   : TEXCOORD4;  

  float4 FogColor  : TEXCOORD5;
  float4 reflectTC : TEXCOORD6;
#if RT_PROFILE == QUALITY_LOW
  float2 texFog    : TEXCOORD7;
#endif
};

extern const float2 ParamOceanDir;
extern const float ParamOceanWindSpeed;
extern const float ParamOceanWaterLevel;
extern const float ParamOceanWavesAmount;
extern const float ParamOceanWavesSpeed;
extern const float ParamOceanWavesSize;
extern const float SoftIntersectionFactor;
extern const float FresnelScale;
extern const float FresnelBias;
extern const float ReflectionAmount;
extern const float WhiteCapsAmount;
extern const float4x4 ReflectMatrix;

sampler2D fftTexSampler : register(s0);


half GetFresnelOpt(half NdotI, half bias, half power)
{
  half facing = (1.0 - NdotI);
  return saturate(bias + (1-bias)*pow(facing, power));
}

half4	FogWater(in half sceneZ)
{
  half effDist = max(0, (sceneZ - F_FogParams.x)*F_FogParams.y);
	half density = exp(-effDist*effDist);
	half3 fogColor = lerp(F_FogColorEnd.xyz, F_FogColor.xyz, density);
	half fogDensity = saturate(F_FogColorEnd.w*density + F_FogColor.w);
#if RT_PROFILE > QUALITY_VERYLOW
  return half4(fogColor, fogDensity);
#else
  return half4(F_FogColorEnd.xyz, saturate(density));
#endif  
}

// Get vertex from screen space into world spasace
void GetGridVertexPos(inout float4 vPos)
{
  /*if (GetShaderQuality() <= QUALITY_MEDIUM)
  {
    const float fGridQuadSize = 1.0 / 200.0;
    vPos.xz = ((vPos.xz + fGridQuadSize) * 2 - 1);
    vPos.xz *= 2.0;
    vPos.y = 0.0;
    vPos.w = 1.0;
    
    float4 vPosWS = mul(C_VP, vPos);
    vPosWS /= vPosWS.w;

    float3 view = C_Eye.xyz - float3(vPosWS.x, ParamOceanWaterLevel, vPosWS.z);  
    float fDirSign = sign(view.y); // make sure to invert sign when bellow water

    // get z plane intersections
    float2 z_isecs = float2( max(fDirSign*C_Eye.y - fDirSign*vPosWS.y, 0.05/C_FarClip.x), fDirSign*C_Eye.y - fDirSign*ParamOceanWaterLevel);
    
    // clamp t just in case
    float t = min(z_isecs.y / z_isecs.x, C_FarClip.x * 5.0);
            
    // project vertex along ray direction
    float2 vRayDir = vPosWS.xz - C_Eye.xz;        
    vPos.xz = C_Eye.xz + vRayDir.xy*t;
      
    float fHeightDistanceSnap = t / C_FarClip.x;
    
    vPos.y = ParamOceanWaterLevel;
    if (t >= C_FarClip.x * 3.5)
      vPos.y = ParamOceanWaterLevel + fHeightDistanceSnap;
  }
  else*/
  {
    float atten = vPos.z;
    
    float fLod0 = 500 * (1/min(ParamOceanWavesAmount, 2.0));
    fLod0 = (atten > 0.97) ? C_FarClip.x*2 : fLod0;

    // Set a 45 degree rotation to the grid
    vPos.xy = 0.707 * vPos.xy + 0.707 * float2(-vPos.y, vPos.x);

    
    float3 mCam[2];
    mCam[0] = C_VI[0].xyz;
    mCam[1] = -C_VI[2].xyz;

    // select better projection axis if front angle too steep
    const float fFrontThreshold = 0.75;
    float fFrontDotUpAxis = abs(C_VI[2].y);
    if (fFrontDotUpAxis > fFrontThreshold)
    {
      mCam[1] = -C_VI[1].xyz;
    }

    // Orientate along camera direction
    float2 vPosRot;
    float3 vGrid = -fLod0*float3(vPos.x, 0, vPos.y-0.3f);
    vPosRot.x = dot(mCam[0], vGrid);
    vPosRot.y = dot(mCam[1], vGrid);

    float2 cpos = C_Eye.xz; // + vPosRot.xz;
    cpos.xy -= frac(cpos.xy / 2) * 2;
    //float fDistCamera = length(vPosRot.xy);
    //fDistCamera *= fDistCamera*0.085;
    //cpos.xy += normalize(vPosRot.xy)*fDistCamera;
    cpos.xy += vPosRot.xy;
    
#if RT_PROFILE == QUALITY_LOW && defined(RT_UNDERWATER)
    float fHeightDistanceSnap = -5 * (atten > 0.95);  
#else
    float fHeightDistanceSnap = 3 * (atten > 0.97);  
#endif
    vPos = float4(cpos.x, cpos.y, ParamOceanWaterLevel+fHeightDistanceSnap, 1);
  }
}

float4 tex2Dlod_bilinear(in sampler2D s, in float2 t, in float2 textureSize, in float2 texelSize)
{
	float2 f = frac(t * textureSize);

	float4 t00 = tex2Dlod(s,  float4(t, 0, 0));
	float4 t10 = tex2Dlod(s, float4(t + float2(texelSize.x, 0), 0, 0));
	float4 tA = lerp(t00, t10, f.x);

	float4 t01 = tex2Dlod(s, float4(t + float2(0, texelSize.y), 0, 0));
	float4 t11 = tex2Dlod(s, float4(t + float2(texelSize.x, texelSize.y), 0, 0));
	float4 tB = lerp(t01, t11, f.x);

	return lerp(tA, tB, f.y);
}

// Get waves
float3 GetWave(in float3 vPos, out half3 vNormal, in float2 FlowDir, in VS_IN IN)
{
  float3 vDispl = float3(0,0,0);
  vNormal = half3(0,0,1);
  if (GetShaderQuality() <= QUALITY_MEDIUM)
    return vDispl;

  // constant to scale down values a bit
  const float fAnimAmplitudeScale = 1.0 / 5.0;

  float3 vPosOrig = vPos;
  float fPhaseTest = length(vPosOrig.xy);

  // Ocean animation
  const float4 vFrequencies = 5*ParamOceanWavesSpeed * float4(0.233, 0.455, 0.6135, -0.1467);
  const float4 vPhases = ParamOceanWavesAmount * float4(0.1, 0.159, 0.557, 0.2199);
  const float4 vAmplitudes = ParamOceanWavesSize * float4(1, 0.5, 0.25, 0.5); 

  float4 vWaveFreq = F_Time.x * vFrequencies;
  
  float4 vCosPhase = (fPhaseTest + vPosOrig.x)* vPhases;
  float4 vCosWave = cos(vWaveFreq * FlowDir.x + vCosPhase);
  
  float4 vSinPhase = (fPhaseTest + vPosOrig.y)* vPhases; 
  float4 vSinWave = sin(vWaveFreq * FlowDir.y + vSinPhase);

  float fAnimSum = dot(vCosWave, vAmplitudes) + dot(vSinWave, vAmplitudes);

  vDispl.z += fAnimSum * fAnimAmplitudeScale;

  float2 tcFFT = vPos.xy*0.0125*ParamOceanWavesAmount * 1.25;
  float3 v0 = tex2Dlod_bilinear(fftTexSampler, tcFFT, 64, 1.0/64).xyz;
  vDispl.xyz += v0.xyz * 0.06 * ParamOceanWavesSize * float3(1.5, 1.5, 1); 
  
	// Compute normal on the fly (todo: prebake this in texture alpha..)
  float bEdgeDispl = 1.0f-saturate(saturate(IN.Position.z+0.05)*2.0f-1.0f);
	float3 h0 =  v0;
	float3 h10 = tex2Dlod_bilinear(fftTexSampler, tcFFT + float2(1.0h/64.0, 0), 64, 1.0/64.0).xyz;
	float3 h11 = tex2Dlod_bilinear(fftTexSampler, tcFFT + float2(0, 1.0h/64.0), 64, 1.0/64.0).xyz;

	half3 vWeights = half3(h0.z, h10.z, h11.z) * bEdgeDispl;
	vNormal = half3((vWeights.x - vWeights.y), (vWeights.x - vWeights.z), 8); 
	vNormal = normalize(vNormal.xyz);                                               
  
  return vDispl;
}

v2fOcean OceanVS(in VS_IN In)
{
  v2fOcean OUT;
  OUT = (v2fOcean)0; 
  
  int nQuality = GetShaderQuality(); 

  float4 vPos = In.Position;          
  float4 vInPos = In.Position;          
  
  float2 FlowDir = ParamOceanDir;

  // Output bump layers texture coordinates
  float2 vTranslation = (F_Time.x * ParamOceanWindSpeed * 0.0025) * FlowDir;  

  GetGridVertexPos(vPos);
  vPos.w = 1;
  float4 vPosOrig = vPos;

  half3 vNormal;
  float3 vDispl = GetWave(vPos.xyz, vNormal, FlowDir, In);
  OUT.Normal.xyz = vNormal;
  //vPos = vPosOrig;
  
  float3 vView = C_Eye.xzy - vPos.xyz;
  float fDistToCam = length(vView);
  float fD = fDistToCam * 0.25f;
  //vPos.xyz += saturate(fD*fD) * vDispl;
  
  float4 vPosT = float4(vPos.xyz + vDispl, vPos.w);
  float4 HPosition = mul(vPos.xzyw, C_VP);
  vPos.xyz += saturate(0.05 * HPosition.w) * vDispl;

  OUT.Position = mul(vPos.xzyw, C_VP);
  OUT.viewVec.xyz = vView.xyz;
  
  if (fDistToCam < 100)
    OUT.Position.z *= abs(vView.z) > 0.2f;
  
  OUT.reflectTC.w = saturate(2*(vPos.z - ParamOceanWaterLevel)*saturate((-OUT.Normal.x - OUT.Normal.y)*0.9+0.2));
  
  float fWaveDispAmount = abs(vPosOrig.z - vPos.z);
  OUT.Normal.w = fWaveDispAmount;
  
  //OUT.Normal.xy *= saturate(1 - OUT.Position.w*0.0075*0.5 + saturate(0.25 / ParamOceanWavesAmount));
  OUT.viewVec.w = OUT.Position.w * C_FarClip.y;

  // Output projected refraction texture coordinates
  OUT.projTC.xyw = HPosToScreenTC(OUT.Position);
  OUT.projTC.z = OUT.Position.z;
#ifdef RT_ENVMAP
  float4 vReflPos = vPos.xyzw; //float4(min(vPos.xy, C_FarClip.xx*0.9), vPos.zw);
  //if (OUT.projTC.w <= 0.2f)
  //  vReflPos.xy += 100 * (1 - saturate(0.1 * OUT.projTC.w));
  OUT.reflectTC.xyz = mul(vReflPos.xzyw, ReflectMatrix).xyw;
#endif

#ifdef RT_FOG
  OUT.FogColor = FogWater(length(C_VI[3].xyz - vPos.xzy));
#endif

  float2 vTex = vPos.xy * 0.005;

  if (nQuality <= QUALITY_LOW)
	  vTranslation.xy *= 2;
    
  OUT.Wave0.xywz = vTex.xyxy * float4(4, 4, 2, 2) + vTranslation.xyxy * float4(1, 1, 2, 2);
   
  // Output texture coordinates for foam - don't touch texture scales! hand tweaked stuff
  float2 fAnimFoam = frac(F_Time.x * ParamOceanWavesSpeed * 0.01 * float2(0.2, 1)*2);
  OUT.baseTC.xywz = vTex.xyxy * float4(58*0.2, 52*0.2, 38, 42)*3 + fAnimFoam.xxyy * float4(-1, -1, 1, 1);
  
#if RT_PROFILE == QUALITY_LOW && defined(RT_UNDERWATER)
  OUT.texFog.x = dot(C_FogMatrix[0], vPos.xzyw);
  OUT.texFog.y = dot(C_FogMatrix[1], vPos.xzyw);
 #endif
  
  return OUT;  
}

half2 FetchNormalMap(sampler2D bumpMap, float2 bumpTC)
{
  half2 bumpNormal;
  bumpNormal.xy = tex2D(bumpMap, bumpTC.xy).xy;  
  return bumpNormal;
}

half3 BumpGen(in v2fOcean IN)
{
  half3 bumpNormal;
  half3 vVertexNormal = IN.Normal.xyz;  

  // Scale first layer bump, for big waves
  half2 bumpColorA = FetchNormalMap(BumpSampler, IN.Wave0.xy) * 2 - 1; 
  half2 bumpColorB = FetchNormalMap(BumpSampler, IN.Wave0.wz) * 2 - 1;           

  half2 bumpLowFreq;
  bumpLowFreq = (bumpColorA + bumpColorB) ;
                
  half2 bumpColorC = FetchNormalMap(BumpSampler, IN.Wave0.xy*2) * 2 - 1;             
  half2 bumpColorD = FetchNormalMap(BumpSampler, IN.Wave0.wz*2) * 2 - 1;           
  
  half2 bumpHighFreq;
  
  bumpHighFreq = (bumpColorC + bumpColorD);    

  // Mix high and low frequency normals
  if (GetShaderQuality() > QUALITY_LOW) 
  	bumpNormal = half3(bumpLowFreq.xy + bumpHighFreq.xy, 1);
  else
 	  bumpNormal = half3(bumpHighFreq.xy, 1);

  // Clamp max bump to 0.1
  bumpNormal.xy *= 0.1;
  if (GetShaderQuality() > QUALITY_LOW) 
  {
    bumpNormal.z = 1;
    bumpNormal.xyz += vVertexNormal.xyz;
  }
  bumpNormal.xyz = normalize(bumpNormal);

  return bumpNormal;
}

half3 BumpGenLow(in v2fOcean IN)
{
  half3 bumpNormal;

  // Scale first layer bump, for big waves
  half2 bumpColorA = FetchNormalMap(BumpSampler, IN.Wave0.xy) * 2 - 1; 
  half2 bumpColorB = FetchNormalMap(BumpSampler, IN.Wave0.wz*1.5f) * 2 - 1;           

  half2 bumpLowFreq = (bumpColorA + bumpColorB);
                
  // Mix high and low frequency normals
  bumpNormal = half3(bumpLowFreq.xy, 1);

  // Clamp max bump to 0.1
  bumpNormal.xy *= 0.1;
  bumpNormal.xyz = normalize(bumpNormal);

  return bumpNormal;
}

half4 OceanLowQualityPS(v2fOcean IN)
{
#if RT_PROFILE < QUALITY_MEDIUM
  half3 vVertexNormal = IN.Normal;  
  half3 bumpNormal = BumpGenLow(IN);
  
  half3 vFinalNormal = bumpNormal.xyz;
    
  half3 viewVec = normalize(IN.viewVec.xyz);  	
  
  half NdotI = abs(dot(viewVec.xyz, vFinalNormal.xyz));
  half fFresnel = FresnelScale * GetFresnelOpt(NdotI, FresnelBias, 4);
    
  float3 vReflect = (2*dot(viewVec, vFinalNormal)*vFinalNormal - viewVec);
  half3 reflectColor = texCUBE(WaterReflectionSampler, vReflect.xzy).xyz;
    
  half3 final;    
  half fAlpha = 1;
  {
    fFresnel = saturate(fFresnel * ReflectionAmount) + 0.5f;     	         

    final.xyz = reflectColor.xyz;
    fAlpha = fFresnel + saturate(1-NdotI*2);
  }
    
#if RT_PROFILE == QUALITY_LOW && defined(RT_UNDERWATER)
  half4 volFog = tex2D(samplerFog, IN.texFog.xy);
  fAlpha = max(fAlpha, volFog.w);
  half3 volColor = volFog.xyz * F_WaterFogColor.xyz;
  //volFog.w *= 0.5f;
  final.xyz = lerp(final.xyz, volColor.xyz, volFog.w);
#elif defined(RT_FOG) 
	final.xyz = lerp(IN.FogColor.xyz, final.xyz, IN.FogColor.w);
 #if RT_PROFILE == QUALITY_VERYLOW
  fAlpha = max(fAlpha, IN.FogColor.w);
 #endif
#endif
  
  return float4(final, fAlpha);
#else
  return 0;
#endif
}

half4 OceanPS(v2fOcean IN) : COLOR
{
  //return half4(0,0,0,1);
  int nQuality = GetShaderQuality(); 
#ifdef OPENGL
  if (nQuality < QUALITY_MEDIUM)
    return OceanLowQualityPS(IN);
#endif

  half3 vVertexNormal = IN.Normal;  
  half3 bumpNormal = BumpGen(IN);
  
  half3 vFinalNormal = bumpNormal.xyz;
  
  half fFoamGloss = tex2D(SpecularSampler, IN.Wave0.wz*0.5*2 + vFinalNormal.xy*0.1).x;
  fFoamGloss *= fFoamGloss;
  fFoamGloss *= 2;
  
  half fFoam = saturate(tex2D(ColorSampler, IN.baseTC.xy - vFinalNormal.xy*0.1).y);
  fFoam += tex2D(ColorSampler, IN.baseTC.wz + vFinalNormal.xy*0.4).x;
  half fFoamLuminance = saturate(fFoam-0.75);
  
  half3 viewVec = normalize(IN.viewVec.xyz);  	
  float sceneDepth = 100.0f;
  float2 refrNewst = IN.projTC.xy / IN.projTC.w;                    
  if (nQuality >= QUALITY_MEDIUM)
    sceneDepth = tex2D(DepthBufferSampler, refrNewst).r * C_FarClip.x;

 	float waterDepth = IN.projTC.z;
 	float depth = (sceneDepth - waterDepth);
 	float softIntersect = saturate(SoftIntersectionFactor * depth); 	
  softIntersect *= softIntersect;
  
  half NdotI = abs(dot(viewVec.xyz, vFinalNormal.xyz));
  half fFresnel = FresnelScale * GetFresnelOpt(NdotI, FresnelBias, 4);
  
  half fInsideReflection = 0;
  if (nQuality >= QUALITY_HIGH)
    fInsideReflection = saturate(-viewVec.z*1000);
  
#ifdef RT_ENVMAP
  float2 vReflect = IN.reflectTC.xy / IN.reflectTC.z + vFinalNormal.xy*0.025f;
  half4 reflectColor = tex2D(WaterReflectionSampler, vReflect.xy);
#else
  float3 vReflect = (2*dot(viewVec, vFinalNormal)*vFinalNormal - viewVec);
  half4 reflectColor = texCUBE(WaterReflectionSampler, vReflect.xzy);
#endif
  half foamSoftIntersect = saturate((0.1 / saturate(fFoamLuminance)) * depth);      
  
  float2 refrTC = bumpNormal.xy*(0.1+0.1*fInsideReflection)*saturate(softIntersect*4);
  
  IN.viewVec.w += 0.0001; // Refraction mask bias
  
  half3 refractColor = 0;
  if (nQuality >= QUALITY_HIGH)
  {
    // ultra-high quality
    float3 depthRefr = tex2D(DepthBufferSampler, refrNewst + refrTC.xy * 1.05).r;
    depthRefr.y = tex2D(DepthBufferSampler, refrNewst + refrTC.xy).r;
    depthRefr.z = tex2D(DepthBufferSampler, refrNewst + refrTC.xy * 0.95).r;

    half3 fRefractionMask = (IN.viewVec.www < depthRefr.xyz);

    refrTC *= fRefractionMask.x * fRefractionMask.y * fRefractionMask.z; 	  

    refractColor.x = tex2D(BackBufferSampler, refrNewst + refrTC * 1.05).x;
    refractColor.y = tex2D(BackBufferSampler, refrNewst + refrTC).y;
    refractColor.z = tex2D(BackBufferSampler, refrNewst + refrTC * 0.95).z;
  }
  else
  if (nQuality == QUALITY_MEDIUM)
  {
    float depthRefr = tex2D(DepthBufferSampler, refrNewst + refrTC.xy).r;
    half fRefractionMask = (IN.viewVec.w < depthRefr);
    refrTC *= fRefractionMask;

    refractColor.xyz = tex2D(BackBufferSampler, refrNewst + refrTC).xyz;
  }
  else
  {
    refractColor.xyz = tex2D(BackBufferSampler, refrNewst + refrTC).xyz;
  }
    
  half3 final;    
  half fAlpha = 1;
  if (nQuality >= QUALITY_HIGH)
  {
    fFresnel = saturate(fFresnel * ReflectionAmount*(fFoamGloss*0.5+0.5));     	 
    
    float fSoft = softIntersect*saturate(IN.projTC.w);        
    
    if (nQuality >= QUALITY_MEDIUM)
    {    
      half fFoam = smoothstep(0.4, 0.8, 1-foamSoftIntersect)*softIntersect;  
  
      float fFactor = saturate(fFresnel+saturate(1-NdotI*2)*fInsideReflection);
      if (nQuality > QUALITY_MEDIUM)
      {
        half fSoftRefl = saturate(0.1 * depth);
        fFactor *= (1 - smoothstep(0.4, 0.8, 1-fSoftRefl));
      }
      final = lerp(refractColor, reflectColor, fFactor); 
      half3 cFoamFinal = fFoam;

      half fCrestFoam = 0.5 * (fFoamLuminance * fFoamGloss * (IN.Normal.w + IN.reflectTC.w));
      half3 cWavesFoam = fFoamGloss*0.3;
      cFoamFinal = (fFoamGloss*0.5+0.5) * saturate((cFoamFinal + cWavesFoam) * fFoamLuminance + fCrestFoam*WhiteCapsAmount);

      final += F_WorldAmbient.xyz * cFoamFinal;
    }
    else
      final = lerp(refractColor, reflectColor, saturate(fFresnel+saturate(1-NdotI*2)*fInsideReflection)); 
    
    //half fFoam = smoothstep(0.4, 0.8, 1-foamSoft);  
    final.xyz = lerp(refractColor, final.xyz, softIntersect*saturate(IN.projTC.w));
  }
  else
  {
    fFresnel = saturate(fFresnel * ReflectionAmount*(fFoamGloss*0.2+0.8)) + 0.5f;     	         

    //final = reflectColor; 
    if (nQuality == QUALITY_MEDIUM)
    {
      final = lerp(refractColor, reflectColor, saturate(fFresnel+saturate(1-NdotI*2)*fInsideReflection)); 
      half fFoam = smoothstep(0.4, 0.8, 1-foamSoftIntersect)*softIntersect;  
      half3 cFoamFinal = fFoam;
      final += F_WorldAmbient.xyz * cFoamFinal * 0.5;
      final.xyz = lerp(refractColor, final.xyz, softIntersect*saturate(IN.projTC.w));
    }
    else
    {
      final.xyz = reflectColor.xyz;
      fAlpha = fFresnel + saturate(1-NdotI*2);
    }
  }
  //final.xyz = reflectColor.xyz;
    
#if RT_PROFILE == QUALITY_LOW && defined(RT_UNDERWATER)
  half4 volFog = tex2D(samplerFog, IN.texFog.xy);
  fAlpha = max(fAlpha, volFog.w);
  half3 volColor = volFog.xyz * F_WaterFogColor.xyz;
  //volFog.w *= 0.5f;
  final.xyz = lerp(final.xyz, volColor.xyz, volFog.w);
#elif defined(RT_FOG) 
	final.xyz = lerp(IN.FogColor.xyz, final.xyz, IN.FogColor.w);
 #if RT_PROFILE == QUALITY_VERYLOW
  fAlpha = max(fAlpha, IN.FogColor.w);
 #endif
#endif
  
  //final = saturate(0.1 * IN.projTC.w);
  return float4(final, fAlpha);
}

//=============================================================================================================================

technique Ocean < string name = "Material Water Ocean."; >
{
  pass p0
  {
#if RT_PROFILE <= QUALITY_MEDIUM || defined(OPENGL)
    VertexShader = compile VS_SHADER_TYPE OceanVS();
    PixelShader = compile PS_SHADER_TYPE OceanPS();
#else
    VertexShader = compile vs_3_0 OceanVS();
    PixelShader = compile ps_3_0 OceanPS();
#endif    
    DEFAULT_STATES
  	ZEnable = true;
  }
}

//=============================================================================================================================

extern const float4 ViewerNear;

struct v2fOccl
{
  float4 hPos			: POSITION; 
};

v2fOccl OcclusionVS(in VS_IN IN)
{
  v2fOccl OUT = (v2fOccl)0; 

  float3 WorldPos = IN.Position.xyz;
  float4 hPos = mul(float4(WorldPos, 1), C_VP);
  OUT.hPos.z *= ViewerNear.x; // effectively prevent any clipping and depth test rejection by muliplying homogeneous z by zero (if close enough to water surface)

  OUT.hPos = hPos;
  
  return OUT;
}

half4 OcclusionPS(v2fOccl IN) : COLOR
{
  return half4(1,0,0,1);
}


technique OcclusionTest < string name = "Material Water Ocean."; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE OcclusionVS();
    PixelShader = compile PS_SHADER_TYPE OcclusionPS();
    
    DEFAULT_STATES
  	ZEnable = true;
  }
}


//=============================================================================================================================

struct v2fZ
{
  float4 hPos			: POSITION; 
  float2 ZPos			: TEXCOORD0; 
};

v2fZ ZPassVS(in VS_IN IN)
{
  v2fZ OUT = (v2fZ)0; 

  float3 WorldPos = IN.Position.xyz;
  const float4 viewPos = mul(float4(WorldPos, 1), C_V);
	OUT.ZPos.xy = float2(viewPos.z*C_FarClip.y, 1.f);

  float4 hPos = mul(float4(WorldPos, 1), C_VP);
  OUT.hPos = hPos;
  
  return OUT;
}

float4 ZPassPS(v2fZ IN) : COLOR
{
	return float4(IN.ZPos.xxx, 1.0f);
}


technique ZPass < string name = "Material Water Ocean."; >
{
  pass p0
  {
    VertexShader = compile VS_SHADER_TYPE ZPassVS();
    PixelShader = compile PS_SHADER_TYPE ZPassPS();
    
    DEFAULT_STATES
  	ZEnable = true;
  }
}

