// General constants declaration file
// All parameters are ordered by usage frequency

#ifndef _CONSTANTS_DECLARED__
#define _CONSTANTS_DECLARED__

#ifdef OPENGL
#define FIXED fixed
#define FIXED3 fixed3
#define FIXED4 fixed4
#define HALF half
#define HALF3 half3
#define HALF4 half4
#else
#define FIXED half
#define FIXED3 half3
#define FIXED4 half4
#define HALF float
#define HALF3 float3
#define HALF4 float4
#endif

#if !defined (OPENGL) || defined(CG) 
  #define MUL_VEC_MAT(v, m) mul(v, m)
  float3 MAT_POS(float4x4 Mat) { return Mat[3].xyz; }
#else
  #define MUL_VEC_MAT(v, m) mul(m, v)
  float3 MAT_POS(float4x4 Mat) { return float3(Mat[0].w, Mat[1].w, Mat[2].w); }
#endif

//// Defines /////////////////////////////////////////////////////////////////////////
// RT_PROFILE values
#define QUALITY_VERYLOW		0
#define QUALITY_LOW				1
#define QUALITY_MEDIUM		2
#define QUALITY_HIGH			3
#define QUALITY_VERYHIGH	4

//// States /////////////////////////////////////////////////////////////////////////
#ifdef OGL
 #if defined(RT_GLOW) && !defined(RT_RENDER_TO_TEXTURE)
	#define DEFAULT_STATES			ColorMask = { 0, 1, 1, 1 };
 #else
	#define DEFAULT_STATES			ColorMask = { 1, 1, 1, 1 };
 #endif
#else
 #if defined(RT_GLOW) && !defined(RT_RENDER_TO_TEXTURE)
	#define DEFAULT_STATES			ColorWriteEnable = 0x7;
 #else
	#define DEFAULT_STATES			ColorWriteEnable = 0xf;
 #endif
#endif

//// Object properties /////////////////////////////////////////////////////////////////////////
uniform float4x4 PI_mW;
uniform float4x4 PI_mWVP;
uniform HALF4   PI_WColor;

uniform float4 mTexTransform[2];		// only 3x2 submatrix to be used!
// Material properties /////////////////////////////////////////////////////////////////////////
uniform FIXED4 materialColor = {1.0f, 1.0f, 1.0f, 0.0f};
uniform FIXED fOpacity = 1.f;
uniform HALF Ks = 0.6;
uniform HALF Glossiness = 20.0;
uniform HALF4 FresnelBiasScalePow = {0.0f, 1.0f, 2.0f, 0.0f};

uniform float fGlowIntensity = 0.1f;
uniform float fDetailDiffuseScale = 4.f;

//// Per light group parameters (VS/PS) ////////////////////////////////////////////////////////////////
uniform HALF4 FresnelColor = { 1, 1, 1, 1 }; 
uniform float4 L_LightPos0 : register (c18);
uniform HALF4 L_LightDiffuse0 : register (c19);
uniform float4 L_LightDir0 : register (c20);
uniform float4 L_LightPos1 : register (c21);
uniform HALF4 L_LightDiffuse1 : register (c22);
uniform float4 L_LightDir1 : register (c23);
uniform float4 L_LightPos2 : register (c24);
uniform HALF4 L_LightDiffuse2 : register (c25);
uniform float4 L_LightDir2 : register (c26);
uniform float4 L_LightPos3 : register (c27);
uniform HALF4 L_LightDiffuse3 : register (c28);
uniform float4 L_LightDir3 : register (c29);

//// Camera parameters (VS) /////////////////////////////////////////////////////////////////////////
uniform float4x4 C_V : register (c240);
uniform float4x4 C_P : register (c244);
uniform float4x4 C_VP : register (c248);
uniform float4x4 C_VI : register (c252);
uniform float4 C_Eye : register (c29);
uniform float4 C_FarClip : register (c30); // x - far clip; y 1/(far clip); z - near clip; w 1/(near clip)
#if RT_PROFILE == QUALITY_LOW
uniform float2x4 C_FogMatrix : register (c236);
uniform float2x4 C_FogEnterMatrix : register (c238);
#else
uniform float4x4 C_VolReflectMatrix : register (c236);
#endif

//// Per frame parameters //////////////////////////////////////////////////////////////////////
uniform float4 F_Time : register (c16);
float TimeScale = 1.f;
uniform float4 F_ScreenSize : register (c31);	// x,y - reciprocial! of w and h, zw - w and h

// texture coord animation
uniform float4 vScrollSpeed = float4(0.0f, 0.91f, 0, 0);
uniform float4 vAnimParams = float4(3.0f, 10.f, 0, 0);	// x - 1.f / num slices, y - frames per second

// Sun (TODO: clean it up)
uniform float4 F_SunColorIntensity : register (c229) = { 1, 1, 1, 1}; // VS only?
uniform float4 F_BetaDashRayleigh : register (c230);
uniform float4 F_BetaDashMie : register (c231);
uniform float4 F_BetaRayleighMie : register (c232);
uniform float4 F_OneOverBetaRM : register (c233);
uniform float4 F_TermMultiplier : register (c234) = {0.8f, 0.2f, 0.2f, 0};
uniform float4 F_HG : register (c235);

// Fog
uniform float4 F_FogColor : register (c12) = {0.71f, 0.74f, 0.93f, 1.0f};
uniform float4 F_FogColorEnd : register (c13) = {0.71f, 0.74f, 0.93f, 1.0f};
uniform float4 F_FogColorUW : register (c238) = {0.07f, 0.64f, 1.0f, 1.0f};
uniform float4 F_FogParams : register (c14) = {3.0f, 0.000012f, 0, 0.052f}; // x - range start, y - density underwater: z - range start w - density
uniform float4 F_WaterFogColor : register (c16) = {0.44f, 0.44f, 0.44f, 0};

uniform FIXED4 F_WorldAmbient : register (c15) = {0.44f, 0.44f, 0.44f, 1};
uniform float4 F_WorldAmbientShadow : register (c17) = {0.44f, 0.44f, 0.44f, 1};

uniform float4 	vPerInstData;  // Mixed per-instance data: [3] - Number of instructions
uniform float4 	PI_Debug;  // Mixed per-instance data: [3] - Number of instructions

//// Other parameters //////////////////////////////////////////////////////////////////////////
// Skining
#ifdef OGL
static const int MAX_NUM_BONES_PER_GROUP = 90*2;
#else
static const int MAX_NUM_BONES_PER_GROUP = 90*2;
#endif
uniform float4 mWorldQuatT[MAX_NUM_BONES_PER_GROUP] : register (c32);

// Bending
//-----------------------------------------------------------------------------------
uniform float4 windParam = {1.0f, 1.0f, 1.0f, 0.06f};
//extern const float4 windParam = {1.0f, 1.0f, 1.0f, 0.03f};
uniform float maxBend = 0.01395f;
uniform float rcpBend = 0.02f;

//// Parameters ////////////////////

uniform float4 BlendParams = {4.0f, 10.0f, 0.0f, 0.0f};  // BlendFactor, BlendFalloff

uniform float toon_DiffuseStripes = 8;
uniform float toon_LightStripes = 5;
uniform float toon_FresnelStripes = 5;
uniform float toon_EdgeWidth = 1.0;
uniform float4 toon_EdgeColor = float4(0, 0, 0, 1);

uniform float building_DayNight = 0;

uniform float micro_ScaleX = 1;
uniform float micro_OffsetX = 0;
uniform float micro_ScaleY = 1;
uniform float micro_OffsetY = 0;

uniform float relSize = 1;

//// TERRAIN INFO ////////////////////////////
uniform float4 TerrainPatchOffset = float4(0, 0, 0, 1);
uniform float4 LightmapCoords = float4(0, 0, 0, 1);
uniform float4 LightmapTRCoords[4];
//shared extern const float4 TerrainColor = float4(1, 1, 1, 1);
uniform float4 FC_DetailScale : register (c224) = float4(1.f, 1.f, 1.f, 1.f);
uniform float4 FC_DetailMicroScale : register (c225) = 1.f;// = float4(3.f, 5.f, 7.f, 0.f);
uniform float4 FC_MaskOffset : register (c226) = 1.f;// = float4(3.f, 5.f, 7.f, 0.f);
//const float2 MaskOffset = {0.03125f, 0.9375f};
//extern const float fScale = 7.0f;
//extern const float cTileScale = 1.0f;

//////// TEXTURES /////////////////////

texture2D 	Texture0;
texture2D 	Texture1;
texture2D 	Texture2;
texture2D 	Texture3;

texture2D 	BumpTexture;
texture2D 	BumpGlossTexture;
texture2D 	WaterNormalTexture;
textureCUBE	WaterReflectionTexture;
textureCUBE	EnvMapTexture;
texture2D 	LightMapTexture;
texture2D 	OpacityTexture;

sampler2D ColorSampler = sampler_state {
	Texture = <Texture0>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
};

sampler2D BumpSampler = sampler_state {
	Texture = <BumpTexture>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
}; 

sampler2D WaterNormalSampler = sampler_state {
	Texture = <WaterNormalTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Wrap;
	AddressV = Wrap;
}; 

#ifdef OPENGL
samplerCUBE WaterReflectionSampler = sampler_state {
	Texture = <WaterReflectionTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Clamp;
	AddressV = Clamp;
	AddressW = Clamp;
}; 
#else
 #ifdef RT_ENVMAP
sampler2D WaterReflectionSampler = sampler_state {
	Texture = <WaterReflectionTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Wrap;
	AddressV = Wrap;
};
 #else 
samplerCUBE WaterReflectionSampler = sampler_state {
	Texture = <WaterReflectionTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
	AddressU = Clamp;
	AddressV = Clamp;
	AddressW = Clamp;
}; 
 #endif
#endif

sampler2D SecondColorSampler = sampler_state {
	Texture = <Texture1>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
}; 

sampler2D ThirdColorSampler = sampler_state {
	Texture = <Texture2>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
}; 

sampler2D FourthColorSampler = sampler_state {
	Texture = <Texture3>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
}; 

sampler2D SpecularSampler = sampler_state {
	Texture = <BumpGlossTexture>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
}; 

sampler2D OpacitySampler = sampler_state {
	Texture = <OpacityTexture>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Clamp;
	AddressV = Clamp;
}; 
sampler2D LightMapSampler = sampler_state {
	Texture = <LightMapTexture>;
	MinFilter = Linear;
	MipFilter = None;
	MagFilter = Linear;
	AddressU = Clamp;
	AddressV = Clamp;
}; 

samplerCUBE EnvMapSampler = sampler_state {
	Texture = <EnvMapTexture>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Wrap;
	AddressV = Wrap;
	AddressW = Wrap;
}; 
samplerCUBE EnvMapSampler2 = sampler_state {
	Texture = <EnvMapTexture>;
	MinFilter = Linear;
	MipFilter = Linear;
	MagFilter = Linear;
	AddressU = Clamp;
	AddressV = Clamp;
}; 

sampler2D samplerFog : register(s6)  = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};
sampler2D samplerFogEnter : register(s7)  = sampler_state
{
    Texture = <Texture1>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Clamp;
    AddressV = Clamp;
};

sampler2D samplerDiffuseAcc : register(s6) = sampler_state {
	Texture = <Texture0>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Wrap;
	AddressV = Wrap;
};
sampler2D samplerSpecularAcc : register(s7) = sampler_state {
	Texture = <Texture0>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Wrap;
	AddressV = Wrap;
};
sampler2D samplerNormals : register(s8) = sampler_state {
	Texture = <Texture0>;
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

//// Special textures //////////////////////////////////////////////////////////////////////////

texture2D ShadowMaskTexture;
sampler2D ShadowMaskSampler = sampler_state
{
	Texture = <ShadowMaskTexture>;
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
	AddressU = Border;
	AddressV = Border;
#ifdef OPENGL
	//BorderColor = { 1,1,1,1 };
#else	
	BorderColor = 0xffffffff;
#endif	
};

texture2D BackBufferTexture;
sampler2D BackBufferSampler = sampler_state
{
	Texture = <BackBufferTexture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
	AddressU = Clamp;
	AddressV = Clamp;
};

texture2D DepthBufferTexture;
sampler2D DepthBufferSampler : register(s5) = sampler_state
{
	Texture = <DepthBufferTexture>;
	MinFilter = Point;
	MagFilter = Point;
	MipFilter = None;
	AddressU = Border;
	AddressV = Border;
#ifdef OPENGL
	//BorderColor = { 1,1,1,1 };
#else	
	BorderColor = 0xffffffff;
#endif	
};

const float PRECISION = 12.0;
float Pack(float3 color)
{   
  float fPrec = 12.0f;

  float3 cMask = float3(fPrec*fPrec, fPrec, 1.0f);
  float3 vVal = floor(color * fPrec * 0.95f);
  float output = dot(vVal, cMask);

  return output;
}
float3 Unpack(float value)
{   
  float3 color;
  float fPrec = 12.0f;

  value /= fPrec;
  color.b = frac(value);
  value -= color.b;

  value /= fPrec;
  color.g = frac(value);
  value -= color.g;

  color.r = value / fPrec;

  return color;
}

float4 FloatToFloat4(float rawValue)
{	
	float4 unpackFactors = float4(1.0/(PRECISION * PRECISION * PRECISION), 1.0/(PRECISION * PRECISION), 1.0/PRECISION, 1.0);
	float4 vVal = rawValue * unpackFactors;
  float4 ret = 0; //fmod(vVal, PRECISION);
  ret.x = frac(vVal / PRECISION);
  return ret / (PRECISION - 1);
}

float Float4ToFloat(float4 encodedValue)
{
	float4 packFactors = float4(PRECISION*PRECISION*PRECISION, PRECISION*PRECISION, PRECISION, 1.0);
  float4 vVal = floor(encodedValue * (PRECISION - 1));

	return dot(vVal, packFactors);
}


//// Configuration profiles //////////////////////////////////////////////////////////////////////////

int GetShaderQuality() { return RT_PROFILE; }

#ifdef OPENGL
  #define PS_SHADER_TYPE glslf "-po userTexCoord"
  #define VS_SHADER_TYPE glslv "-po userTexCoord"
  //#define PS_SHADER_TYPE arbfp1
  //#define VS_SHADER_TYPE arbvp1 "-po MaxLocalParams=256"
#else
 #if RT_PROFILE == QUALITY_VERYHIGH || RT_PROFILE == QUALITY_HIGH
	#define SM_30
  #define PS_SHADER_TYPE ps_3_0
  #define VS_SHADER_TYPE vs_3_0
 #else 
  #define VS_SHADER_TYPE vs_2_0
  #define PS_SHADER_TYPE ps_2_b
 #endif
#endif

//=======================================================================================================================

#endif // __CONSTANTS_DECLARED__