#include "constants.fxh"

#ifdef fragInput
struct fragPass
{
	fragInput IN;

  // Usage flags fo shared stuff   

  // shared fields
  int nQuality;           // shader quality level
  int nPass;              // shader pass id (used for glass)
  int nType;              // light type { 1: sun+amb; 2: (sun+amb)/2 }
  bool bBuilding;
  
  FIXED4 cShadowOcclMap;		// shadows
  
#if RT_PROFILE > QUALITY_VERYLOW
  half3 mTangentToWS0;  // tangent to world space transformation matrix - might be required for some vectors
  half3 mTangentToWS1;  // tangent to world space transformation matrix - might be required for some vectors
  half3 mTangentToWS2;  // tangent to world space transformation matrix - might be required for some vectors
  half3 vView;           // eye vector
  half3 vReflVec;        // reflection vector
  half fNdotE;           // per pass constant NdotE
  half fNdotE_Toon;
  half fSpecPow;         // specular power
  half3 vNormal;         // normal vector
#endif

#if RT_PROFILE > QUALITY_LOW
  half3 vAmbVec;         // ambient cubemap vector
#endif

  FIXED3 cBumpMap;        // tangent space normal map
  FIXED4 cDiffuseMap;     // diffuse map
  FIXED4 cVertexColor;    // vertex color
  FIXED  fSpecularMap;    // specular map

  FIXED fAlpha;           // opacity   
  
  // final values
  half3 cAmbientAcc;		 // final ambient
  half3 cDiffuseAcc;		 // final diffuse
  half3 cSpecularAcc;		 // final specular

  bool bTest;
  bool bIgnoreShadows;		// flag not to receive shadows
  int  nTerrainIndex;		// flag to detect if it's a usual material terrain shader 
};

struct fragLightPass
{
  int		nType;			// light type
  
  FIXED3 cDiffuse;    // light diffuse color
  FIXED3 cSpecular;   // light specular color
  FIXED3 cFilter;     // light filter color
  half3 vLight;      // light vector
    
  half fNdotL;       // normal dot light
  FIXED fFallOff;     // light attenuation  
  FIXED fOcclShadow;  // light shadow term
};

// Encode a float value into 3 bytes (input value should be in the range of [0, 1])
float3 PackFloatToVec3(const float value)
{
	const float3 bitSh = float3(256.0 * 256.0, 256.0, 1.0);
	const float3 bitMsk = float3(0.0, 1.0/256.0, 1.0/256.0);
	float3 res = frac(value * bitSh);
	res -= res.xxy * bitMsk;
	return res;
}

float UnpackFloatFromVec3(const float3 value)
{
	const float3 bitSh = float3(1.0/(256.0*256.0), 1.0/256.0, 1.0);
	return dot(value, bitSh);
}


#endif