
#ifdef fragInput

#include "Deferred.fxh"

#ifdef RT_DEBUG
half4 DebugOutput(in float4 cFinalColor, inout fragPass pPass)
{
  half4 Color = half4(1,1,1,1);
  if (RT_DEBUG == 1)  // Fillrate output
  {
    Color.xyz = PI_Debug[3];	// Measure overdraw
  }
  else
  if(RT_DEBUG == 2)		// Alpha blend/test output
  {
#if defined(RT_ALPHA_TEST)
    Color.xyz = half3(0, 1, 0);	
#else
 #if defined(RT_BLEND_ALPHA) 
    Color.xyz = half3(1, 0, 0);
 #elif defined(RT_BLEND_ADD) 
    Color.xyz = half3(0, 0, 1);
 #else
    pPass.bTest = true;
 #endif	
#endif    
    Color.a = max(0.1h, cFinalColor.a);
  }
  return Color;
}
#endif

void frag_unify(inout fragPass pPass, in fragInput IN)
{
  // Set quality level (LOW or HIGH)
  pPass.nQuality = GetShaderQuality();
  pPass.nPass = 0;
  pPass.nType = 0;
  pPass.bBuilding = false;
  pPass.nTerrainIndex = 0;
  pPass.fAlpha = 0;
  pPass.bTest = false;

  pPass.cSpecularAcc = 0;
  pPass.cDiffuseAcc = 0;
  pPass.cAmbientAcc = 0;
  
  pPass.IN = IN;
}

void frag_final_composition( inout fragPass pPass, inout FIXED3 cFinal)
{
#if (defined (RT_SPECULAR) || defined(BUILDING)) && RT_PROFILE > QUALITY_VERYLOW
 #ifdef RT_ENVMAP
  // do a reflection cubemap lookup
  #if RT_PROFILE > QUALITY_LOW
  {
    //pPass.fGlossMap = 128;
    #if defined(GLOSS_IN_DIFFUSEALPHA) && RT_PROFILE > QUALITY_MEDIUM // use per-pixel env-map bluring
      half fGlossinessLod = 9.h + log2(1.h-exp2(-1.h/pPass.fSpecPow));
      FIXED3 vEnv = texCUBElod(EnvMapSampler, float4(pPass.vReflVec.xyz, fGlossinessLod)).rgb;
    #else		
      FIXED3 vEnv = texCUBE(EnvMapSampler, pPass.vReflVec).rgb;
     #ifdef OPENGL
      half fFresnelTerm = GetFresnel(pPass.fNdotE_Toon, pPass.nQuality, true);
      vEnv *= fFresnelTerm * Ks;
     #endif
    #endif
/*
    #ifdef BUILDING
      FIXED3 vEnv2 = texCUBE(EnvMapSampler2, pPass.vAmbVec).rgb;
      vEnv = lerp(vEnv, vEnv2, building_DayNight);
    #endif
*/
    #ifdef RT_TOON
      //float ss0 = saturate(dot(float3(0.30, 0.59, 0.11), vEnv.xyz));        
      //int3 vv0 = vEnv.xyz*toon_DiffuseStripes;
      //vEnv.xyz = saturate((float3)vv0/toon_DiffuseStripes*0.75 + ss0.xxx*0.25);
    #endif
  
    #if defined(GLASS) 
     if (pPass.nPass)
     {
       pPass.cSpecularAcc = vEnv;
     }
    #else
     pPass.cSpecularAcc += vEnv;
    #endif
  }
  #endif
 #endif // RT_ENVMAP

  //if(pPass.nQuality >= QUALITY_MEDIUM)
    pPass.cSpecularAcc *= pPass.fSpecularMap;
  if (pPass.nQuality >= QUALITY_HIGH)
    pPass.cSpecularAcc *= materialColor.rgb;
#endif 

#if defined (RT_FRESNEL) && RT_PROFILE > QUALITY_VERYLOW
  // affect specular by a Fresnel term
  bool bSpecular = true;
  #if defined(GLASS) 
   bSpecular = false;
   if (pPass.nPass)
   {
     half fFresnelTerm = GetFresnel(pPass.fNdotE, pPass.nQuality, true);
     pPass.cSpecularAcc *= saturate(fFresnelTerm+0.25h);
   }
  #endif
  if (bSpecular)
  {
    half fFresnelTerm = GetFresnel(pPass.fNdotE_Toon, pPass.nQuality, true);
   #if defined(RT_SPECULAR) || defined(BUILDING) || defined(RT_ENVMAP)
    #if RT_PROFILE >= QUALITY_MEDIUM
     pPass.cSpecularAcc += fFresnelTerm * saturate(pPass.vNormal.y * 2.h) * Ks * FresnelColor;
    #endif
    pPass.fAlpha *= fFresnelTerm;
   #endif // RT_SPECULAR
  }
#endif // RT_FRESNEL

#if (defined(RT_LIGHTMAP) && defined(RT_CUSTOM_LIGHTMAP)) && !defined(NO_LIGHTING) && !defined(TEST) && (defined(RT_SPECULAR) || defined(BUILDING)) && RT_PROFILE > QUALITY_VERYLOW
  pPass.cSpecularAcc *= pPass.cAmbientAcc;
#endif

  half3 cDiffuse = (pPass.cAmbientAcc.xyz + pPass.cDiffuseAcc.xyz) * pPass.cDiffuseMap.xyz * pPass.cVertexColor.rgb;
  if (pPass.nType == 2 && RT_PROFILE > QUALITY_LOW)
    cDiffuse *= 0.5f;
    
#if RT_PROFILE > QUALITY_LOW && !defined(RT_STATIC_LIGHTING) // static lighting from terrain
  cDiffuse *= materialColor.rgb;
#endif		

  bool bLighting = true;
#if defined(GLASS) 
  if (pPass.nPass)
  {
    bLighting = false;
    cFinal = pPass.cSpecularAcc;
  }
#endif
  if (bLighting)
  {
    cFinal += cDiffuse;
 #if (defined(RT_SPECULAR) || defined(RT_ENVMAP) || defined(BUILDING)) && !defined(RT_RENDER_TO_TEXTURE) && RT_PROFILE > QUALITY_VERYLOW
    cFinal += pPass.cSpecularAcc;
 #endif		
  }
  
 #if defined (RT_GLOW)
  // for glowing objects
  cFinal.xyz *= pPass.fAlpha;
 #endif

#if defined(BUILDING)
  if (pPass.bBuilding)
  { 
   #if RT_PROFILE > QUALITY_LOW  
    #ifdef GLOSS_IN_DIFFUSEALPHA // use per-pixel env-map bluring
      half fGlossinessLod = 9.h + log2(1.h-exp2(-1.h/pPass.fSpecPow));
     #if RT_PROFILE > QUALITY_MEDIUM
      half3 vEnvRefl = texCUBElod(EnvMapSampler, float4(pPass.vReflVec.xyz, fGlossinessLod)).rgb;
     #else
      half3 vEnvRefl = texCUBE(EnvMapSampler, pPass.vReflVec).rgb;
     #endif
    #else		
      half3 vEnvRefl = texCUBE(EnvMapSampler, pPass.vReflVec).rgb;
    #endif
    vEnvRefl *= pPass.fAlpha;
    half3 ambVec = normalize(pPass.vAmbVec);
    ambVec.y = ambVec.y*1.707h - 0.707h;
    half3 vEnvAmb = texCUBE(EnvMapSampler2, ambVec).rgb;
    vEnvAmb = lerp(pPass.cAmbientAcc.xyz, vEnvAmb, 1.0 - pPass.cDiffuseAcc);
   #else
    half3 vEnvRefl = half3(0,0,0);
    half3 vEnvAmb = F_WorldAmbient.rgb;
   #endif
    
    half3 cDay = (vEnvAmb.xyz + pPass.cDiffuseAcc.xyz) * pPass.cDiffuseMap.xyz * pPass.cVertexColor.rgb + vEnvRefl;
    
    //half3 cNight = (pPass.cDiffuseMap.xyz * materialColor.xyz * pPass.fAlpha) + vEnvAmb.xyz * (1.0f - pPass.fAlpha);
  float2 co = ceil( pPass.IN.Tex0 );
  float rnd = frac(sin(dot(co,float2(12.9898, 78.233))) * 43758.5453);
  float intencity = step( rnd, 0.5 );    
    half3 cNight = pPass.cDiffuseMap.xyz*lerp(vEnvAmb.xyz, materialColor.xyz*intencity, pPass.fAlpha);
    
    cFinal.xyz = lerp(cDay, cNight, building_DayNight);
  }
#endif
  
  //cFinal = pPass.fGlossMap;
  //pPass.fAlpha = 1;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

half4 frag_shared_output(inout fragPass pPass)
{ 
  FIXED4 cOut = 0;     

  // initial values
  pPass.cDiffuseMap = FIXED4(1,1,1,1);
  pPass.cBumpMap = FIXED3(0,0,1);
  pPass.cShadowOcclMap = 1.h;
#if RT_PROFILE > QUALITY_VERYLOW
  pPass.vNormal = pPass.IN.WorldNormal.xyz;
  pPass.mTangentToWS0 = half3(1, 0, 0);  
  pPass.mTangentToWS1 = half3(0, 1, 0);  
  pPass.mTangentToWS2 = pPass.IN.WorldNormal.xyz;  
#endif  

#if defined(BUILDING) && RT_PROFILE > QUALITY_LOW
  pPass.vAmbVec = pPass.IN.GlobalNontranslatedPos.xyz;
#endif

  pPass.cVertexColor = FIXED4(1,1,1,1);
  pPass.fSpecularMap = Ks;
 
#if (defined(RT_SHADOWS) || defined(RT_SSAO)) && RT_PROFILE > QUALITY_LOW
  pPass.cShadowOcclMap = tex2Dproj(ShadowMaskSampler, pPass.IN.ScreenCoordProj.xyzz);
#endif

#if defined (RT_ALPHA_GRADIENT)
    pPass.cShadowOcclMap = 1;
#endif

  half2 baseTC = pPass.IN.Tex0;
  half2 bumpTC = pPass.IN.Tex0;

#ifdef RT_POINT_SHADOWMAP
  baseTC = pPass.IN.Tex1.xy + frac(pPass.IN.Tex0.xy) * relSize;
#endif

#if defined (TANGENT_BASIS) && RT_PROFILE > QUALITY_VERYLOW
  pPass.mTangentToWS0 = pPass.IN.WorldTangent.xyz;  
  pPass.mTangentToWS1 = pPass.IN.WorldBinormal.xyz;  
  pPass.mTangentToWS2 = pPass.IN.WorldNormal.xyz;  
  if( pPass.nQuality >= QUALITY_HIGH )
  {
    pPass.mTangentToWS0 = normalize(pPass.mTangentToWS0);
    pPass.mTangentToWS1 = normalize(pPass.mTangentToWS1);
    pPass.mTangentToWS2 = normalize(pPass.mTangentToWS2);
  }
#endif

#if defined (VERTEX_COLOR) && RT_PROFILE > QUALITY_VERYLOW
  #if RT_PROFILE > QUALITY_LOW
    pPass.cVertexColor = pPass.IN.VtxColor;
  #else
    pPass.cVertexColor.a = pPass.IN.Diffuse.a;
  #endif
  #if defined(RT_STATIC_LIGHTING)
    pPass.cVertexColor.rgb = float3(1,1,1);
  #endif  
#endif
    //pPass.cVertexColor = 1;

#ifdef DECAL_DIFFUSE
  pPass.cDiffuseMap = tex2D(ColorSampler, baseTC);
  // multiple textures
  #if defined(SECONDARY_DIFFUSE) && defined(MACRO_AND_MICRO)
    half2 tcMicro = half2(pPass.IN.WorldPos.x*micro_ScaleX + micro_OffsetX, pPass.IN.WorldPos.z*micro_ScaleY + micro_OffsetY);
    half4 vSecondColor = tex2D(SecondColorSampler, tcMicro);
    pPass.cDiffuseMap.xyz *= vSecondColor.xyz;
  #elif defined(SECONDARY_DIFFUSE) && defined(VERTEX_COLOR) && RT_PROFILE > QUALITY_VERYLOW
    half4 vSecondColor = tex2D(SecondColorSampler, pPass.IN.Tex1.zw);
    float fA = pPass.cVertexColor.a;
  #if defined(ROAD_BLEND)
    {
      fA *= pPass.cDiffuseMap.a;
      half blendFactor = fA * (1 + BlendParams.x);
      fA = saturate(pow(blendFactor, BlendParams.y));  // Falloff
    }
   #endif
    pPass.cDiffuseMap = lerp(pPass.cDiffuseMap, vSecondColor, fA);
  #endif
#endif

#ifdef RT_TOON
  float ss0 = saturate(dot(float3(0.30, 0.59, 0.11), pPass.cDiffuseMap.xyz));        
  int3 vv0 = pPass.cDiffuseMap.xyz*toon_DiffuseStripes;
  pPass.cDiffuseMap.xyz = saturate((float3)vv0/toon_DiffuseStripes*0.75 + ss0.xxx*0.25);
#endif

#if (defined(BUMP_DIFFUSE) && defined(TANGENT_BASIS)) || defined(RT_SPECULAR)
  if( pPass.nQuality > QUALITY_LOW )
  {
      half4 cBumpData = tex2D(SpecularSampler, bumpTC);

    #ifdef RT_SPECULAR
     #ifdef GLOSS_IN_DIFFUSEALPHA
      //pPass.fGlossMap *= pPass.cDiffuseMap.a;
     #endif
     #ifdef SPECULAR_IN_DIFFUSEALPHA
      pPass.fSpecularMap *= pPass.cDiffuseMap.a;
     #else
      pPass.fSpecularMap *= cBumpData.a;
     #endif
    #endif

    #ifdef TANGENT_BASIS
      pPass.cBumpMap = cBumpData.rgb * 2.h - 1.h;
      if( pPass.nQuality > QUALITY_HIGH )
        pPass.cBumpMap = normalize(pPass.cBumpMap);
    #endif
  }
#elif defined(BUILDING) && defined(SPECULAR_IN_DIFFUSEALPHA)
  pPass.fSpecularMap *= pPass.cDiffuseMap.a;
#endif

  // Get main vectors/coeficients	     
#if RT_PROFILE > QUALITY_VERYLOW
  if( pPass.nQuality > QUALITY_LOW )
  {
 #ifndef TANGENT_BASIS
    pPass.vNormal.xyz = pPass.mTangentToWS2;
 #else
    pPass.vNormal.xyz  = pPass.cBumpMap.x * pPass.mTangentToWS0;
    pPass.vNormal.xyz += pPass.cBumpMap.y * pPass.mTangentToWS1;
    pPass.vNormal.xyz += pPass.cBumpMap.z * pPass.mTangentToWS2;
    if (pPass.nQuality >= QUALITY_HIGH)
      pPass.vNormal = normalize(pPass.vNormal);
 #endif
  }
#endif

#if defined (RT_ALPHA_GRADIENT) || defined(RT_DEFERRED) || ((defined(RT_SHADOWS) || defined(RT_SSAO) || defined(USE_SCREENCOORDS)) && RT_PROFILE > QUALITY_LOW)
 half2 tcScreen = pPass.IN.ScreenCoordProj.xy / pPass.IN.ScreenCoordProj.z;
 #ifdef OPENGL
  tcScreen.y = 1 - tcScreen.y;
 #endif
#endif

#if RT_PROFILE > QUALITY_VERYLOW
  half3 vNormalForward = pPass.vNormal.xyz; 
#endif
#ifdef RT_DEFERRED
  float4 vNormal = GetWorldSpaceNormals(samplerNormals, tcScreen.xy, true);
  pPass.vNormal.xyz = vNormal.xyz;
#endif  

  // Store some constant coeficients
  
#if RT_PROFILE > QUALITY_VERYLOW
  pPass.vView = normalize(pPass.IN.WorldView.xyz);
  pPass.fNdotE = dot(pPass.vView, vNormalForward);
  pPass.fNdotE_Toon = dot(pPass.vView, vNormalForward);
  pPass.fSpecPow = max(4.h, Glossiness);

 #ifdef RT_TOON
  pPass.fNdotE_Toon = (float)((int)(pPass.fNdotE*toon_FresnelStripes)) / toon_FresnelStripes;
 #endif
  pPass.vReflVec = 2 * pPass.fNdotE * vNormalForward - pPass.vView;
#endif

#if defined(RT_BLEND_ALPHA) || defined(RT_BLEND_ADD) || defined(GLASS) 
  pPass.bIgnoreShadows = true; // Disable shadows receiving for transparent objects
#endif

#if defined(RT_SKINNING) && RT_PROFILE <= QUALITY_MEDIUM
  pPass.bIgnoreShadows = true; // Disable shadows receiving for animated objects on medium profile
#endif

#if defined(RT_LIGHTMAP) && defined(RT_SKINNING) && !defined (RT_PLANAR_LIGHTMAP)
  pPass.bIgnoreShadows = true; // Disable shadows receiving for animated objects on medium profile
#endif

#if defined(RT_RENDER_TO_TEXTURE)
  pPass.bIgnoreShadows = true; // No shadows for render-to-texture
#endif

// Set default alpha
pPass.fAlpha = pPass.cDiffuseMap.w
#ifdef RT_OPACITY_TRACK
    * fOpacity
#endif
#if defined (RT_ALPHA_GRADIENT)
    * tex2D(OpacitySampler, tcScreen).r
#elif defined (RT_OPACITY_TEXTURE) && RT_PROFILE > QUALITY_VERYLOW
    * tex2D(OpacitySampler, float2(saturate(pPass.fNdotE * .5h + .5h), 0)).a
#endif
#if !defined(SECONDARY_DIFFUSE) && defined(VERTEX_COLOR)
  * pPass.cVertexColor.a
#endif
    ;

  // Light types
  half3 cDiffuseSun = 0;
#if defined(RT_NUM_LIGHTS) && RT_PROFILE > QUALITY_LOW
  const int aLType[4] = { RT_LIGHTTYPE0, RT_LIGHTTYPE1, RT_LIGHTTYPE2, RT_LIGHTTYPE3 };
  if (pPass.nType)
  {
    for (int iLight=0; iLight<RT_NUM_LIGHTS; iLight++)
    {
      int nType = aLType[iLight];
      if(nType != LIGHT_DIRECTIONAL)
        continue;
      half3 Diffuse = LGetDiffuse(iLight);
      cDiffuseSun += Diffuse;
    }
  }
#endif
  pPass.cDiffuseAcc.xyz = cDiffuseSun;
  
  half fOcclShadow = 1.h;
#if defined(RT_NUM_LIGHTS) && !defined(DISABLE_RTLIGHTING) && RT_PROFILE > QUALITY_LOW
  pPass.cDiffuseAcc = 0;
  for (int iLight=0; iLight<RT_NUM_LIGHTS; iLight++)
  {
    int nType = aLType[iLight];
    if(nType == LIGHT_UNKNOWN)
      continue;

    half3 Diffuse = LGetDiffuse(iLight);
    half3 Specular = LGetSpecular(iLight);

    if (!pPass.bIgnoreShadows && iLight < 3)
    {
      half4 ShadowChanMask = LGetShadowMask(iLight);
      fOcclShadow = dot(pPass.cShadowOcclMap, ShadowChanMask);
    }

    half3 vDirToLight;
    FIXED fFallOff;
    GetLighting(vDirToLight, fFallOff, pPass.IN.WorldPos.xyz, nType, iLight, pPass.nTerrainIndex);

    // Get projector filter color if needed
    half3 filterColor = 1.h;

    // Compute diffuse
    half fNdotL = dot(vDirToLight, pPass.vNormal);
#ifdef RT_TOON
    fNdotL = (float)((int)(fNdotL*toon_LightStripes)) / toon_LightStripes;
#endif

    { 
      // Initialize light pass structure with per-light shared data
      fragLightPass pLight;
      pLight.fNdotL = 0;
      pLight.fFallOff = 0;
      pLight.fOcclShadow = 0;
      pLight.vLight = half3(0,0,0);
      
      pLight.nType = nType;

      pLight.cDiffuse = Diffuse;
      pLight.cSpecular = Specular;
      pLight.vLight = vDirToLight;
      
 #if defined (RT_PLANAR_LIGHTMAP)
      if (nType == LIGHT_DIRECTIONAL)
      {
        half4 lm = tex2D(LightMapSampler, pPass.IN.Tex1.xy); // g-terrain shadow, r-static shadows, b-sky accessibility, a-sun accessibility
        if (pPass.nQuality >= QUALITY_HIGH)
          fOcclShadow *= lm.g * lm.r; // use lm from static objects and terrain;
        else
          fOcclShadow *= lm.a; // use lm from static objects and terrain;
      }
 #elif defined (RT_STATIC_LIGHTING) && defined(VERTEX_COLOR)
      if (nType == LIGHT_DIRECTIONAL)
      { // Grass lightmap
        half4 lm = pPass.IN.VtxColor; // b-static brightness
        fOcclShadow = min(fOcclShadow, lm.b);          // use lm from static objects and terrain;
        fNdotL = 1;
        pPass.IN.VtxColor.b = 1;
        fFallOff = 1;
      }
 #elif defined (RT_LIGHTMAP) // Terrain lightmap
      if (pPass.nTerrainIndex && nType == LIGHT_DIRECTIONAL)
      {  
        if(pPass.nQuality > QUALITY_MEDIUM)
        {
          half4 lm = tex2D(LightMapSampler, pPass.IN.Tex1.xy); // g-terrain shadow, r-static shadows, b-sky accessibility, a-sun accessibility
   #if defined(RT_SHADOWS)
          fOcclShadow *= lm.g; // use lm from terrain;
   #else
          fOcclShadow *= lm.g * lm.r; // use lm from static objects and terrain;
   #endif  	      
        }
        else
          fOcclShadow *= tex2D(ColorSampler, pPass.IN.Tex0.xy).a * 1.25h; // get LM from terrain mask alpha
      }
 #else  	  
      if (pPass.nTerrainIndex && nType == LIGHT_DIRECTIONAL && pPass.nQuality <= QUALITY_MEDIUM)
        fOcclShadow *= tex2D(ColorSampler, pPass.IN.Tex0.xy).a * 1.25h; // get LM from terrain mask alpha
 #endif
 #if defined (RT_ALPHA_GRADIENT)
      fOcclShadow = 1;
 #endif
      pLight.fNdotL = fNdotL;
      pLight.fOcclShadow = fOcclShadow;
      
      pLight.fFallOff = fFallOff;
      pLight.cFilter = filterColor;

      // per shader custom shading 
 #if defined (RT_STATIC_LIGHTING) && defined(VERTEX_COLOR)
      if (nType != LIGHT_DIRECTIONAL)
        frag_custom_per_light(pPass, pLight);
      else
        pPass.cDiffuseAcc += pLight.cDiffuse * fOcclShadow;
 #else
      frag_custom_per_light(pPass, pLight);
 #endif
      //pPass.cDiffuseAcc.xyz = fNdotL;
    }
  }
#elif RT_PROFILE == QUALITY_LOW || defined(DISABLE_RTLIGHTING)
  if (pPass.nType == 0)
    pPass.cDiffuseAcc = 0;
 #if RT_PROFILE <= QUALITY_LOW
  pPass.cDiffuseAcc = pPass.IN.Diffuse.rgb;
 #endif
 #if defined (RT_PLANAR_LIGHTMAP) && !defined(NO_LIGHTING)
  pPass.cDiffuseAcc *= tex2D(LightMapSampler, pPass.IN.Tex1.xy).a; // use lm from static objects and terrain;
 #else
  if(pPass.nTerrainIndex) // Terrain lightmap
    pPass.cDiffuseAcc *= tex2D(ColorSampler, pPass.IN.Tex0.xy).a * 1.25f; // get LM from terrain mask alpha
 #endif     
#elif RT_PROFILE == QUALITY_VERYLOW
  pPass.cDiffuseAcc = pPass.IN.Diffuse.rgb;
#endif

  bool bGeneralLightmpap = false;
#if defined(RT_LIGHTMAP) && !defined (RT_PLANAR_LIGHTMAP)
  bGeneralLightmpap = true;
#endif
  if(pPass.nTerrainIndex)
    bGeneralLightmpap = false;
    
  // Ambient term
  pPass.cAmbientAcc = 0;
#if defined(RT_LIGHTMAP) && !defined(RT_PLANAR_LIGHTMAP)
  if (bGeneralLightmpap)
  {
    pPass.cAmbientAcc = tex2D(LightMapSampler, pPass.IN.Tex1.xy).rgb * 2.h;
 #if !defined(RT_SKINNING)
    if (!pPass.bIgnoreShadows)
      pPass.cAmbientAcc *= dot(pPass.cShadowOcclMap, half4(0.25, 0.25, 0.25, 0.25));
 #endif
  }
  else
#endif
  {
    if (pPass.nQuality <= QUALITY_LOW && pPass.nTerrainIndex)
    {
      pPass.cAmbientAcc = F_WorldAmbient.rgb * 0.75f;
   #if RT_PROFILE > QUALITY_LOW
      pPass.cAmbientAcc *= pPass.vNormal.y * .25h + .75h;
   #endif
    }
    else
    {
   #if RT_PROFILE > QUALITY_LOW || defined (RT_STATIC_LIGHTING) || defined (RT_PLANAR_LIGHTMAP)
    pPass.cAmbientAcc = F_WorldAmbient.rgb;
    #ifdef TEST
    half3 vEnv = texCUBE(EnvMapSampler, pPass.vNormal).rgb;
    pPass.cAmbientAcc.xyz = vEnv;
    #endif
   #endif 
    if (pPass.nQuality == QUALITY_MEDIUM && pPass.nTerrainIndex)
    {
      pPass.cAmbientAcc = F_WorldAmbient.rgb * 0.75f;
    }
   
   // hemispherical sky occlusion
   #if !defined (RT_STATIC_LIGHTING) && RT_PROFILE > QUALITY_MEDIUM
    pPass.cAmbientAcc *= pPass.vNormal.y * .25h + .75h;
   #endif		
    }
  }
  
#if defined (RT_SSAO) && !defined (RT_STATIC_LIGHTING) && !defined(RT_BLEND_ALPHA) && !defined(RT_BLEND_ADD) && !defined(RT_RENDER_TO_TEXTURE)
  if (pPass.nQuality > QUALITY_LOW)
    pPass.cAmbientAcc *= pPass.cShadowOcclMap.a;
#endif

  // Deferred stuff
#ifdef RT_DEFERRED  
  half4 diffuseAcc = tex2D(samplerDiffuseAcc, tcScreen.xy);
  pPass.cDiffuseAcc += diffuseAcc.xyz;
  //pPass.cAmbientAcc = 0; // diffuse contains ambient term and SSAO
  pPass.cAmbientAcc.xyz = lerp(F_WorldAmbientShadow, pPass.cAmbientAcc, fOcclShadow); // In shadows another ambient
 #ifdef RT_SPECULAR
  half4 specularAcc = tex2D(samplerSpecularAcc, tcScreen.xy);
  pPass.cSpecularAcc += specularAcc.xyz;
 #endif
#endif  
  

  // final composition
#ifndef FRAG_CUSTOM_END
  frag_final_composition(pPass, cOut.rgb);
#else
  frag_custom_end(pPass, cOut.rgb);
#endif

#if RT_PROFILE > QUALITY_VERYLOW && !defined(OPENGL)
  cOut.rgb *= half3(pPass.IN.WorldView.w, pPass.IN.WorldNormal.w, pPass.IN.Tex0.z);
#endif  

  half4 fogColor = 0;
#if defined(RT_FOG)
  fogColor = pPass.IN.FogColor;
 #if defined(RT_FOG_LIT)
  float fLerp = Luminance(pPass.cDiffuseAcc.xyz);
  fLerp = min(fLerp, 1);
  //fogColor.xyz = lerp(fogColor.xyz, pPass.cDiffuseAcc.xyz, fLerp);
  fogColor.xyz *= pPass.cDiffuseAcc.xyz*0.5+float3(1,1,1);
 #endif
#endif

#if !defined(RT_RENDER_TO_TEXTURE)
 #if defined (RT_ALPHA_GRADIENT)
  cOut.w = pPass.fAlpha;
 #elif defined (RT_GLOW)
  // for glowing objects
  //cOut.w = pPass.cDiffuseMap.w;
 #else
  cOut.w = pPass.fAlpha;
 #endif
#else
 #ifdef OPENGL
  cOut.w = pPass.fAlpha;
 #else
  #if defined(RT_BLEND_ALPHA) || defined(RT_ALPHA_TEST)
  // for transparent objects
  cOut.w = pPass.fAlpha;
  #else
  cOut.w = 1;
  #endif
 #endif
#endif

#ifdef RT_BLEND_MULTI
  cOut.xyz = lerp(float3(1,1,1), cOut.xyz, pPass.fAlpha);
#endif

  //cOut.xyz = pPass.cDiffuseMap.xyz;
 #ifdef RT_TOON
  //cOut.xyz = tex2D(OpacitySampler, baseTC).a;
 #endif
 
  // Setup final fog
#if defined(RT_FOG) && RT_PROFILE > QUALITY_VERYLOW
  frag_fog_setup(fogColor, cOut);
#endif
  
#if RT_PROFILE == QUALITY_LOW && defined (RT_UNDERWATER)
  half4 volFog = tex2D(samplerFog, pPass.IN.texFog.xy);
  half4 volFogEnter = tex2D(samplerFogEnter, pPass.IN.texFog.zw);
  half4 volColor = volFog * volFogEnter;
  fogColor.xyz = F_WaterFogColor.xyz;
 #if defined(RT_BLEND_ADD)  
  fogColor.xyz = 0;
 #endif
  volColor.xyz *= fogColor.xyz;
  cOut.xyz = lerp(cOut.xyz, volColor.xyz, volColor.w);
#endif
 
#if defined(RT_DEBUG)
  half4 cCol = DebugOutput(cOut, pPass);
  if (!pPass.bTest)
    return cCol;
#endif

//#ifdef RT_DEFERRED
//cOut.xyz = diffuseAcc; //pPass.cDiffuseAcc.xyz;
//#endif


//if (pPass.nType == 2)
 //cOut.xyz = pPass.cDiffuseAcc.xyz;

#if defined(RT_ALPHA_TEST) && defined(OPENGL_ES)
  clip(cOut.w - 0.5f);
#endif

 //if (pPass.nTerrainIndex && pPass.nQuality <= QUALITY_MEDIUM)
#if defined(RT_LIGHTMAP) && !defined(RT_PLANAR_LIGHTMAP)
 //cOut.xyz = frac(pPass.IN.Tex1.xyz * 10); // * pPass.IN.Diffuse.rgb; // get LM from terrain mask alpha
#endif
/*#ifdef RT_ENVMAP
  cOut.xyz = pPass.cSpecularAcc;
#endif*/
//cOut.xyz = pPass.vNormal.xyz;

  return cOut;
}

#endif
