/*

 Copyright (c) Secret Sign, 2005

 Engine2 fx library

*******************************************************************************/

#include "constants.fxh"
#include "lighting.fxh"


/////////////////////////////////////// Tweakables ////////
uniform half4 GlowIntensity;
uniform half4 GlowAlphaIntensity;
uniform half4 HighlightThreshold;
uniform half4 Radius;

texture2D DownsampleMap;

sampler2D DownsampleSampler = sampler_state 
{
  texture = <DownsampleMap>;
  AddressU  = CLAMP;        
  AddressV  = CLAMP;
  AddressW  = CLAMP;
  MIPFILTER = LINEAR;
  MINFILTER = LINEAR;
  MAGFILTER = LINEAR;
};

sampler2D OrigSampler = sampler_state 
{
  texture = <DownsampleMap>;
  AddressU  = CLAMP;        
  AddressV  = CLAMP;
  AddressW  = CLAMP;
  MIPFILTER = NONE;
  MINFILTER = LINEAR;
  MAGFILTER = LINEAR;
};

/////////////////////////////////// data structures ///////
struct VS_OUTPUT
{
 	float4 Position   : POSITION;
  half2 TexCoord0  : TEXCOORD0;
};

struct vs_in {
	float3 Pos: POSITION;
	half2 Tex0: TEXCOORD0;
};

VS_OUTPUT VS_Quad4(vs_in In)
{
  VS_OUTPUT OUT;
  OUT.Position = float4(In.Pos.xyz, 1);
  OUT.TexCoord0 = In.Tex0.xy + F_ScreenSize.xy * 0.5h * 4.0h;

  return OUT;
}
VS_OUTPUT VS_Quad2(vs_in In)
{
  VS_OUTPUT OUT;
  OUT.Position = float4(In.Pos.xyz, 1);
  OUT.TexCoord0 = In.Tex0.xy + F_ScreenSize.xy * 0.5h * 2.0h;
  return OUT;
}

half4 PS_BrightPass(in VS_OUTPUT IN) : COLOR0
{
	half4 orig = tex2D(DownsampleSampler, IN.TexCoord0);
	half3 color = orig.rgb * (1.h + orig.a * GlowAlphaIntensity.x);
	half fLum = Luminance(color.xyz);
	half fBright = smoothstep(HighlightThreshold.x, 1, fLum);
	color.xyz = normalize(color.xyz) * fBright;
	return half4(color, 0.h);
}

half4 PS_Comp(in VS_OUTPUT IN) : COLOR0
{
  half4 bright[4];

  bright[0] = tex2Dlod(DownsampleSampler, float4(IN.TexCoord0.xy, 0, Radius.x+1)) * 1.0h;
  bright[1] = tex2Dlod(DownsampleSampler, float4(IN.TexCoord0.xy, 0, Radius.x+2)) * 1.2h;
  bright[2] = tex2Dlod(DownsampleSampler, float4(IN.TexCoord0.xy, 0, Radius.x+3)) * 1.4h;
  bright[3] = tex2Dlod(DownsampleSampler, float4(IN.TexCoord0.xy, 0, Radius.x+4)) * 1.6h;

  half2 baseTC = IN.TexCoord0.xy;
#ifdef OPENGL 
  baseTC.y = 1 - baseTC.y;
#endif
  half4 orig = tex2D(OrigSampler, baseTC.xy);

  half3 glow = 0.h;
  for(int i=0; i<4; ++i)
    glow += bright[i].rgb;
  glow *= 1.h / 6;
  //glow = bright[0].rgb;
 
  return half4(glow.rgb * GlowIntensity.x + orig.rgb, orig.a);
}

/////////////////////////////////////// techniques /////////
technique Bloom
{
	pass BrightPass
	{
		VertexShader = compile VS_SHADER_TYPE VS_Quad2();
		PixelShader  = compile PS_SHADER_TYPE PS_BrightPass();
#ifdef OGL
		CullMode = FrontAndBack;
#else
		CullMode = none;
#endif
		ZEnable = false;
		AlphaBlendEnable = false;
	}
	pass FinalCombine
	{
		VertexShader = compile vs_3_0 VS_Quad4();
		PixelShader  = compile ps_3_0 PS_Comp();
#ifdef OGL
		CullMode = FrontAndBack;
    BlendFunc = { One, One };
#else
		CullMode = none;
		SrcBlend = One;
		DestBlend = One;
#endif
		AlphaBlendEnable = false;
		ZEnable = false;
	}
}


//===========================================================================================

struct vs_out {
	float4 Pos: POSITION;
	float2  Tex0: TEXCOORD0;
};

sampler2D samplerTex = sampler_state
{
    Texture = <Texture0>;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = None;
    AddressU = Clamp;
    AddressV = Clamp;
};

vs_out VSShow(vs_in In)
{
	vs_out Out;
	Out.Pos = float4(In.Pos.xyz, 1);
	Out.Tex0 = In.Tex0;
	return Out;
}

float4 PSShow(vs_out In): COLOR
{
	float4 Col = tex2D(samplerTex, In.Tex0);
	
	return Col;
}

technique TShow
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSShow();
		PixelShader  = compile PS_SHADER_TYPE PSShow();
	}
}
