// Scattering

#include "constants.fxh"
#include "lighting.fxh"

uniform float LerpFactor = 0.0;

struct VSOUTPUT_SKYDOME
{
	float4 Pos			: POSITION;
	half4 Inscatter : COLOR0;
	half2 Texcoord0 : TEXCOORD0;
#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
  float4 FogColor : TEXCOORD1;
 #endif
#endif
#if RT_PROFILE == QUALITY_VERYLOW 
	half Fog    		  : FOG;
#endif
};

struct VSOUTPUT_SKYDOME2
{
	float4 Pos			: POSITION;
	half2 Texcoord0 : TEXCOORD0;
#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
  float4 FogColor : TEXCOORD1;
 #endif
#endif
#if RT_PROFILE == QUALITY_VERYLOW 
	half Fog    		  : FOG;
#endif
};

half4	FogWater(in half sceneZ)
{
  half effDist = max(0, (sceneZ - F_FogParams.x)*F_FogParams.y);
	half density = exp(-effDist*effDist);
	half3 fogColor = lerp(F_FogColorEnd.xyz, F_FogColor.xyz, density);
	half fogDensity = saturate(F_FogColorEnd.w*density + F_FogColor.w);
#if RT_PROFILE > QUALITY_VERYLOW
  return half4(fogColor, fogDensity);
#else
  return half4(F_FogColorEnd.xyz, saturate(density));
#endif  
}

VSOUTPUT_SKYDOME VSSky(in float3 vPos:POSITION, in float2 Tex:TEXCOORD)
{
	VSOUTPUT_SKYDOME Out=(VSOUTPUT_SKYDOME)0;
	
	float3 view = MUL_VEC_MAT(float4(vPos.xyz,1), PI_mW).xyz;
	float dist = length(C_Eye - view);
	view = normalize(C_Eye - view);

	half4 Extinction;
	Scattering(view, dist, Out.Inscatter, Extinction);

	Out.Pos = MUL_VEC_MAT(MUL_VEC_MAT(float4(vPos.xyz,1), PI_mW), C_VP);
	Out.Texcoord0 = Tex;

#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
  Out.FogColor = FogWater(dist);
 #endif
#endif
#if RT_PROFILE == QUALITY_VERYLOW 
  Out.Fog = 1;
#endif

	return Out;
}

half4 PSSkyDome(in VSOUTPUT_SKYDOME In):COLOR
{
	half4 cOut = In.Inscatter;
#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
	cOut.xyz = lerp(In.FogColor.xyz, cOut.xyz, In.FogColor.w);
 #endif
#endif	
  return cOut;
}

half4 PSSkyDomeTex(in VSOUTPUT_SKYDOME In):COLOR
{
	half4 cOut = tex2D(ColorSampler, In.Texcoord0);
#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
	cOut.xyz = lerp(In.FogColor.xyz, cOut.xyz, In.FogColor.w);
 #endif
#endif	
  return cOut;
}

//==================================================================================================

VSOUTPUT_SKYDOME2 VSSkyDomeTex2(in float3 vPos:POSITION, in float2 Tex:TEXCOORD)
{
	VSOUTPUT_SKYDOME2 Out=(VSOUTPUT_SKYDOME2)0;
	
	float4 view = MUL_VEC_MAT(float4(vPos.xyz, 1.0f), PI_mW);
	Out.Pos = MUL_VEC_MAT(view, C_VP);
	Out.Texcoord0 = Tex;

#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
  float dist = length(C_Eye - view.xyz);
  Out.FogColor = FogWater(dist);
 #endif
#endif
#if RT_PROFILE == QUALITY_VERYLOW 
  Out.Fog = 1;
#endif

	return Out;
}

half4 PSSkyDomeTex2(in VSOUTPUT_SKYDOME2 In):COLOR
{
	half4 cOut = tex2D(ColorSampler, float2(In.Texcoord0.x*1.5f, In.Texcoord0.y));
	half4 cOut1 = tex2D(SecondColorSampler, float2(In.Texcoord0.x*1.5f, In.Texcoord0.y));
	cOut.xyz = lerp(cOut.xyz, cOut1.xyz, LerpFactor);
#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
	cOut.xyz = lerp(In.FogColor.xyz, cOut.xyz, In.FogColor.w);
 #endif
#endif	
  return cOut;
}

half4 PSSkyDomeTex2_(in VSOUTPUT_SKYDOME2 In):COLOR
{
	half4 cOut = tex2D(ColorSampler, In.Texcoord0);
	half4 cOut1 = tex2D(SecondColorSampler, In.Texcoord0);
	cOut.xyz = lerp(cOut.xyz, cOut1.xyz, LerpFactor);
#if RT_PROFILE == QUALITY_VERYLOW 
 #if defined (RT_UNDERWATER)
	cOut.xyz = lerp(In.FogColor.xyz, cOut.xyz, In.FogColor.w);
 #endif
#endif	
  return cOut;
}


technique SkyDome
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSSkyDome();
		VertexShader = compile VS_SHADER_TYPE VSSky();
	}
}

technique SkyDomeTex
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSSkyDomeTex();
		VertexShader = compile VS_SHADER_TYPE VSSky();
	}
}

technique SkyDomeTex2
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSSkyDomeTex2();
		VertexShader = compile VS_SHADER_TYPE VSSkyDomeTex2();
	}
}

technique SkyDomeTex2_
{
	pass P0
	{
		PixelShader = compile PS_SHADER_TYPE PSSkyDomeTex2_();
		VertexShader = compile VS_SHADER_TYPE VSSkyDomeTex2();
	}
}
