// Common functions for shaders 
// Copyright (c) Secret Sign, 2005
// Skining functions


static const int MAX_MATRICES = 26;


// !!!
//
float3 _Quat_x_Vec3(const float4 quat, const float3 vec)
{
	float3 r2,r3;
	r2	= cross(quat.xyz,vec);	//mul+mad
	r2	= quat.w*vec+r2;		//mad
	r3	= cross(quat.xyz,r2);	//mul+mad
	r3	= r3*2+vec;				//mad
	return r3;
}


float3 _QuatT_x_Vec3(const float4 q, const float3 t, const float3 v)
{
  float3 r2,r3;
  r2 = cross(q.xyz, v);
  r2 = q.w*v + r2;
  r3 = cross(q.xyz,r2); 
  r3 = r3*2 + v;			//mad
  return r3 + t.xyz;        //add
}

float3 _QuatTS_x_Vec3(const float4 q, const float4 t, const float3 vec) 
{
	float3 r2,r3;  
	r2	= cross(q.xyz,vec);		//mul+mad 
	r2	= q.w*vec+r2;				//mad 
	r3	= cross(q.xyz,r2);		//mul+mad 
	r3	= r3*2+vec;						//mad 

	return (t.w + 0.000001)* r3+t.xyz;    //mad
} 

float4 _Quat_x_Quat(const float4 q, const float4 p) 
{
	float4 c, r;
	c.xyz = cross(q.xyz, p.xyz);	// 2 CROSS
	c.w = -dot(q.xyz, p.xyz);		// 1 DOT3
	r = p * q.w + c;				// 1 MAD
	r.xyz = (q * p.w + r).xyz;		// 1 MAD
	return r;
}

float3x3 _Quat2Mat33(const float4 quat)
{
  float3 r0,r1,r2;  float4 q2;
  float3 c0 = float3(-1,0,1);

  q2    = quat+quat;				//add
  r2    = q2.w*c0.xyz;				//mul
  r0    = quat.wzy*r2.zxz+c0.xyy;	//mad ->tangent
  r0    = quat.x*q2.xyz+r0;			//mad
  r1    = quat.zwx*r2.zzx+c0.yxy;	//mad ->binormal
  r1    = quat.y*q2.xyz+r1;	        //mad
  r2	= cross(r0,r1);				//mul+mad ->binormal

  return float3x3(r0,r1,r2);
}

// Skin position
float3 _SkinningPosition(int4 Indices, float4 Weights, in float4 InPos)
{
  float3 Pos;
  Indices *= 2;
//  Weights = float4(63.75f,63.75f,63.75f,63.75f);
  Pos =	(_QuatT_x_Vec3(mWorldQuatT[Indices.x],mWorldQuatT[Indices.x+1], InPos)*Weights.x + 
  _QuatT_x_Vec3(mWorldQuatT[Indices.y], mWorldQuatT[Indices.y+1], InPos)*Weights.y + 
  _QuatT_x_Vec3(mWorldQuatT[Indices.z], mWorldQuatT[Indices.z+1],InPos)*Weights.z + 
  _QuatT_x_Vec3(mWorldQuatT[Indices.w], mWorldQuatT[Indices.w+1], InPos)*Weights.w).xyz;
  Pos *= mWorldQuatT[Indices.x+1].w / 255.0f;
  Pos = MUL_VEC_MAT(float4(Pos, 1), PI_mW).xyz;
  return Pos;
}

// !!!
//
// Skin normal

float3 _SkinningNormal(int4 Indices, float4 Weights, in float3 InNorm)
{
  float3 Norm;
  Indices *= 2;
  Norm = _Quat_x_Vec3(mWorldQuatT[Indices.x], InNorm)*Weights.x + _Quat_x_Vec3(mWorldQuatT[Indices.y], InNorm)*Weights.y + _Quat_x_Vec3(mWorldQuatT[Indices.z], InNorm)*Weights.z + _Quat_x_Vec3(mWorldQuatT[Indices.w], InNorm)*Weights.w;
  Norm = MUL_VEC_MAT(float4(Norm, 0), PI_mW);  
  return normalize(Norm);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// Skin position and tangent space (Dual Quaternion Skinning)
void __SkinningPositionTangent(int4 BoneIndices, float4 Weights, inout float3 Pos, inout float3 Tangent, inout float3 Binormal, inout float3 Normal)
{
  int4 idx = BoneIndices * 2;

  float4 inPos = float4(Pos, 1);

  Pos =	_QuatT_x_Vec3(mWorldQuatT[idx.x], mWorldQuatT[idx.x+1], inPos)*Weights.x + _QuatT_x_Vec3(mWorldQuatT[idx.y], mWorldQuatT[idx.y+1], inPos)*Weights.y + _QuatT_x_Vec3(mWorldQuatT[idx.z], mWorldQuatT[idx.z+1], inPos)*Weights.z + _QuatT_x_Vec3(mWorldQuatT[idx.w], mWorldQuatT[idx.w+1], inPos)*Weights.w;
  Pos *= mWorldQuatT[idx.x+1].w;
  Pos.xyz /= 255.0f;
  Pos = MUL_VEC_MAT(float4(Pos, 1), PI_mW);

  Tangent = _Quat_x_Vec3(mWorldQuatT[idx.x], Tangent)*Weights.x + _Quat_x_Vec3(mWorldQuatT[idx.y], Tangent)*Weights.y + _Quat_x_Vec3(mWorldQuatT[idx.z], Tangent)*Weights.z + _Quat_x_Vec3(mWorldQuatT[idx.w], Tangent)*Weights.w;
  Tangent = MUL_VEC_MAT(float4(Tangent, 0), PI_mW);
  Tangent = normalize(Tangent);

  Binormal = _Quat_x_Vec3(mWorldQuatT[idx.x], Binormal)*Weights.x + _Quat_x_Vec3(mWorldQuatT[idx.y], Binormal)*Weights.y + _Quat_x_Vec3(mWorldQuatT[idx.z], Binormal)*Weights.z + _Quat_x_Vec3(mWorldQuatT[idx.w], Binormal)*Weights.w;
  Binormal = MUL_VEC_MAT(float4(Binormal, 0), PI_mW);  
  Binormal = normalize(Binormal);

  Normal = _Quat_x_Vec3(mWorldQuatT[idx.x], Normal)*Weights.x + _Quat_x_Vec3(mWorldQuatT[idx.y], Normal)*Weights.y + _Quat_x_Vec3(mWorldQuatT[idx.z], Normal)*Weights.z + _Quat_x_Vec3(mWorldQuatT[idx.w], Normal)*Weights.w;
  Normal = MUL_VEC_MAT(float4(Normal, 0), PI_mW);  
  Normal = normalize(Normal);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// Skin position and normal (Dual Quaternion Skinning)
void __SkinningPositionNormal(int4 BoneIndices, float4 Weights, inout float3 Pos, inout float3 Normal)
{
  int4 idx = BoneIndices * 2;

  float4 inPos = float4(Pos, 1);

  Pos =	_QuatT_x_Vec3(mWorldQuatT[idx.x], mWorldQuatT[idx.x+1], inPos)*Weights.x + _QuatT_x_Vec3(mWorldQuatT[idx.y], mWorldQuatT[idx.y+1], inPos)*Weights.y + _QuatT_x_Vec3(mWorldQuatT[idx.z], mWorldQuatT[idx.z+1], inPos)*Weights.z + _QuatT_x_Vec3(mWorldQuatT[idx.w], mWorldQuatT[idx.w+1], inPos)*Weights.w;
  Pos *= mWorldQuatT[idx.x+1].w;
  Pos.xyz /= 255.0f;
  Pos = MUL_VEC_MAT(float4(Pos, 1), PI_mW);

  Normal = _Quat_x_Vec3(mWorldQuatT[idx.x], Normal)*Weights.x + _Quat_x_Vec3(mWorldQuatT[idx.y], Normal)*Weights.y + _Quat_x_Vec3(mWorldQuatT[idx.z], Normal)*Weights.z + _Quat_x_Vec3(mWorldQuatT[idx.w], Normal)*Weights.w;
  Normal = MUL_VEC_MAT(float4(Normal, 0), PI_mW);  
  Normal = normalize(Normal);
}
