// An empty material, which simply transforms the vertex and sets the color to white.

#include "constants.fxh"
//------------------------------------
//float4x4 PI_mW : World;
//float4x4 C_V : WorldViewProjection;
//float4x4 C_P : Projection;
//float4x4 mBVP : Projection;

//------------------------------------
struct vertexInput {
    float4 Position	: POSITION;
    float3 Offset : NORMAL;
    float2 TexCoord: TEXCOORD0;

};


struct vertexInput1 {
    float3 HPosition : POSITION;
    float3 TexCoord: TEXCOORD0;
};

struct vertexInput2 {
    float3 Position	: POSITION;
    float4 TexCoord: TEXCOORD0;
};

struct vertexOutput {
    float4 HPosition : POSITION;
    float2 TexCoord: TEXCOORD0;
#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW
	  half4 FogColor  : COLOR1;
 #else	
	  half FogColor   : FOG;
 #endif	
#endif
};


/*
texture Texture0  : DIFFUSE;
sampler ColorSampler= sampler_state 
{
    texture = <Texture0>;
    AddressU  = WRAP;        
    AddressV  = WRAP;
    AddressW  = WRAP;
    MIPFILTER = POINT;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};
*/

//  Setup fog. Per-vertex for alpha blended objects and per-object volumetric fog
#if RT_PROFILE > QUALITY_VERYLOW
half4	Fog(const in half sceneZ)
#else
half	Fog(const in half sceneZ)
#endif
{
  half effDist = max(0, (sceneZ - F_FogParams.x)*F_FogParams.y);
	half density = exp(-effDist*effDist);
	half3 fogColor = lerp(F_FogColorEnd.xyz, F_FogColor.xyz, density);
	half fogDensity = saturate(F_FogColorEnd.w*density + F_FogColor.w);
#if RT_PROFILE > QUALITY_VERYLOW
  return half4(fogColor, fogDensity);
#else
  return saturate(exp(-effDist*effDist));
#endif  
}

#if RT_PROFILE > QUALITY_VERYLOW
#if defined(RT_FOG)
void FogPS(const in half4 FogColor, inout half3 cOut)
{
	cOut = lerp(FogColor.xyz, cOut, FogColor.w);
}
#endif
#endif
//------------------------------------
vertexOutput VS_TransformDiffuse(vertexInput IN) 
{
    vertexOutput OUT;
    float4 w0norm = normalize(PI_mW[0]);
    float scale = PI_mW._11/w0norm.x;
  
    float4x4 WV = mul(PI_mW, C_V);
    OUT.HPosition =  mul( float4(IN.Position.xyz, 1.0) , WV);
    float4 offset = float4(IN.Offset.x, IN.Offset.y, 0, 0);
    OUT.HPosition = mul(OUT.HPosition + offset * scale, C_P);// + offres;

#if defined (RT_FOG)
	OUT.FogColor = 1;
#endif
    
    OUT.TexCoord = IN.TexCoord;
    return OUT;
}

//------------------------------------
vertexOutput VS_TransformDiffuse1(vertexInput1 IN) 
{
  vertexOutput OUT; 
  OUT.HPosition = float4(IN.HPosition.xyz,1);// + offres;
  
  OUT.TexCoord.xy = IN.TexCoord.xy;
    
#if defined (RT_FOG)
	OUT.FogColor = Fog(IN.TexCoord.z);
#endif
    
  return OUT;
}

//------------------------------------
vertexOutput VS_TransformDiffuseRain(vertexInput2 IN) 
{
    vertexOutput OUT;
    float4 w0norm = normalize(PI_mW[0]);
    float scale = PI_mW._11/w0norm.x;
  
    float4x4 WV = mul(PI_mW, C_V);
    OUT.HPosition =  mul( float4(IN.Position.xyz + float3(0,IN.TexCoord.w,0), 1.0) , WV);
    float4 offset = float4(IN.TexCoord.z, 0, 0, 0);
    OUT.HPosition = mul(OUT.HPosition + offset * scale, C_P);// + offres;

#if defined (RT_FOG)
	OUT.FogColor = 1;
#endif
    
    OUT.TexCoord = float2(IN.TexCoord.x,IN.TexCoord.y);
    return OUT;
}

//------------------------------------
vertexOutput VS_TransformDiffuseSnow(vertexInput2 IN) 
{
    vertexOutput OUT;
    float4 w0norm = normalize(PI_mW[0]);
    float scale = PI_mW._11/w0norm.x;
  
    float4x4 WV = mul(PI_mW, C_V);
    OUT.HPosition =  mul( float4(IN.Position.xyz, 1.0) , WV);
    float4 offset = float4(IN.TexCoord.z, IN.TexCoord.w, 0, 0);
    OUT.HPosition = mul(OUT.HPosition + offset * scale, C_P);// + offres;

#if defined (RT_FOG)
	OUT.FogColor = 1;
#endif
    
    OUT.TexCoord = float2(IN.TexCoord.x,IN.TexCoord.y);
    return OUT;
}


float4 PS(vertexOutput IN):COLOR
{
	float4 cOut = tex2D(ColorSampler, IN.TexCoord.xy);
	
#if RT_PROFILE > QUALITY_VERYLOW
#if defined(RT_FOG)
  FogPS(IN.FogColor, cOut.rgb);
#endif
#endif
  return cOut;
}


//-----------------------------------
technique Default
{
    pass p0 
    {		
		  VertexShader = compile vs_2_0 VS_TransformDiffuse();
		  PixelShader = compile ps_2_0 PS();
    }
}
//-----------------------------------
technique Imposter
{
    pass p0 
    {		
		VertexShader = compile vs_2_0 VS_TransformDiffuse1();
		PixelShader = compile ps_2_0 PS();
//		FogEnable = false;
	
    }
}

//-----------------------------------
technique Rain
{
    pass p0 
    {		
		VertexShader = compile vs_2_0 VS_TransformDiffuseRain();
		PixelShader = compile ps_2_0 PS();
//		FogEnable = false;
	
    }
}

technique Snow
{
    pass p0 
    {		
		VertexShader = compile vs_2_0 VS_TransformDiffuseSnow();
		PixelShader = compile ps_2_0 PS();
//		FogEnable = false;
	
    }
}