// Terrain

#undef DECAL_DIFFUSE
#undef RT_PLANAR_LIGHTMAP

#include "lighting.fxh"

sampler2D Sampler4 = sampler_state {
	Texture = <BumpTexture>;
	minfilter = linear;
	magfilter = linear;
	mipfilter = linear;
	AddressU = WRAP;
	AddressV = WRAP;
};
    
sampler2D  Sampler5 = sampler_state {
	Texture = <BumpGlossTexture>;
	minfilter = linear;
	magfilter = linear;
	mipfilter = linear;
	AddressU = WRAP;
	AddressV = WRAP;
};

sampler2D Sampler6 = sampler_state {
	Texture = <EnvMapTexture>;
	minfilter = linear;
	magfilter = linear;
	mipfilter = linear;
	AddressU = WRAP;
	AddressV = WRAP;
};

struct VSOUTPUT_LAND
{
	float4 HPosition 				: POSITION;
#if RT_PROFILE <= QUALITY_LOW
	half4  Diffuse 				: COLOR0;
#endif
#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW
	half4 FogColor    			: COLOR1;
 #else
	half FogColor    			  : FOG;
 #endif	
#endif
	float4 Tex0 						: TEXCOORD0;
	float3 TexLayer1				: TEXCOORD1;	
	float4 TexLayerLayer23 	: TEXCOORD2;	
	
  half4 WorldNormal			  : TEXCOORD3;
  float4 WorldView				: TEXCOORD4;
#if RT_PROFILE > QUALITY_LOW
  float4 WorldPos					: TEXCOORD5;		// camera depth in w (for fog)
#endif

#if defined(RT_LIGHTMAP)
	float2 Tex1     				: TEXCOORD6;	
#endif

#if RT_PROFILE > QUALITY_LOW
#if defined(RT_DEFERRED) || (defined(RT_SHADOWS) || defined(RT_SSAO) || defined(USE_SCREENCOORDS))
	half3  ScreenCoordProj  : TEXCOORD7;	
#endif
#endif

#if RT_PROFILE == QUALITY_LOW
#if defined (RT_UNDERWATER)
	half4 texFog						: TEXCOORD7;
#endif
#endif
};

struct VS_INPUT
{
	float4 vPos					: POSITION;
#ifdef OGL
	half3 vNorm					: COLOR1;
#else
	half3 vNorm					: NORMAL;
#endif
#ifdef VERTEX_COLOR
	half4 vVertexColor	: COLOR0;
#endif
};

VSOUTPUT_LAND _VS(in VS_INPUT In, bool bDetailTexture)
{
	VSOUTPUT_LAND Out = (VSOUTPUT_LAND)0;

  Out.Tex0.xy =(In.vPos.xz - TerrainPatchOffset.xy) / 32.0f * 33.0f/32.0f;

#ifdef RT_LIGHTMAP
  Out.Tex1.xy = (In.vPos.xz - TerrainPatchOffset.xy) / 32.0f * TerrainPatchOffset.w/(TerrainPatchOffset.w+2);
  Out.Tex1.xy += 0.5f/TerrainPatchOffset.w;
  Out.Tex1.y = 1 - Out.Tex1.y;
#endif

	//mask alignment fix
	Out.Tex0.xy = FC_MaskOffset.x + Out.Tex0.xy * FC_MaskOffset.y;// - 0.5f/32.0f;
 
  Out.TexLayer1.xy = (In.vPos.xz ) * FC_MaskOffset.z; // 32.0f * FC_MaskOffset.w;

	half dist = length(C_Eye.xyz - In.vPos.xyz);

	//      
  Out.Tex0.w =  1.f - dist / max(5.f, FC_DetailScale.w);
	Out.TexLayer1.z = 1.h - FC_DetailMicroScale.w;

	float4 OPos = mul(float4(In.vPos.xyz,1), C_VP);

	half3 vNorm = normalize(In.vNorm.xyz * 2.h - 1.h);

	Out.WorldNormal.xyz = vNorm;
  Out.WorldView.xyz = C_Eye.xyz - In.vPos.xyz;
#if RT_PROFILE > QUALITY_LOW
  Out.WorldPos.xyz = In.vPos.xyz;
	Out.WorldPos.w = dist;// camera depth in w (for fog)
#else
	Out.Diffuse.a = 1.h;
	Out.Diffuse.rgb = float3(0,0,0);
	#if defined(RT_NUM_LIGHTS)
		half fNdotL = saturate(dot(vNorm, LGetDir(0).xyz));
		Out.Diffuse.rgb += LGetDiffuse(0).rgb * fNdotL; // * TerrainColor.xyz;
	#endif
#endif

	if(bDetailTexture)
	{
		Out.TexLayerLayer23.zw = Out.TexLayer1 * FC_DetailMicroScale.z;
		Out.TexLayerLayer23.xy = Out.TexLayer1 * FC_DetailMicroScale.y;
		Out.TexLayer1 = Out.TexLayer1 * FC_DetailMicroScale.x;
	}
	else
	{
		Out.TexLayerLayer23.zw = Out.TexLayer1 * FC_DetailScale.z;//*0.5;
		Out.TexLayerLayer23.xy = Out.TexLayer1 * FC_DetailScale.y;//*0.5;
		Out.TexLayer1 = Out.TexLayer1 * FC_DetailScale.x;//*0.5;
	}
#if RT_PROFILE == QUALITY_LOW
#if defined (RT_UNDERWATER)
  float4 vWPos = float4(In.vPos.xyz, 1);
  Out.texFog.x = dot(C_FogMatrix[0], vWPos);
  Out.texFog.y = dot(C_FogMatrix[1], vWPos);
  Out.texFog.z = dot(C_FogEnterMatrix[0], vWPos);
  Out.texFog.w = dot(C_FogEnterMatrix[1], vWPos);
#endif
#endif

  Out.WorldView.w = PI_mW[0].w + 1.0f;
  Out.WorldNormal.w = PI_mW[1].w + 1.0f;
  Out.Tex0.z = PI_mW[2].w + 1.0f;

	Out.HPosition = OPos;

#if defined (RT_FOG)
 #if RT_PROFILE > QUALITY_VERYLOW
	Out.FogColor = Fog(dist);
 #else
	Out.FogColor = Fog(Out.HPosition.w);
 #endif	
#endif

	GEN_SHADOW_COORD
	return Out;
}

VSOUTPUT_LAND VS_Det(in VS_INPUT In)
{
  return _VS(In, true);
}
VSOUTPUT_LAND VS(in VS_INPUT In)
{
  return _VS(In, false);
}

VSOUTPUT_LAND VS_GlobalMap(in VS_INPUT In)
{
	VSOUTPUT_LAND Out = (VSOUTPUT_LAND)0;

  Out.TexLayer1.xy = In.vPos.xz / F_FogParams.xy;
  Out.TexLayer1.y = 1.0f - Out.TexLayer1.y;

	float4 OPos = mul(float4(In.vPos.xyz,1), C_VP);

	half3 vNorm = normalize(In.vNorm.xyz * 2.h - 1.h);
	vNorm = half3(-vNorm.z, vNorm.y, -vNorm.x);

  half dist = length(C_Eye.xyz - In.vPos.xyz);

	Out.WorldNormal.xyz = vNorm;
  Out.WorldView.xyz = C_Eye.xyz - In.vPos.xyz;
#if RT_PROFILE > QUALITY_LOW
  Out.WorldPos.xyz = In.vPos.xyz;
	Out.WorldPos.w = dist;// camera depth in w (for fog)
#else
	Out.Diffuse.a = 1.h;
	#if defined(RT_NUM_LIGHTS)
		half fNdotL = saturate(dot(vNorm, LGetDir(0).xyz));
		Out.Diffuse.rgb = LGetDiffuse(0).rgb * fNdotL; // * TerrainColor.xyz;
	#endif
#endif

#if RT_PROFILE > QUALITY_VERYLOW
#if defined (RT_FOG)
  Out.FogColor = Fog(dist);
#endif
#endif

  Out.WorldView.w = PI_mW[0].w + 1.0f;
  Out.WorldNormal.w = PI_mW[1].w + 1.0f;
  Out.Tex0.z = PI_mW[2].w + 1.0f;

	Out.HPosition = OPos;
	GEN_SHADOW_COORD
	return Out;
}

#define fragInput VSOUTPUT_LAND
#include "fragDef.fxh"

void frag_custom_per_light(inout fragPass pPass, inout fragLightPass pLight)
{
  pLight.fNdotL = saturate( pLight.fNdotL );
  half cK = pLight.fFallOff * pLight.cFilter;

  half3 cDiffuse = min(pLight.fNdotL, pLight.fOcclShadow) * pLight.cDiffuse.xyz;
  pPass.cDiffuseAcc += cDiffuse * cK;
}

#define FRAG_CUSTOM_END

void frag_custom_end(inout fragPass pPass, inout FIXED3 cFinal)
{
	if(pPass.nTerrainIndex == 1)
	{
		// Old Terrain
		half3  mask = tex2D(ColorSampler, pPass.IN.Tex0.xy);
		half3  col1 = tex2D(SecondColorSampler, pPass.IN.TexLayer1.xy).rgb;
		half3  col2 = tex2D(ThirdColorSampler, pPass.IN.TexLayerLayer23.xy).rgb;
		half3  col3 = tex2D(FourthColorSampler, pPass.IN.TexLayerLayer23.zw).rgb;
		pPass.cDiffuseMap.rgb = col1*mask.r + col2*mask.g + col3*mask.b;
	}
	else if(pPass.nTerrainIndex == 2)
	{
		// Old Terrain Single
		pPass.cDiffuseMap.rgb = tex2D(SecondColorSampler,pPass.IN.TexLayer1.xy).rgb;
	}
	else
	{
		// Old Terrain Single Big
		pPass.cDiffuseMap.rgb = tex2D(ColorSampler,pPass.IN.Tex0.xy).rgb;
	}

  half3 cDiffuse = (pPass.cAmbientAcc.xyz + pPass.cDiffuseAcc.xyz) * pPass.cDiffuseMap.xyz;

  cFinal += cDiffuse;
}

#include "fragLib.fxh"

half4 PSOldTerrain(in VSOUTPUT_LAND In):COLOR
{
	// Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, In);
	pPass.nTerrainIndex = 1;
  half4 cFinal = frag_shared_output(pPass);
  //cFinal.xyz = tex2D(ColorSampler, pPass.IN.Tex0.xy); //pPass.cDiffuseMap.xyz;
	return cFinal;
}

half4 PSOldTerrainSingle(in VSOUTPUT_LAND In):COLOR
{
	// Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, In);
	pPass.nTerrainIndex = 2;
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

half4 PSOldTerrainSingleBig(in VSOUTPUT_LAND In):COLOR
{
	// Initialize fragPass structure
  fragPass pPass;
#ifndef OPENGL
  pPass = (fragPass)0;
#endif
  frag_unify(pPass, In);
	pPass.nTerrainIndex = 3;
  half4 cFinal = frag_shared_output(pPass);
	return cFinal;
}

half4 PSDetailedMulSingle(in VSOUTPUT_LAND In):COLOR
{
	half   tileC = max((half)In.TexLayer1.z, saturate((half)In.Tex0.w));
	half3 Color = tex2D(SecondColorSampler, In.TexLayer1.xy).rgb;
	Color = lerp(1.h, Color, tileC);
#if RT_PROFILE > QUALITY_VERYLOW
#if defined(RT_FOG)
	Color = lerp(1.h, Color, In.FogColor.w);
#endif
#endif
  return half4(Color,1);
}

half4 PSDetailedMul(in VSOUTPUT_LAND In):COLOR
{
	half   tileC = max((half)In.TexLayer1.z, saturate(In.Tex0.w));
	half3  mask = tex2D(ColorSampler, In.Tex0.xy);
	//if(GetShaderQuality() > QUALITY_LOW)
		//mask = normalize(mask + .0001h);
	half3  col1 = tex2D(Sampler4, In.TexLayer1.xy).rgb;
	half3  col2 = tex2D(Sampler5, In.TexLayerLayer23.xy).rgb;
	half3  col3 = tex2D(Sampler6, In.TexLayerLayer23.zw).rgb;
	half3 Color = col1*mask.r + col2*mask.g + col3*mask.b;
	Color = lerp(1.h, Color, tileC);
#if RT_PROFILE > QUALITY_VERYLOW
#if defined(RT_FOG)
	Color = lerp(1.h, Color, In.FogColor.w);
#endif
#endif
  return half4(Color,1.h);
}

struct VSOUTPUT_SIMPLE_TERRAIN
{
	float4 HPosition 				: POSITION;
	float4 Tex0 					: TEXCOORD0;
	float4 Tex1						: TEXCOORD1;	
};


VSOUTPUT_SIMPLE_TERRAIN VSTerrainFFPDetailed(in VS_INPUT In)
{	
	VSOUTPUT_SIMPLE_TERRAIN Out = (VSOUTPUT_SIMPLE_TERRAIN)0;
    Out.Tex0.xy =(In.vPos.xz - TerrainPatchOffset.xy) / 32.0f * 33.0f/32.0f;
	Out.HPosition = mul(float4(In.vPos.xyz,1), C_VP); // World always identity for terrain
	Out.Tex0.xy = FC_MaskOffset.x + Out.Tex0.xy * FC_MaskOffset.y;// in FC_MaskOffset.x - 0.5f/32.0f;
    Out.Tex0.zw = In.vPos.xz * FC_MaskOffset.z; // / 32.0f in FC_MaskOffset.z
    Out.Tex1.zw = Out.Tex0.zw * FC_DetailScale.z;// in detailscale *0.5;
	Out.Tex1.xy = Out.Tex0.zw * FC_DetailScale.y;//*0.5;
	Out.Tex0.zw = Out.Tex0.zw * FC_DetailScale.x;//*0.5;

	return Out;    
}


half4 PSTerrainFFPDetailed(in VSOUTPUT_SIMPLE_TERRAIN In):COLOR
{
	half3  mask = tex2D(ColorSampler, In.Tex0.xy);
	half3  col1 = tex2D(SecondColorSampler, In.Tex0.zw).rgb;
	half3  col2 = tex2D(ThirdColorSampler, In.Tex1.xy).rgb;
	half3  col3 = tex2D(FourthColorSampler, In.Tex1.zw).rgb;
	return half4(col1*mask.r + col2*mask.g + col3*mask.b, 1);

}



technique TerrainFFPDetailed
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VSTerrainFFPDetailed();
		PixelShader = compile PS_SHADER_TYPE PSTerrainFFPDetailed();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
}

technique Terrain
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VS();
		PixelShader = compile PS_SHADER_TYPE PSOldTerrain();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
}

technique TerrainDetailed
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VS();
		PixelShader = compile PS_SHADER_TYPE PSOldTerrain();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
#if RT_PROFILE > QUALITY_LOW
	pass P1
	{
		VertexShader = compile VS_SHADER_TYPE VS_Det();
		PixelShader = compile PS_SHADER_TYPE PSDetailedMul();
#ifdef OGL
		BlendEnable = true;
		BlendFunc = { Zero, SrcColor };
#else
		AlphaBlendEnable = true;
		SrcBlend = Zero;
		DestBlend = SrcColor;
#endif
		DEFAULT_STATES
	}
#endif
}

technique TerrainSingle
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VS();
		PixelShader = compile PS_SHADER_TYPE PSOldTerrainSingle();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
}

technique TerrainSingleDetailed
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VS();
		PixelShader = compile PS_SHADER_TYPE PSOldTerrainSingle();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
#if RT_PROFILE > QUALITY_LOW
	pass P1
	{
		VertexShader = compile VS_SHADER_TYPE VS_Det();
		PixelShader = compile PS_SHADER_TYPE PSDetailedMulSingle();
#ifdef OGL
		BlendEnable = true;
		BlendFunc = { Zero, SrcColor };
#else
		AlphaBlendEnable = true;
		SrcBlend = Zero;
		DestBlend = SrcColor;
#endif
		DEFAULT_STATES
	}
#endif
}

technique TerrainSingleBig
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VS_Det();
		PixelShader = compile PS_SHADER_TYPE PSOldTerrainSingleBig();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
}

technique TerrainGlobalMap
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VS_GlobalMap();
		PixelShader = compile PS_SHADER_TYPE PSOldTerrainSingle();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
}

technique TerrainGlobalMapBig
{
	pass P0
	{
		VertexShader = compile VS_SHADER_TYPE VS_GlobalMap();
		PixelShader = compile PS_SHADER_TYPE PSOldTerrainSingleBig();
		AlphaBlendEnable = false;
		DEFAULT_STATES
	}
}
