/*
	ZPrepass
*/

#include "fragDef.fxh"
#include "Transform.fxh"

const float fAlphaTest = 128.f/255.f; //0.49995f;

struct VS_OUT
{
	float4 Pos	: POSITION;
	float4 Tex0	: TEXCOORD0;
#ifdef RT_POINT_SHADOWMAP
  float2 Tex1	: TEXCOORD1;
#endif    
#ifdef RT_DEFERRED
  half3 WorldNormal	  : TEXCOORD2;
 #ifdef TANGENT_BASIS
  half3 WorldTangent	: TEXCOORD3;
  half3 WorldBinormal	: TEXCOORD4;
 #endif
#endif
};
struct PX_OUT
{
	float4 Col0	: COLOR0;
#ifdef RT_DEFERRED
	float4 Col1	: COLOR1;
#endif
};

VS_OUT VSLinearNormalized(in VS_IN In, int nType) // 1: Alpha, 2: Terrain
{
	VS_OUT Out;
  float3 Pw = WorldPosition(In);
  bool bTC = nType == 1;
#if defined (RT_TOON) || defined(RT_DEFERRED) || defined(RT_POINT_SHADOWMAP)
  bTC = true;
#endif
  if (bTC)
  {
    Out.Tex0.zw = GetTexCoords(In.Tex0.xy);
#ifdef RT_POINT_SHADOWMAP
    Out.Tex1.zw = In.Tex1.xy;
#endif    
  }
  else
    Out.Tex0.zw = 0;

  Out.Pos = MUL_VEC_MAT(float4(Pw,1), C_VP);
  Out.Tex0.xy = Out.Pos.zw;
	
#ifdef RT_DEFERRED
 #ifdef TANGENT_BASIS
	Out.WorldTangent  = WorldNormal(In, In.Tangent);
	Out.WorldBinormal = WorldNormal(In, In.Binormal);
 #endif
	Out.WorldNormal   = WorldNormal(In, In.Normal);
	if (nType == 2) // Terrain
	{
    half3 vNorm = normalize(In.Normal.xyz * 2.h - 1.h);
	  vNorm = half3(-vNorm.z, vNorm.y, -vNorm.x);
	  Out.WorldNormal = vNorm;
	}
	
 #ifdef RT_SKINNING
  float3 vWorldNormal = In.Normal;
   float3 vWorldPos = (float3)0;
  int4 IndexVector = D3DCOLORtoUBYTE4(In.BoneID);
  // cast the vectors to arrays for use in the for loop below
  float4 BlendWeightsArray = D3DCOLORtoUBYTE4(In.Weight);///255.0f;
  #if defined(TANGENT_BASIS)	
   float3 vWorldTangent = In.Tangent;
   float3 vWorldBinormal = In.Binormal;
   __SkinningPositionTangent(IndexVector, BlendWeightsArray, vWorldPos, vWorldTangent, vWorldBinormal, vWorldNormal);
	Out.WorldTangent  = vWorldTangent;
	Out.WorldBinormal = vWorldBinormal;
  #else
   __SkinningPositionNormal(IndexVector, BlendWeightsArray, vWorldPos, vWorldNormal);
  #endif
	Out.WorldNormal = vWorldNormal;
 #endif
#endif
	
	return Out;
}

PX_OUT PSLinearNormalized(in VS_OUT In, int nType)
{
  PX_OUT px;

	half alpha = 1.h;
	if (nType == 1)
	{
	  half2 baseTC = In.Tex0.zw;
#ifdef RT_POINT_SHADOWMAP
    baseTC = In.Tex1.zw + frac(In.Tex0.zw) * relSize;
#endif	
		alpha = tex2D(ColorSampler, baseTC.xy).a;
		clip(alpha - fAlphaTest);
	}
	px.Col0 = 0;
#ifdef RT_TOON
  px.Col0.b = tex2D(OpacitySampler, In.Tex0.zw).a;
  px.Col0.g = Pack(toon_EdgeColor.xyz);
  px.Col0.a = toon_EdgeWidth;
  px.Col0.r = In.Tex0.z;

  //float3 EColor = float3(1.0, 0.0, 0.0);
  //px.Col0.g = Pack(EColor.xyz);
#else  
#ifdef OPENGL
  float fDepth = (In.Tex0.x / In.Tex0.y + 1.0f) * 0.5f;
#else
  float fDepth = In.Tex0.x * C_FarClip.y;
#endif
  px.Col0 = fDepth;
#endif

#ifdef RT_DEFERRED
 half fGlossMap = 1.0f;
 #if defined(RT_SPECULAR) && defined(GLOSS_IN_DIFFUSEALPHA)
  //fGlossMap = tex2D(ColorSampler, In.Tex0.xy).a;
 #endif
 #ifdef TANGENT_BASIS
  half4 cBumpData = tex2D(SpecularSampler, In.Tex0.xy);
  half3 cBumpMap = cBumpData.rgb * 2.h - 1.h;
  #if RT_PROFILE > QUALITY_HIGH
    cBumpMap = normalize(cBumpMap);
  #endif
  half3 mTangentToWS0 = In.WorldTangent.xyz;  
  half3 mTangentToWS1 = In.WorldBinormal.xyz;  
  half3 mTangentToWS2 = In.WorldNormal.xyz;  
  half3 vNormal = cBumpMap.x * mTangentToWS0;
  vNormal.xyz += cBumpMap.y * mTangentToWS1;
  vNormal.xyz += cBumpMap.z * mTangentToWS2;
  px.Col1 = half4(vNormal.xyz, 1);
 #else
  px.Col1 = half4(In.WorldNormal.xyz, 1);
 #endif
  px.Col1 = half4(In.WorldNormal.xyz, 1);
  px.Col1.xyz = (px.Col1.xyz + 1.0f) * 0.5f;
  px.Col1.w = (Glossiness / 255.0f) * fGlossMap;
#endif

	return px;
}

VS_OUT VSLinear(in VS_IN In) // 1: Alpha, 2: Terrain
{
  return VSLinearNormalized(In, 0); // 1: Alpha, 2: Terrain
}
VS_OUT VSLinear_AT(in VS_IN In) // 1: Alpha, 2: Terrain
{
  return VSLinearNormalized(In, 1); // 1: Alpha, 2: Terrain
}
VS_OUT VSLinear_Ter(in VS_IN In) // 1: Alpha, 2: Terrain
{
  return VSLinearNormalized(In, 2); // 1: Alpha, 2: Terrain
}

PX_OUT PSLinear(in VS_OUT In)
{
  return PSLinearNormalized(In, 0); // 1: Alpha, 2: Terrain
}
PX_OUT PSLinear_AT(in VS_OUT In)
{
  return PSLinearNormalized(In, 1); // 1: Alpha, 2: Terrain
}
PX_OUT PSLinear_Ter(in VS_OUT In)
{
  return PSLinearNormalized(In, 2); // 1: Alpha, 2: Terrain
}

technique GenerateLinearNormalized
{
  pass p0 
  {
		VertexShader = compile VS_SHADER_TYPE VSLinear();
		PixelShader  = compile PS_SHADER_TYPE PSLinear();
  }
}

technique GenerateLinearNormalized_Terrain
{
  pass p0 
  {
		VertexShader = compile VS_SHADER_TYPE VSLinear_Ter();
		PixelShader  = compile PS_SHADER_TYPE PSLinear_Ter();
  }
}

technique GenerateLinearNormalizedAlpha
{
	pass p0 
	{
		VertexShader = compile VS_SHADER_TYPE VSLinear_AT();
		PixelShader  = compile PS_SHADER_TYPE PSLinear_AT();
	}
}
